/****************************************************************************
                             Hyper's CD Catalog
		A multiplatform qt and xml based catalog program

 Author    : Peter Deak (hyperr@freemail.hu)
 License   : GPL
 Copyright : (C) 2003 Peter Deak
****************************************************************************/

#include "icons.h"
#include <qpixmap.h>

static const char* const m_catalog_icon_data[] = {
    "16 16 111 2",
    "  	c None",
    ". 	c #DC0202",
    "+ 	c #FF0A0A",
    "@ 	c #FF1010",
    "# 	c #FF1A1A",
    "$ 	c #FE2424",
    "% 	c #FF0808",
    "& 	c #FE0303",
    "* 	c #F50101",
    "= 	c #DF0000",
    "- 	c #BE0000",
    "; 	c #FC0505",
    "> 	c #FF0909",
    ", 	c #FF1D1D",
    "' 	c #FF2D2D",
    ") 	c #FF3B3B",
    "! 	c #FF2B2B",
    "~ 	c #FF1818",
    "{ 	c #FE0A0A",
    "] 	c #EE0303",
    "^ 	c #D40101",
    "/ 	c #A40000",
    "( 	c #FF0303",
    "_ 	c #FF0707",
    ": 	c #FF0F0F",
    "< 	c #FF3333",
    "[ 	c #FF4A4A",
    "} 	c #FF4E4E",
    "| 	c #FF3939",
    "1 	c #F30B0B",
    "2 	c #D10303",
    "3 	c #B50101",
    "4 	c #9F0000",
    "5 	c #8B0000",
    "6 	c #FF0202",
    "7 	c #FF0505",
    "8 	c #FF0B0B",
    "9 	c #FF2F2F",
    "0 	c #FF4B4B",
    "a 	c #FF5555",
    "b 	c #FF3E3E",
    "c 	c #FC1B1B",
    "d 	c #D80606",
    "e 	c #AE0000",
    "f 	c #940000",
    "g 	c #800000",
    "h 	c #6C0000",
    "i 	c #630000",
    "j 	c #FF0101",
    "k 	c #FF2020",
    "l 	c #FF3636",
    "m 	c #FF4040",
    "n 	c #FE2E2E",
    "o 	c #EF3B3C",
    "p 	c #D56F73",
    "q 	c #C07278",
    "r 	c #994043",
    "s 	c #660909",
    "t 	c #500000",
    "u 	c #FD0303",
    "v 	c #FB0707",
    "w 	c #F90F0F",
    "x 	c #F91A1A",
    "y 	c #F51D1C",
    "z 	c #EA4043",
    "A 	c #E9C9D7",
    "B 	c #EAF1FF",
    "C 	c #E8EEFF",
    "D 	c #E8E5F3",
    "E 	c #AD919B",
    "F 	c #711316",
    "G 	c #EE0101",
    "H 	c #D00505",
    "I 	c #C70808",
    "J 	c #B70707",
    "K 	c #C67C86",
    "L 	c #E1E9FF",
    "M 	c #D8DAFE",
    "N 	c #CACAFE",
    "O 	c #D9DAFF",
    "P 	c #DBDFFE",
    "Q 	c #8C7797",
    "R 	c #BA0000",
    "S 	c #A90101",
    "T 	c #930101",
    "U 	c #B18590",
    "V 	c #DFEBFF",
    "W 	c #C4C6FD",
    "X 	c #9694CC",
    "Y 	c #A8A7E2",
    "Z 	c #C0CDFF",
    "` 	c #A9BEEA",
    " .	c #7D0000",
    "..	c #5A0000",
    "+.	c #7F5057",
    "@.	c #D7DFF9",
    "#.	c #CACFFE",
    "$.	c #A5A3DD",
    "%.	c #A8A8E6",
    "&.	c #BAC9FF",
    "*.	c #B7CCFA",
    "=.	c #510000",
    "-.	c #670C0E",
    ";.	c #B194AB",
    ">.	c #E8ECFF",
    ",.	c #D7DAFF",
    "'.	c #C1CCFF",
    ").	c #C3D5FF",
    "!.	c #8E9098",
    "~.	c #C5CAE9",
    "{.	c #C5D4FA",
    "                                ",
    "                                ",
    "    . . . .         . . . .     ",
    "  . + @ # $ .     . % & * = -   ",
    ". ; > @ , ' ) ) ! ~ { ] ^ - / / ",
    ". ( _ : , < [ } | , 1 2 3 4 5 5 ",
    ". 6 7 8 ~ 9 0 a b c d e f g h i ",
    ". j ( _ : k l m n o p q r s t h ",
    "  . j u v w x y z A B C D E F   ",
    "    . G . H I J K L M N O P Q   ",
    "      . R S T g U V W X Y Z `   ",
    "        . S  ...+.@.#.$.%.&.*.  ",
    "          . ..=.-.;.>.,.'.).`   ",
    "            . =..   !.~.{.`     ",
    "              .                 ",
    "                                "
};

static const char* const m_unknown_icon_data[] = {
    "16 18 38 1",
    ". c None",
    "y c #000000",
    "d c #000000",
    "x c #000000",
    "H c #000000",
    "c c #000000",
    "G c #000000",
    "t c #000000",
    "n c #000000",
    "s c #000000",
    "w c #000000",
    "J c #000000",
    "D c #000000",
    "B c #000000",
    "v c #000000",
    "r c #000000",
    "C c #000000",
    "l c #000000",
    "k c #000000",
    "I c #000000",
    "j c #000000",
    "q c #000000",
    "b c #000000",
    "a c #00385a",
    "# c #003862",
    "h c #00406a",
    "o c #004473",
    "g c #00508b",
    "f c #005594",
    "F c #00599c",
    "e c #2981c5",
    "u c #2985c5",
    "m c #3185c5",
    "z c #4191cd",
    "p c #5299cd",
    "i c #6aaad5",
    "A c #6aaade",
    "E c #8bbee6",
    "....#a#a#abcd...",
    "...aefbbbfgbbcd.",
    "..hifbjklamgfbnd",
    "..opbqrstsaugbvc",
    "..#efbwx.yhzfblt",
    "..cagfbnd.hzgblt",
    "..ynbbbvcaAfbqvc",
    "...ynBCDaEFbqrnd",
    "....ycGaAFbqrnd.",
    "......augbqrnd..",
    "......#efbksd...",
    "......cbbqvc....",
    "......ynBBnd....",
    ".......aasx.....",
    "......#ifbnd....",
    "......HbbIJH....",
    "......ynBBnd....",
    ".......yccd....."
};

static const char* const m_cd_icon_data[] = {
    "18 16 172 2",
    "  	c None",
    ". 	c #80807E",
    "+ 	c #8B8B83",
    "@ 	c #757567",
    "# 	c #413C31",
    "$ 	c #514D47",
    "% 	c #363434",
    "& 	c #737373",
    "* 	c #90A189",
    "= 	c #A4C781",
    "- 	c #CBE781",
    "; 	c #E8EB91",
    "> 	c #DAC99C",
    ", 	c #C19F8D",
    "' 	c #976A65",
    ") 	c #523E40",
    "! 	c #131313",
    "~ 	c #777B77",
    "{ 	c #71B76F",
    "] 	c #4BEF35",
    "^ 	c #7EF63C",
    "/ 	c #CAFF65",
    "( 	c #EBF189",
    "_ 	c #D3C4A5",
    ": 	c #D0AFA0",
    "< 	c #C69090",
    "[ 	c #BB778A",
    "} 	c #856974",
    "| 	c #3E3C3E",
    "1 	c #93B682",
    "2 	c #55FF39",
    "3 	c #06F800",
    "4 	c #48FE1E",
    "5 	c #B4FF5B",
    "6 	c #F0FA80",
    "7 	c #DDC89F",
    "8 	c #CFA89D",
    "9 	c #C18991",
    "0 	c #BC849C",
    "a 	c #C189AF",
    "b 	c #987E96",
    "c 	c #1A1A1A",
    "d 	c #929684",
    "e 	c #B8DC92",
    "f 	c #97F062",
    "g 	c #4BFF2D",
    "h 	c #23FC18",
    "i 	c #96FD57",
    "j 	c #E8F591",
    "k 	c #E6CEA0",
    "l 	c #C9A1A1",
    "m 	c #BC8699",
    "n 	c #C18BAE",
    "o 	c #C397C2",
    "p 	c #BE99C3",
    "q 	c #544D57",
    "r 	c #7A7A7A",
    "s 	c #B5B69A",
    "t 	c #C1C997",
    "u 	c #BBD292",
    "v 	c #A6EC72",
    "w 	c #89EC76",
    "x 	c #BC9CB1",
    "y 	c #C29FC7",
    "z 	c #B9A5CF",
    "A 	c #B3A8D4",
    "B 	c #7E7599",
    "C 	c #373739",
    "D 	c #878783",
    "E 	c #C4BCA4",
    "F 	c #C8C49C",
    "G 	c #CACB9D",
    "H 	c #C9CEA2",
    "I 	c #616961",
    "J 	c #B1AED4",
    "K 	c #AAAAD6",
    "L 	c #A6A9D8",
    "M 	c #9597C1",
    "N 	c #4B4B53",
    "O 	c #908A86",
    "P 	c #D4BEA4",
    "Q 	c #D1BDA1",
    "R 	c #D2BEA2",
    "S 	c #D0C4B2",
    "T 	c #A3B2D9",
    "U 	c #9BA6D3",
    "V 	c #9AA4D0",
    "W 	c #909DD6",
    "X 	c #2D303F",
    "Y 	c #918583",
    "Z 	c #DBAFA3",
    "` 	c #D9AFA3",
    " .	c #DBACA3",
    "..	c #D7B1B1",
    "+.	c #98B2D0",
    "@.	c #899FC1",
    "#.	c #899CBF",
    "$.	c #869AC4",
    "%.	c #2A2F3A",
    "&.	c #7E7476",
    "*.	c #BF8B90",
    "=.	c #D99DA8",
    "-.	c #DAA0B2",
    ";.	c #D6ACBE",
    ">.	c #D5A7CF",
    ",.	c #4AFC85",
    "'.	c #3DFFBD",
    ").	c #59EDF3",
    "!.	c #76BBCA",
    "~.	c #7BA8BB",
    "{.	c #6C90AA",
    "].	c #444B50",
    "^.	c #756F71",
    "/.	c #B58396",
    "(.	c #D79DBA",
    "_.	c #D7A3C3",
    ":.	c #B7A3CD",
    "<.	c #9AB8D6",
    "[.	c #47FDE1",
    "}.	c #68D6D5",
    "|.	c #4E898E",
    "1.	c #353739",
    "2.	c #8D7D86",
    "3.	c #D09EC5",
    "4.	c #D1A3CE",
    "5.	c #C3A2CE",
    "6.	c #A7A4CA",
    "7.	c #7CDDE8",
    "8.	c #39FFA9",
    "9.	c #07FF47",
    "0.	c #15FF68",
    "a.	c #2DFFA9",
    "b.	c #33FBC5",
    "c.	c #334F4E",
    "d.	c #A78AAA",
    "e.	c #C699D1",
    "f.	c #BBA0CA",
    "g.	c #ADA0CA",
    "h.	c #A0A2CE",
    "i.	c #9AB5D4",
    "j.	c #8DC8D3",
    "k.	c #3FFFC7",
    "l.	c #0DFF6E",
    "m.	c #00F42E",
    "n.	c #05FF57",
    "o.	c #42946C",
    "p.	c #202020",
    "q.	c #767476",
    "r.	c #9A82A8",
    "s.	c #AE91D3",
    "t.	c #A9A0CE",
    "u.	c #A1A8CF",
    "v.	c #99B9D1",
    "w.	c #8DC7D3",
    "x.	c #5DE9CD",
    "y.	c #1BFF96",
    "z.	c #11D744",
    "A.	c #2E7C3C",
    "B.	c #1D2520",
    "C.	c #847F8F",
    "D.	c #8886B2",
    "E.	c #8E97BA",
    "F.	c #88B4DA",
    "G.	c #7EC8DE",
    "H.	c #52D2C8",
    "I.	c #309E76",
    "J.	c #244632",
    "K.	c #74787E",
    "L.	c #758795",
    "M.	c #42585E",
    "N.	c #364848",
    "O.	c #282E2C",
    "          . + @ # $ %               ",
    "      & * = - ; > , ' ) !           ",
    "    ~ { ] ^ / ( _ : < [ } |         ",
    "  & 1 2 3 4 5 6 7 8 9 0 a b c       ",
    "  d e f g h i j k l m n o p q       ",
    "r s t u v w h i k m x y z A B C     ",
    "D E F G H v w I I x y J K L M N     ",
    "O P Q R S H I     I J T U V W X     ",
    "Y Z `  ...S I     I +.+.@.#.$.%.    ",
    "&.*.=.-.;.>.;.I I ,.'.).!.~.{.].    ",
    "^./.(._.>.>.:.:.<.,.,.'.[.}.|.1.    ",
    "  2.3.4.5.:.6.<.7.8.9.0.a.b.c.      ",
    "  & d.e.f.g.h.i.j.k.l.m.n.o.p.      ",
    "    q.r.s.t.u.v.w.x.y.z.A.B.        ",
    "      & C.D.E.F.G.H.I.J.c           ",
    "          & K.L.M.N.O.              "
};


static const char* const m_dvd_icon_data[] = {
    "16 16 81 1",
    " 	c None",
    ".	c #A9A9A9",
    "+	c #B0B0B0",
    "@	c #9C9C9C",
    "#	c #6F6F6F",
    "$	c #7F7F7F",
    "%	c #6C6C6C",
    "&	c #9F9F9F",
    "*	c #BCBCBC",
    "=	c #C8C8C8",
    "-	c #D5D5D5",
    ";	c #DEDEDE",
    ">	c #DBDBDB",
    ",	c #CBCBCB",
    "'	c #A8A8A8",
    ")	c #7B7B7B",
    "!	c #505050",
    "~	c #A5A5A5",
    "{	c #BABABA",
    "]	c #B9B9B9",
    "^	c #BFBFBF",
    "/	c #D4D4D4",
    "(	c #DDDDDD",
    "_	c #DCDCDC",
    ":	c #D9D9D9",
    "<	c #CECECE",
    "[	c #A2A2A2",
    "}	c #727272",
    "|	c #C2C2C2",
    "1	c #C1C1C1",
    "2	c #A7A7A7",
    "3	c #B6B6B6",
    "4	c #CFCFCF",
    "5	c #D7D7D7",
    "6	c #C9C9C9",
    "7	c #C5C5C5",
    "8	c #B4B4B4",
    "9	c #555555",
    "0	c #B5B5B5",
    "a	c #D8D8D8",
    "b	c #CCCCCC",
    "c	c #BDBDBD",
    "d	c #B2B2B2",
    "e	c #CDCDCD",
    "f	c #E2E2E2",
    "g	c #D6D6D6",
    "h	c #C6C6C6",
    "i	c #CACACA",
    "j	c #D0D0D0",
    "k	c #848484",
    "l	c #D2D2D2",
    "m	c #D3D3D3",
    "n	c #AEAEAE",
    "o	c #949494",
    "p	c #E0E0E0",
    "q	c #DFDFDF",
    "r	c #818181",
    "s	c #DADADA",
    "t	c #E1E1E1",
    "u	c #B3B3B3",
    "v	c #E3E3E3",
    "w	c #696969",
    "x	c #C7C7C7",
    "y	c #C3C3C3",
    "z	c #C0C0C0",
    "A	c #7E7E7E",
    "B	c #C4C4C4",
    "C	c #9B9B9B",
    "D	c #6D6D6D",
    "E	c #000000",
    "F	c #ADADAD",
    "G	c #767676",
    "H	c #AFAFAF",
    "I	c #ACACAC",
    "J	c #5A5A5A",
    "K	c #A1A1A1",
    "L	c #5B5B5B",
    "M	c #969696",
    "N	c #828282",
    "O	c #747474",
    "P	c #636363",
    "     .+@#$%     ",
    "   &*=-;>,')!   ",
    "  ~{]^/(_:<^[}  ",
    " &|1234(;567689 ",
    " 0abcdeffghi4jk ",
    "~bl/lmdefh4->;+#",
    "n-/-:lmoo4-pq;<r",
    "8_s>t:o  op;ag-%",
    "u;;;vto  o--6=6w",
    "~6>(t;tooxyi7z8A",
    "&1>(;;aaaxxyhBCD",
    " naEEaaE/EFEEcG ",
    " &zEgEaElEHEIEJ ",
    "  KEEaagEx0EEL  ",
    "   &+|=mj]M%9   ",
    "     &~nNOP     "
};

static const char* const m_hdd_icon_data[] = {
    "16 16 105 2",
    "Qt c None",
    ".1 c None",
    "#j c None",
    "#K c None",
    "#A c None",
    "#s c None",
    ".K c None",
    ".y c None",
    ".n c None",
    "#G c None",
    ".h c None",
    "#t c None",
    "#k c None",
    ".# c None",
    ".c c None",
    "## c None",
    ".i c None",
    ".o c None",
    "#M c #000020",
    "#L c #62657b",
    "#z c #080439",
    "#r c #000031",
    ".z c #f6f2ff",
    "#F c #181841",
    "#J c #39385a",
    ".p c #cdced5",
    ".j c #cdcad5",
    ".T c #ded6e6",
    ".2 c #d5d2d5",
    ".L c #dedae6",
    ".b c #b4b6c5",
    ".d c #c5c2cd",
    "#H c #9495a4",
    ".g c #acaab4",
    "#q c #62657b",
    "#B c #9495a4",
    ".m c #9c9dac",
    "#y c #737583",
    "#u c #a4a1ac",
    "#E c #8b899c",
    ".x c #a4a1ac",
    ".a c #ded6e6",
    "#l c #b4b2bd",
    "#I c #94919c",
    ".J c #a4a5b4",
    ".A c #dedade",
    "#a c #cdc2d5",
    ".S c #9c99ac",
    ".q c #e6e2ee",
    "#v c #5a5d83",
    ".4 c #5aea62",
    "#C c #62658b",
    "#f c #7b799c",
    "#w c #837da4",
    "#m c #83819c",
    "#e c #8381a4",
    ".9 c #8b89a4",
    "#. c #8b8dac",
    ".8 c #948dac",
    ".7 c #9495ac",
    ".Z c #9c99ac",
    "#n c #9c99b4",
    ".W c #9c9db4",
    "#i c #a49dac",
    ".Y c #a4a1b4",
    ".X c #a4a1bd",
    "#x c #a4a5bd",
    ".Q c #acaabd",
    "#g c #acaac5",
    ".O c #acaec5",
    ".P c #b4b2c5",
    "#D c #b4b6c5",
    ".N c #bdb6cd",
    ".R c #bdbacd",
    ".G c #c5becd",
    ".V c #c5bed5",
    ".6 c #c5c2cd",
    ".H c #c5c2d5",
    ".F c #cdc6d5",
    "#h c #cdcade",
    "#p c #cdced5",
    "#c c #d5cede",
    ".v c #d5d2de",
    "#d c #d5d2e6",
    ".M c #d5d6de",
    "#b c #d5e2de",
    ".u c #ded6e6",
    ".I c #dedae6",
    "#o c #dedaee",
    ".3 c #deffe6",
    ".0 c #e6dee6",
    ".C c #e6e2ee",
    ".E c #e6e6ee",
    ".D c #eee6ee",
    ".B c #eee6f6",
    ".k c #eeeaee",
    ".w c #eeeaf6",
    ".U c #eeeeee",
    ".5 c #f6eaff",
    ".s c #f6f2f6",
    ".e c #f6f2ff",
    ".l c #f6f6ff",
    ".t c #fff6ff",
    ".r c #fffaff",
    ".f c #ffffff",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQt.#.a.bQtQtQtQtQtQt",
    "QtQtQtQtQt.c.d.e.f.f.g.hQtQtQtQt",
    "QtQtQt.i.j.k.f.f.f.l.f.m.nQtQtQt",
    "Qt.o.p.q.f.r.s.l.t.u.v.w.x.yQtQt",
    ".z.A.e.k.B.C.D.E.F.G.H.H.I.J.KQt",
    ".L.f.v.v.v.v.M.N.O.P.P.P.Q.R.SQt",
    ".T.U.f.F.V.H.N.W.X.Y.W.Z.V.f.0.1",
    ".2.3.4.5.6.O.7.8.9#..P.C.k.f.M.1",
    "###a#b#c#d.H#e#f#g.H.R#h.D.f#i#j",
    "Qt#k#l.u.P.P.H.X#m#n.O#o#p#q#r#s",
    "QtQt#t#u#h.7.N#v#w#x.F#y#z#AQtQt",
    "QtQtQt.h#B.V.Q#C#D#E#F#AQtQtQtQt",
    "QtQtQtQt#G#H.R#I#J#KQtQtQtQtQtQt",
    "QtQtQtQtQtQt#L#MQtQtQtQtQtQtQtQt"
};

static const char* const m_net_icon_data[] = {
    "16 16 108 2",
    "Qt c None",
    "#L c None",
    ".Y c None",
    ".Q c None",
    "#F c None",
    "#u c None",
    "#f c None",
    "#M c None",
    "#p c None",
    "#G c None",
    ".f c None",
    ".v c None",
    ".G c None",
    ".p c None",
    ".b c None",
    ".Z c None",
    "#B c #52505a",
    ".8 c #ffffff",
    ".X c #000029",
    "#E c #000029",
    ".u c #626173",
    ".# c #c5c2c5",
    ".h c #f6eeee",
    ".F c #181431",
    ".P c #000029",
    ".q c #ffffff",
    "#C c #9c99a4",
    ".g c #ffffff",
    "#t c #080c29",
    "#v c #ee9100",
    ".a c #8b8d8b",
    "#y c #945000",
    ".l c #f69d00",
    "#g c #dedade",
    ".o c #c57500",
    ".O c #525552",
    ".9 c #7b7d7b",
    "#P c #a4a59c",
    ".H c #c58508",
    ".w c #4a4c52",
    ".R c #ffd600",
    "#s c #736d6a",
    ".i c #7b757b",
    "#J c #000000",
    ".C c #000c18",
    ".s c #080808",
    ".1 c #101420",
    "#r c #313031",
    "#H c #4a484a",
    "#z c #52595a",
    "#n c #5a6173",
    "#. c #626162",
    ".e c #626562",
    ".k c #626973",
    ".r c #6a656a",
    "#e c #6a696a",
    "#j c #73694a",
    "#h c #736d73",
    "#N c #73757b",
    ".x c #838594",
    "## c #83899c",
    ".0 c #8b7d62",
    ".D c #9c999c",
    ".j c #a4a1a4",
    ".t c #aca5ac",
    "#A c #acaaac",
    "#q c #acaeac",
    ".E c #b4b2ac",
    ".c c #b4b6bd",
    "#c c #b4bede",
    ".M c #bd8952",
    ".6 c #bda15a",
    "#I c #bdb6bd",
    ".2 c #c5bab4",
    "#D c #c5c2c5",
    "#o c #c5c6cd",
    ".N c #c5cad5",
    ".5 c #cdb273",
    ".B c #d59108",
    "#K c #d5d2cd",
    "#d c #d5deee",
    "#i c #dedade",
    "#O c #dedede",
    ".W c #e69100",
    ".7 c #ee8d00",
    "#b c #eeba39",
    "#x c #f69d00",
    "#w c #ffa500",
    "#k c #ffbe20",
    ".y c #ffc218",
    ".n c #ffc220",
    "#m c #ffca08",
    "#l c #ffca10",
    ".m c #ffca29",
    ".V c #ffce18",
    ".L c #ffd218",
    ".K c #ffd24a",
    ".U c #ffd652",
    ".4 c #ffe26a",
    ".T c #ffe694",
    "#a c #ffee94",
    ".3 c #fff6c5",
    ".J c #fffade",
    ".S c #fffaf6",
    ".I c #ffffcd",
    ".z c #ffffd5",
    ".A c #fffff6",
    ".d c #ffffff",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQt.#.a.bQtQtQtQtQtQtQtQtQtQtQt",
    "QtQt.c.d.e.fQtQtQtQtQtQtQt.g.hQt",
    "QtQt.i.j.d.k.l.m.n.o.p.q.r.s.t.u",
    "QtQt.v.w.x.y.z.d.d.A.B.C.D.d.E.F",
    "QtQtQt.G.H.I.d.d.J.K.L.M.N.O.P.Q",
    "QtQtQtQt.R.d.d.S.T.U.V.W.X.YQtQt",
    "QtQtQt.Z.0.1.2.3.4.5.6.7.QQtQtQt",
    "Qt.8.9.s#..d###a#b#c#d#e#fQtQtQt",
    "#g#.#h.d#i#j#k#l#m#n#o.d#e#pQtQt",
    "#q#r.d#s#t#u#v#w#x#y#z#A.d.D#BQt",
    "#C#D#E.YQtQtQt#F#F.Q#G#H#I#J#K#L",
    "Qt.QQtQtQtQtQtQtQtQtQt#M#N#O#P.Y",
    "QtQtQtQtQtQtQtQtQtQtQtQtQt.Y.YQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt"
};

static const char* const m_flash_icon_data[] = {
    "16 16 97 2",
    ".e c #2048a4",
    ".j c #2050a4",
    ".f c #3155ac",
    ".q c #3165ac",
    ".d c #395dac",
    ".n c #3961ac",
    ".A c #3969b4",
    ".# c #415dac",
    "Qt c #4161b4",
    ".a c #4165b4",
    ".s c #4169b4",
    ".U c #4179bd",
    ".b c #4a65b4",
    ".c c #4a69b4",
    ".h c #4a71b4",
    ".o c #4a71bd",
    ".x c #4a79bd",
    ".R c #4a79c5",
    ".J c #4a7dbd",
    ".g c #5271bd",
    ".i c #5275bd",
    ".X c #5289c5",
    "#. c #5291c5",
    ".k c #5a79bd",
    ".p c #5a7dbd",
    ".t c #5a7dc5",
    ".D c #5a85c5",
    ".W c #5a8dc5",
    ".V c #5a8dcd",
    ".v c #6289c5",
    ".F c #628dc5",
    ".O c #628dcd",
    ".K c #6291c5",
    ".H c #6291cd",
    ".N c #6295cd",
    ".4 c #6299cd",
    ".E c #6a89c5",
    ".G c #6a89cd",
    ".w c #6a8dc5",
    ".u c #6a91cd",
    ".I c #6a95c5",
    ".M c #6a99cd",
    ".7 c #6a9dcd",
    ".6 c #6aa1cd",
    ".L c #7399cd",
    "#a c #73a1cd",
    ".5 c #73a1d5",
    "#p c #73a5d5",
    ".Y c #7ba1d5",
    "#b c #7baad5",
    "#j c #7baed5",
    "#h c #7baede",
    ".C c #83a1d5",
    ".8 c #83aed5",
    "#d c #83b2d5",
    "#i c #83b2de",
    "#o c #83b6de",
    "#s c #83bade",
    "#c c #8bb2de",
    "#f c #8bb6de",
    "#u c #8bb6e6",
    "#k c #8bbade",
    "#l c #8bbede",
    "#B c #8bc2e6",
    ".S c #94b2de",
    "#e c #94b6e6",
    "#g c #94bae6",
    "#n c #94bede",
    "#m c #94bee6",
    "#E c #94c2e6",
    ".P c #9cbade",
    "#t c #9cc2e6",
    "#v c #9cc6e6",
    "#x c #9ccae6",
    "#C c #9ccaee",
    ".2 c #a4c2de",
    "#y c #a4c6ee",
    "#w c #a4cae6",
    "#D c #a4caee",
    "#z c #a4ceee",
    ".0 c #acc6e6",
    ".1 c #accae6",
    ".m c #b4c2e6",
    ".Q c #b4ceee",
    "#r c #b4d2ee",
    ".B c #bdcae6",
    ".3 c #bdd2ee",
    "#q c #c5daee",
    ".T c #d5def6",
    ".Z c #d5e6f6",
    "## c #dee6f6",
    ".y c #e6eaff",
    "#A c #e6f2ff",
    ".r c #eeeef6",
    ".9 c #eeeeff",
    ".z c #fffaff",
    ".l c #ffffff",
    "Qt.#Qt.#Qt.#Qt.#Qt.#Qt.#Qt.#Qt.#",
    ".a.b.a.c.a.b.d.e.f.c.a.c.a.b.a.c",
    ".g.h.i.h.g.j.k.l.m.n.i.o.g.h.i.o",
    ".k.p.p.k.q.l.l.l.r.s.p.p.p.p.k.t",
    ".u.v.w.x.y.z.A.B.C.D.E.F.G.F.E.F",
    ".F.H.I.H.l.J.K.F.K.L.M.L.N.O.I.L",
    ".P.Q.R.S.T.U.O.F.V.W.O.X.Y.Z.V.u",
    ".l.l.l.l.l.l.l.l.l.l.l.l.l.l.l.0",
    ".l.l.l.l.l.l.l.l.l.l.l.l.l.l.l.1",
    ".2.3.4.5.6.7.8.9#..5.6.6.8###a#b",
    "#c#d#e#f#g#f#h.l#h#i#i#i#f#j#k#k",
    "#l#m#n#m#n#m#o.Z.l#p#q#r#s#m#n#g",
    "#t#t#t#t#t#t#t#i.z.l.l.l#u#t#t#t",
    "#v#w#t#x#v#y#v#v#s#z.l#A#B#C#t#x",
    "#D#D#z#w#z#w#z#w#D#v#E#E#D#w#z#w",
    "#w#z#w#z#w#z#w#z#w#z#w#z#w#z#w#z"
};

static const char* const m_other_icon_data[] = {
    "16 16 103 2",
    "Qt c None",
    "#p c None",
    "#j c None",
    "#K c None",
    ".n c None",
    ".Z c None",
    ".O c None",
    ".c c None",
    ".# c #d5cef6",
    "#k c #d5d6ff",
    "#H c #9c99cd",
    "#a c #d5d2f6",
    ".F c #c5baee",
    ".h c #9491cd",
    "#B c #9c9dde",
    "#v c #7375c5",
    ".N c #bdb6e6",
    "#C c #c5beee",
    ".Y c #bdaee6",
    "#w c #d5cef6",
    "#G c #9491cd",
    ".y c #6a69b4",
    "#J c #7b79ac",
    ".d c #eeeaff",
    ".b c #c5beee",
    ".s c #ffffff",
    "#q c #e6e2ff",
    "## c #aca5de",
    ".i c #fffaff",
    ".v c #f6f6ff",
    ".I c #8b89bd",
    ".Q c #9c95c5",
    ".u c #acb2f6",
    ".z c #fffaff",
    ".D c #948db4",
    ".a c #c5c2de",
    ".e c #eeeaff",
    ".0 c #9c9dbd",
    "#I c #c5c6de",
    "#u c #cdc6f6",
    ".1 c #eee6f6",
    ".C c #acaee6",
    ".j c #e6e6f6",
    ".G c #f6f6ff",
    "#A c #cdc6ee",
    ".R c #ffeef6",
    ".H c #c5c6ee",
    "#F c #d5cef6",
    ".P c #dedef6",
    ".J c #ffffff",
    ".E c #fffaff",
    ".9 c #0034d5",
    "#h c #0038cd",
    "#g c #003cd5",
    ".8 c #185dd5",
    ".W c #187df6",
    "#n c #2079ee",
    ".V c #29a1ff",
    "#f c #3981ee",
    "#m c #4a9df6",
    ".7 c #4aa5ff",
    ".L c #4aaeff",
    ".6 c #5abaff",
    ".U c #62b6ff",
    "#e c #73c6ff",
    "#l c #7bcaff",
    "#s c #83caff",
    "#d c #83ceff",
    ".T c #83d2ff",
    "#o c #8bbeff",
    ".5 c #8bceff",
    "#i c #94beff",
    ".K c #a4deff",
    ".4 c #ace2ff",
    "#t c #b4daff",
    "#c c #b4e6ff",
    ".r c #bdc6ff",
    ".x c #c5caff",
    "#. c #c5e2ff",
    ".m c #cdceff",
    ".f c #cdd2ee",
    ".g c #cdd2ff",
    "#z c #d5d2ff",
    "#E c #d5d6ff",
    ".3 c #d5ffff",
    "#y c #dedaff",
    ".B c #dedeff",
    "#b c #deffff",
    ".t c #e6e2f6",
    ".k c #e6e6f6",
    "#D c #e6e6ff",
    ".X c #e6f2ff",
    ".S c #e6ffff",
    "#x c #eeeeff",
    ".M c #eef2ff",
    ".o c #f6eeff",
    ".w c #f6f2f6",
    ".q c #f6f2ff",
    "#r c #f6f6ff",
    ".2 c #f6ffff",
    ".p c #fffaf6",
    ".A c #fffaff",
    ".l c #ffffff",
    "QtQtQtQtQtQtQtQt.#.a.bQtQtQtQtQt",
    "QtQtQtQtQt.c.d.e.f.g.hQtQtQtQtQt",
    "QtQtQt.c.i.j.k.k.l.m.nQtQtQtQtQt",
    "QtQt.c.o.p.q.l.l.l.rQtQtQtQtQtQt",
    "QtQt.s.l.q.l.l.l.t.uQtQtQtQtQtQt",
    "QtQt.v.l.w.l.l.q.x.yQtQtQtQtQtQt",
    "QtQt.z.A.l.l.B.C.D.E.FQtQtQtQtQt",
    "QtQt.G.q.w.H.I.J.K.L.M.NQtQtQtQt",
    "Qt.O.l.P.Q.R.S.T.U.V.W.X.YQtQtQt",
    "Qt.Z.0.1.2.3.4.5.6.7.8.9#.##QtQt",
    "Qt#a.l.l.l#b#c#d#e#f#g#h#i.q#jQt",
    "Qt#k.l.l.l.l.4#l#m#n#o.l.l.q#pQt",
    "QtQt#q.l#r.l.l#s#t.l.q.A#u#vQtQt",
    "QtQtQt#w#x#y.l.A#z.q#A#BQtQtQtQt",
    "QtQtQtQt#C.B#D#E#F#GQtQtQtQtQtQt",
    "QtQtQtQtQt#H#I#J#KQtQtQtQtQtQtQt"
};

static const char* const m_floppy_icon_data[] = {
    "16 16 140 2",
    "Qt c None",
    "#Z c None",
    "ag c None",
    "aj c None",
    ".1 c None",
    ".e c None",
    ".# c None",
    ".f c None",
    ".n c None",
    "#D c #000000",
    "a# c #000000",
    "#0 c #000000",
    ".R c #eee6ee",
    "aa c #000000",
    "#O c #000000",
    "#R c #000000",
    "#E c #000000",
    "ab c #000000",
    "#t c #292829",
    "#3 c #000000",
    "ah c #393c39",
    "#F c #202020",
    "#C c #181c18",
    "#Q c #000000",
    "#2 c #000000",
    "#P c #000000",
    ".G c #7b797b",
    "#1 c #000000",
    "a. c #202429",
    "#N c #202020",
    "#b c #bdbebd",
    "#4 c #414441",
    ".H c #f6f2f6",
    "#a c #4a4441",
    ".a c #cdc6c5",
    "ai c #312c29",
    ".o c #dedade",
    "#u c #949194",
    ".g c #d5d2d5",
    ".m c #9c959c",
    "af c #292c29",
    "#S c #6a696a",
    ".2 c #bdbabd",
    ".x c #d5d6d5",
    ".Q c #737173",
    ".b c #d5ced5",
    "#k c #a4a5a4",
    "#Y c #313039",
    "#M c #292831",
    "#9 c #313439",
    "#B c #393c4a",
    "ae c #413c41",
    "#X c #414052",
    "#j c #525052",
    "#s c #525962",
    "ac c #625d62",
    "#8 c #6a657b",
    ".0 c #6a696a",
    "#r c #7371a4",
    "## c #737594",
    "#i c #7b79a4",
    ".Z c #8b89ac",
    "#. c #8b8d94",
    ".F c #948d94",
    ".w c #949194",
    ".P c #9491ac",
    ".O c #9c95b4",
    "#L c #9c99a4",
    "#5 c #9c9d9c",
    ".E c #9c9dac",
    ".u c #a4a1ac",
    ".z c #acaaac",
    ".k c #acaab4",
    ".v c #acaabd",
    ".l c #acaeb4",
    ".J c #b4b2b4",
    "#G c #b4b2d5",
    ".d c #bdb6bd",
    "#7 c #bdbab4",
    ".h c #bdbabd",
    ".C c #bdbac5",
    "ad c #bdbacd",
    ".W c #bdbad5",
    ".N c #bdbed5",
    "#T c #bdbede",
    "#v c #c5c2c5",
    ".5 c #c5c2cd",
    "#6 c #c5c2de",
    ".c c #c5c6c5",
    ".i c #cdc6cd",
    ".p c #cdcacd",
    ".D c #cdcad5",
    ".M c #cdcade",
    ".I c #cdced5",
    "#A c #d5b27b",
    "#W c #d5c29c",
    "#U c #d5ce7b",
    ".A c #d5ced5",
    ".q c #d5d2d5",
    ".L c #d5d2de",
    "#h c #debe52",
    "#n c #deceac",
    ".V c #ded6e6",
    ".6 c #dedee6",
    ".K c #dedeee",
    "#l c #e6deee",
    ".T c #e6e2e6",
    ".U c #e6e2ee",
    ".Y c #e6e68b",
    ".j c #e6e6e6",
    "#m c #e6e6f6",
    ".X c #eee6c5",
    ".t c #eeeaee",
    "#c c #eeeaf6",
    "#d c #f6d6ac",
    ".4 c #f6eef6",
    ".3 c #f6f2f6",
    ".B c #f6f6f6",
    "#q c #ffc608",
    "#z c #ffce00",
    "#y c #ffce18",
    "#K c #ffd64a",
    "#J c #ffda18",
    "#p c #ffde31",
    "#x c #ffe641",
    "#V c #ffee31",
    "#I c #ffee41",
    "#o c #fff252",
    "#g c #fff641",
    "#w c #fff662",
    ".y c #fff6f6",
    ".S c #fff6ff",
    "#H c #fffa83",
    ".r c #fffaff",
    ".9 c #ffff62",
    "#f c #ffff8b",
    ".8 c #ffff9c",
    "#e c #ffffb4",
    ".7 c #ffffc5",
    ".s c #ffffff",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQt.#.a.b.c.d.eQtQtQt",
    "QtQtQtQtQt.f.g.h.i.j.k.l.mQtQtQt",
    "QtQtQt.n.o.p.q.r.s.t.u.v.wQtQtQt",
    "QtQtQt.x.y.z.A.B.C.D.D.E.F.GQtQt",
    "QtQt.H.j.s.I.J.K.L.M.N.O.P.QQtQt",
    "QtQt.R.c.S.T.U.V.M.W.X.Y.Z.0.1Qt",
    "QtQtQt.2.3.4.5.M.6.7.8.9#.###aQt",
    "QtQtQt#b.A#c.5.s#d#e#f#g#h#i#jQt",
    "QtQtQtQt#k#l.L#m#n.8#o#p#q#r#s#t",
    "QtQtQtQt#u.6.V#v#e#w#x#y#z#A#B#C",
    "QtQt#D#E#F.w.D#G#H#I#J#K#L#M#NQt",
    "Qt#O#P#Q#R#S#m#T#U#V#W#X#Y#ZQtQt",
    "#0#1#P#2#3#4#5#6#7#8#9a.QtQtQtQt",
    "QtQta#aaab#OacadaeafagQtQtQtQtQt",
    "QtQtQtQtQtQtahaiajQtQtQtQtQtQtQt"
};


static const char * v_folderlocked_icon_data[]={
    "16 15 13 1",
    "g c #808080",
    "e c #cece63",
    "k c #c3c3c3",
    "c c #ffffce",
    "j c #dca149",
    "h c #ffce9c",
    "d c #ffff9c",
    "f c #c05800",
    "a c #000000",
    "i c #ff8000",
    "  c None",
    "b c #f7f7f7",
    "# c #9c9c00",
    "                ",
    "  #####a        ",
    " #bccdd#a       ",
    "#eeeeeee#fg###  ",
    "#ccccccccfgcdea ",
    "#cddddddfgddhea ",
    "#cdddddcfgchdea ",
    "#cddddciiiichea ",
    "#cddddcijkihdea ",
    "#cddddciiiichea ",
    "#cdddhdhfgchhea ",
    "#chdhdhdfghhhea ",
    "#eeeeeeefgeeeea ",
    " aaaaaaafgaaaaa ",
    "                "
};


static const char* v_folderclosed_icon_data[]={
    "16 16 8 1",
    "e c #cece63",
    "c c #ffffce",
    "f c #ffce9c",
    "d c #ffff9c",
    "a c #000000",
    "  c None",
    "b c #f7f7f7",
    "# c #9c9c00",
    "                ",
    "  #####a        ",
    " #bccdd#a       ",
    "#eeeeeee######  ",
    "#cccccccccccdea ",
    "#cddddddddddfea ",
    "#cdddddddfdfdea ",
    "#cddddddddfdfea ",
    "#cdddddfdfdfdea ",
    "#cddddddfdfdfea ",
    "#cdddfdfdfdffea ",
    "#cfdfdfdfdfffea ",
    "#eeeeeeeeeeeeea ",
    " aaaaaaaaaaaaaa ",
    "                ",
    "                "
};

static const char* v_folderopen_icon_data[]={
    "16 16 7 1",
    "c c #cece63",
    "e c #ffce9c",
    "b c #ffff9c",
    "d c #000000",
    "  c None",
    "a c #ffffff",
    "# c #9c9c00",
    "                ",
    "   #####        ",
    "  #aaaaa#       ",
    " #abbbbba###### ",
    " #abbbbbbbaaaacd",
    " #abbbbbbbbbbbcd",
    "############cbcd",
    "#abbbbbbebecdccd",
    "#abbbbebbebcdccd",
    " #abbbbbebee#d#d",
    " #abbebebeeecd#d",
    "  #aebebeeeec#dd",
    "  ############dd",
    "   ddddddddddddd",
    "                ",
    "                "
};

static const char * v_file_icon_data []={
    "14 16 25 1",
    " 	c None",
    ".	c #B3B3B3",
    "+	c #FFFFFF",
    "@	c #ACACAC",
    "#	c #FDFDFD",
    "$	c #FCFCFC",
    "%	c #FBFBFB",
    "&	c #FAFAFA",
    "*	c #F9F9F9",
    "=	c #F7F7F7",
    "-	c #F6F6F6",
    ";	c #F4F4F4",
    ">	c #F1F1F1",
    ",	c #F8F8F8",
    "'	c #F5F5F5",
    ")	c #F2F2F2",
    "!	c #ECECEC",
    "~	c #F3F3F3",
    "{	c #EEEEEE",
    "]	c #EBEBEB",
    "^	c #FEFEFE",
    "/	c #EDEDED",
    "(	c #EFEFEF",
    "_	c #F0F0F0",
    ":	c #BEBEBE",
    "............. ",
    ".+++++++++++. ",
    "@+++++##$$%&. ",
    "@++++#%*=-;>. ",
    "@+++#$&,-')!. ",
    "@+++#&*='~{]. ",
    "@++^$&=-~{!/. ",
    "@^^$&,')({{(. ",
    "@==-'~)____>. ",
    "@~~~~~)____>. ",
    "@--------:::. ",
    "@,,,,,,--:!.  ",
    "@&&&&&&--:.   ",
    "@.........    ",
    "              ",
    "              "
};


static const char* const v_back_icon_data[] = {
    "16 16 33 1",
    " 	c None",
    ".	c #2E59A4",
    "+	c #2D58A3",
    "@	c #3D66AD",
    "#	c #90AAD6",
    "$	c #3660A9",
    "%	c #6085C2",
    "&	c #BDCDE9",
    "*	c #C6D3EB",
    "=	c #7C9ACD",
    "-	c #3C64AC",
    ";	c #335DA7",
    ">	c #3A63AB",
    ",	c #B1C4E3",
    "'	c #C7D5EC",
    ")	c #91ACD7",
    "!	c #94ACD7",
    "~	c #8DA9D5",
    "{	c #85A2D2",
    "]	c #8CA9D4",
    "^	c #83A1D1",
    "/	c #3862AA",
    "(	c #355FA8",
    "_	c #4169AF",
    ":	c #4870B3",
    "<	c #5178B8",
    "[	c #6B91C8",
    "}	c #7093C9",
    "|	c #4069AF",
    "1	c #4B74B6",
    "2	c #4A73B5",
    "3	c #305AA5",
    "4	c #3760A9",
    "                ",
    "                ",
    "     ..         ",
    "   +@#$         ",
    " ..%&*=-;..     ",
    "..>,')!~{]^/.   ",
    " +..(-__:<[}|.  ",
    "   +...  .;123. ",
    "     +.    .|4. ",
    "           .4(. ",
    "          ..3.  ",
    "         ....   ",
    "        ....    ",
    "       ...      ",
    "                ",
    "                "
};



static const char* const a_0_data[] = {
    "40 40 88 2",
    "Qt c None",
    "#l c #000000",
    "#h c #200000",
    "#k c #310000",
    ".7 c #4a0000",
    "#s c #4a0010",
    "#p c #521000",
    ".4 c #521010",
    ".9 c #523039",
    ".S c #620800",
    "#o c #620810",
    ".Z c #621010",
    "#u c #622029",
    ".0 c #731000",
    ".5 c #731010",
    ".6 c #7b1010",
    ".O c #8b3039",
    "#b c #8b616a",
    ".1 c #941000",
    ".P c #941010",
    ".T c #941029",
    ".Q c #941810",
    "#. c #941829",
    ".A c #9c1029",
    ".V c #9c1810",
    ".K c #9c2010",
    ".U c #a42029",
    ".8 c #ac1810",
    ".B c #ac2010",
    ".W c #ac2029",
    ".n c #ac5952",
    "#j c #ac9d94",
    ".R c #bd2010",
    ".o c #bd2029",
    ".2 c #bd3029",
    "#n c #bd8d7b",
    "#g c #bdb6bd",
    "#d c #cd2010",
    ".w c #cd2029",
    "#e c #cd2839",
    ".u c #cd3029",
    ".J c #cdaeac",
    ".3 c #d53839",
    ".g c #de2029",
    "## c #de2839",
    ".p c #de3029",
    ".j c #de3039",
    "#q c #de3829",
    ".C c #de3839",
    ".v c #de3852",
    ".X c #de4839",
    ".H c #de95ac",
    ".d c #de9dac",
    ".h c #deaeac",
    "#r c #deb6bd",
    ".D c #debeac",
    ".x c #debebd",
    ".M c #decebd",
    ".E c #ded6d5",
    ".G c #dedeee",
    "#f c #e6596a",
    "#m c #e69594",
    ".s c #e69d94",
    "#v c #e6ded5",
    "#i c #e6e6ee",
    ".z c #ee2829",
    ".t c #ee2839",
    ".m c #ee3029",
    ".r c #ee3039",
    ".Y c #ee3829",
    "#a c #ee3839",
    ".y c #ee3852",
    ".I c #ee4839",
    ".b c #ee5052",
    ".a c #ee6152",
    ".k c #ee616a",
    ".# c #ee717b",
    ".i c #ee797b",
    ".L c #ee957b",
    ".l c #ee9594",
    ".F c #eebebd",
    ".e c #eeded5",
    "#c c #eee6d5",
    ".c c #eeeeee",
    ".f c #f6716a",
    ".q c #f6cebd",
    "#t c #f6d6d5",
    ".N c #ff697b",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQt.#.a.b.#.cQtQtQtQtQtQtQtQtQtQtQt.d.b.b.b.eQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt.f.g.b.d.h.i.b.jQtQtQtQtQtQtQtQt.#.b.k.l.h.f.m.gQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQt.n.o.p.f.h.c.e.b.m.m.qQtQtQtQtQt.g.g.r.s.c.e.l.b.m.tQtQtQtQtQtQtQt",
    "QtQtQtQtQtQt.h.o.o.u.v.f.s.e.c.f.g.m.sQtQtQt.w.g.r.c.c.x.l.a.y.m.z.rQtQtQtQtQtQt",
    "QtQtQtQtQtQt.A.B.B.o.C.k.i.D.E.c.F.p.g.eQt.w.g.k.c.G.x.H.#.b.I.m.r.z.G.xQtQtQtQt",
    "QtQtQtQtQt.J.K.K.B.o.u.b.L.h.M.E.M.d.z.g.w.w.N.x.M.G.M.H.k.y.m.m.r.m.rQtQtQtQtQt",
    "QtQtQtQtQt.O.P.Q.A.A.o.u.b.i.H.H.i.f.a.z.R.j.f.f.l.H.l.k.b.m.m.m.m.t.rQtQtQtQtQt",
    "QtQtQtQtQt.S.T.P.K.U.V.W.o.p.k.a.a.b.y.m.R.u.X.b.b.k.y.Y.m.t.m.m.z.r.mQtQtQtQtQt",
    "QtQtQtQtQt.Z.0.1.P.T.K.U.W.w.p.y.b.b.y.m.W.2.3.C.b.r.j.m.m.m.r.r.m.m.zQtQtQtQtQt",
    "QtQtQtQtQt.4.5.P.6.P.U.V.o.W.w.p.b.b.b.m.W.u.C.b.p.j.z.r.m.z.m.t.m.r.mQtQtQtQtQt",
    "QtQtQtQtQt.7.5.5.0.P.P.K.8.o.W.g.g.b.f.f.H.f.b.p.p.z.g.z.z.m.z.r.m.m.mQtQtQtQtQt",
    "QtQtQtQtQt.9.S.5.5#..P.U.K.B.W.R.w.p.y.l.h.a.m##.g.p.z.z.z.r.r.m#a.r.mQtQtQtQtQt",
    "QtQtQtQtQt#b.4.Z.6.P.Q.Q.8.W.W.R.R.w.w.g.p.w.g.p.g.g.m.z.m.r.m#a.b.m.mQtQtQtQtQt",
    "QtQtQtQtQt#c.7.7.5.5.T.Q.U.8.W.W.o.o#d.w.w.g.g.g#e.z.g.m.m#a.I.b.b#a#fQtQtQtQtQt",
    "QtQtQtQtQt#g#h.7.Z.5.P.P.A.K.W.B.o.o.o#d.w.g.g.g.g.t.j.r.I.y.a.f.a.m#iQtQtQtQtQt",
    "QtQtQtQtQtQt#j#k.Z.Z.P.T.Q.U.B.o.B.w.o.w.w.w.w.p.g.p.r#a.b.f.i.s.k.pQtQtQtQtQtQt",
    "QtQtQtQtQtQtQt#l.7.Z.5.P.Q.K.W.W.B.o.o.w.w.g.w.g.j.m.Y.b.f#m.h.d.y.eQtQtQtQtQtQt",
    "QtQtQtQtQtQtQt#n#h.7.5.5.P.A.8.V.o.B.o.o.o.w.w.p.z#a.b.#.d.E.E.l.mQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt#h#k.S.5.6.P.V.W.W.B.R.o.w.w.u###a.b.i.h.c.c.M.m.cQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt#k.7#o.5.T.P.P.W.W.o.R.w.w.p.p.y.#.d.c.c.c.f#mQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt.E#k#p.Z.5.P.V.U.V.W.o.o.w#q.y.f.d.G.c.c.d.bQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQt#j.7#o.Z.6.P.Q.K.U.W.o.u.C.a.l#r.c.c#m.YQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQt.M#p.5.Z.5.P.Q.A.B.W.u.X.k.H.D.M.f.bQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQt.e#s.5.6.6.1.Q.V.o.u.v.f#m.d.b.iQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQt#o.5.6.Q.P.U.W.p.b.#.L.b#tQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#b.5.6.P.K.W.j.f.f.YQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#u.6.A.2.f.Y.eQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#v.5.p.dQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt"
};

static const char* const a_1_data[] = {
    "40 40 93 2",
    "Qt c None",
    "#i c #200000",
    "#k c #310000",
    "## c #4a0000",
    "#b c #4a0010",
    "#h c #521000",
    "#r c #521010",
    "#t c #620800",
    "#o c #620810",
    ".4 c #621010",
    "#y c #622029",
    ".7 c #6a0800",
    "#p c #6a0810",
    ".Y c #731000",
    ".8 c #731010",
    "#e c #735952",
    ".Z c #7b1010",
    "#f c #7b1029",
    "#q c #941000",
    ".V c #941010",
    "#c c #941029",
    ".0 c #941810",
    ".U c #941829",
    "#w c #945952",
    ".1 c #9c1029",
    ".Q c #9c1810",
    ".H c #9c2010",
    "#d c #a42029",
    ".C c #ac1829",
    ".J c #ac2010",
    ".I c #ac2029",
    ".P c #ac9d94",
    ".3 c #bd2010",
    ".s c #bd2029",
    ".5 c #bd3029",
    ".r c #bdb6bd",
    "#a c #cd2010",
    ".t c #cd2029",
    "#j c #cd2810",
    "#s c #cd2839",
    ".D c #cd3029",
    ".j c #cd3039",
    "#z c #cdaeac",
    ".6 c #d53829",
    ".G c #de2029",
    ".q c #de2839",
    ".u c #de3029",
    ".9 c #de3039",
    "#x c #de3829",
    ".K c #de3839",
    ".i c #de3852",
    "#. c #de4839",
    ".2 c #de5052",
    ".B c #de796a",
    ".W c #de95ac",
    ".E c #de9dac",
    ".l c #deaeac",
    ".h c #debebd",
    ".T c #decebd",
    ".S c #ded6d5",
    "#n c #dedeee",
    ".m c #e6596a",
    "#A c #e6716a",
    ".N c #e69594",
    ".A c #e69d94",
    ".d c #e6a5ac",
    "#u c #e6ae94",
    ".L c #e6ded5",
    ".# c #e6e6ee",
    ".x c #ee2829",
    ".O c #ee2839",
    ".p c #ee3029",
    ".n c #ee3039",
    ".z c #ee3829",
    "#g c #ee3839",
    ".g c #ee3852",
    ".w c #ee4839",
    ".b c #ee5052",
    ".c c #ee6152",
    ".a c #ee616a",
    ".f c #ee717b",
    ".R c #ee797b",
    ".X c #ee957b",
    ".o c #ee9594",
    "#v c #eeaeac",
    ".e c #eebebd",
    ".M c #eeded5",
    ".y c #eee6d5",
    ".v c #eeeeee",
    "#l c #f6596a",
    ".k c #f6716a",
    "#m c #f6cebd",
    ".F c #f6d6d5",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt.#.a.b.c.dQtQtQtQtQtQtQtQtQtQtQtQt.e.f.g.g.e.hQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt.i.j.k.l.l.k.g.mQtQtQtQtQtQtQtQt.h.n.c.o.l.o.p.q.#.hQtQtQtQtQtQt",
    "QtQtQtQtQt.rQt.s.t.u.k.h.v.h.w.p.x.yQtQtQtQtQt.i.u.z.f.v.v.A.g.p.x.vQtQtQtQtQtQt",
    "QtQtQtQtQtQt.B.C.s.D.i.k.E.v.v.a.p.x.FQtQtQt.t.G.n.e.v.h.o.b.b.p.p.n.hQtQtQtQtQt",
    "QtQtQtQt.rQt.H.I.J.s.K.c.f.E.L.v.E.G.G.vQt.D.G.c.v.M.h.N.f.b.n.p.p.O.AQtQtQtQtQt",
    "QtQtQtQtQt.P.Q.H.J.I.D.g.R.E.S.h.S.l.x.t.G.t.a.T.T.S.l.o.c.g.z.n.n.p.nQtQtQtQtQt",
    "QtQtQtQtQt.U.V.Q.Q.J.I.s.b.k.W.E.o.k.c.x.t.D.k.X.o.E.o.c.b.p.p.n.x.p.pQtQtQtQtQt",
    "QtQtQtQtQt.Y.Z.V.U.0.1.I.s.2.a.c.a.b.b.n.3.j.2.g.c.c.g.z.O.n.p.O.p.p.n.vQtQtQtQt",
    "QtQtQtQtQt.4.Z.V.0.V.H.J.I.s.u.b.b.b.p.n.I.5.6.b.b.n.n.x.x.p.p.x.p.p.n.MQtQtQtQt",
    "QtQtQtQtQt.7.8.Z.V.0.0.H.J.I.t.u.b.g.w.p.I.j.9#..i.p.G.p.O.p.n.p.p.p.p.vQtQtQtQt",
    "QtQtQtQtQt##.8.8.Z.V.V.Q.J.s.I#a.u.b.k.b.k.2.c.n.G.u.x.p.x.n.p.n.p.n.p.yQtQtQtQt",
    "QtQtQtQtQt#b.Y.8.8.V#c#d.H.J.I.s.t.u.b.l.M.k.u.j.u.u.n.x.n.n.p.g.g.p.nQtQtQtQtQt",
    "QtQtQtQtQt#e#b.7#f.V.V.U.1.J.s.I.3.t.t.G.u.u.G.G.G.G.G.p.n.p#g.b.z#g.pQtQtQtQtQt",
    "QtQtQtQtQt.L###h.8.8.V.V.Q.C.I.3.s.s.t.s.t.t.u.G.u.q.x.n.p.g.b.b.b.b.wQtQtQtQtQt",
    "QtQtQtQtQt.r#i##.4.Y#c.V.U.H.J.J.J.s.s.t#j.D.G.u.G.u.p#g.b.b.a.a.k.n.MQtQtQtQtQt",
    "QtQtQtQtQtQt.P#k.4.4.8#c.0.H.J.I.3.3.t.t.t.G.t.G.u.n.p#g.c.f.o.N.a.GQtQtQtQtQtQt",
    "QtQtQtQtQtQt.r#i#b.8.Y.V.0.0.H.C.I.s.3.t.t.D.t.u.9.n.b.c.X.E.l.E#l#mQtQtQtQtQtQt",
    "QtQtQtQtQtQtQt.P#k##.8.8.V.0.1.C.s.J.s.3.t.t.G.u.z.w.c.o.l.L#n.A.pQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt#i###o.8.8.V.U.H.C.I.s.s.t.t.q.9.b.c.W.h.v.v.M.b.vQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt#k#b#p.Y#q.V.H.Q.J.s.I.t.t.u#g.c.o.h.v.v.v.R.AQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt.S###r.8.8.Z.0.1.1.I.J.s#s.K.b.k.l.v.v.v.A#lQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQt.P#b#t.4.Z.0.0#d.I.J.I.D.K.c.X.T.v.v#u.pQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQt.S##.8#t.Z#q.H.1.I.s.j#..k.W.S.S.N.2QtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQt.L#r.8.8.Z.U.H.J.I.D.g.k.l.h.k.RQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQt#b.8.8.V#c.I.I.i.b.o.E.a#vQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#w.8.8.0#d.I#x.f.X.g.rQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt.L#y.Z.0.D.f.b.EQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#z.Z.u#AQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt"
};

static const char* const a_2_data[] = {
    "40 40 89 2",
    "Qt c None",
    "#m c #200000",
    "#i c #310000",
    "#c c #4a0000",
    "#e c #4a0010",
    "#q c #521000",
    "#f c #521010",
    ".9 c #620800",
    ".8 c #621010",
    "#j c #622029",
    ".Y c #731000",
    ".R c #731010",
    "#n c #735952",
    "#. c #7b1010",
    ".2 c #7b1029",
    ".N c #8b3039",
    "#u c #8b616a",
    ".Z c #941000",
    ".S c #941010",
    ".3 c #941029",
    ".T c #941810",
    ".0 c #941829",
    "#w c #945952",
    "## c #9c1029",
    ".O c #9c1810",
    ".U c #9c2010",
    ".I c #a42029",
    "#g c #ac1810",
    "#a c #ac1829",
    ".5 c #ac2010",
    ".B c #ac2029",
    "#h c #ac9d94",
    ".V c #bd2010",
    ".r c #bd2029",
    ".W c #bd3029",
    ".6 c #bd3039",
    "#l c #bdb6bd",
    ".h c #cd2029",
    ".y c #cd3029",
    ".7 c #cd3039",
    "#v c #cdaeac",
    ".K c #de2029",
    ".E c #de2839",
    ".D c #de3029",
    ".s c #de3039",
    ".4 c #de3829",
    "#t c #de3839",
    ".C c #de3852",
    "#b c #de4839",
    ".L c #de5052",
    ".J c #de796a",
    ".P c #de95ac",
    ".# c #de9dac",
    ".k c #deaeac",
    ".F c #debeac",
    ".Q c #debebd",
    ".o c #decebd",
    ".j c #ded6d5",
    "#o c #dedeee",
    "#r c #dee6ee",
    ".x c #e6596a",
    "#k c #e69594",
    ".w c #e69d94",
    ".q c #e6a5ac",
    "#p c #e6ded5",
    ".A c #ee2829",
    ".p c #ee2839",
    ".a c #ee3029",
    ".n c #ee3039",
    ".i c #ee3829",
    ".X c #ee3839",
    ".H c #ee3852",
    ".c c #ee4839",
    ".b c #ee5052",
    ".f c #ee6152",
    ".z c #ee616a",
    ".g c #ee717b",
    ".t c #ee797b",
    ".G c #ee8d94",
    ".M c #ee957b",
    ".d c #ee9594",
    ".1 c #eeaeac",
    ".e c #eebebd",
    ".m c #eeded5",
    "#d c #eee6d5",
    ".v c #eeeeee",
    ".l c #f6716a",
    "#s c #f6cebd",
    ".u c #f6d6d5",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt.#.a.b.c.dQtQtQtQtQtQtQtQtQtQtQtQt.e.f.b.a.gQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt.h.i.d.j.k.g.b.lQtQtQtQtQtQtQtQt.m.n.l.d.o.#.b.p.qQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQt.r.h.s.t.u.v.w.i.n.pQtQtQtQtQtQt.x.y.n.z.v.v.#.z.a.A.qQtQtQtQtQtQt",
    "QtQtQtQtQtQt.B.r.B.y.C.f.w.v.v.i.D.pQtQtQtQt.C.D.E.w.v.F.G.z.H.i.p.AQtQtQtQtQtQt",
    "QtQtQtQtQt.u.I.B.B.r.C.b.l.#.j.v.J.K.AQtQt.L.h.D.v.m.k.M.l.b.n.n.a.A.HQtQtQtQtQt",
    "QtQtQtQtQt.N.O.O.I.B.h.C.g.P.k.Q.o.w.D.h.L.h.z.o.o.Q.k.M.z.b.n.a.A.a.aQtQtQtQtQt",
    "QtQtQtQtQt.R.S.T.U.U.V.W.L.l.d.#.d.t.f.A.h.D.g.d.P.P.d.f.H.X.a.p.n.p.n.vQtQtQtQt",
    "QtQtQtQtQt.Y.Z.S.0.T.U.B.r.L.f.z.f.f.b.a.r.y.b.f.z.l.H.X.a.A.n.n.a.a.n.1QtQtQtQt",
    "QtQtQtQtQt.R.Z.2.T.3.O.O.B.r.4.b.b.H.c.a.5.6.7.H.b.X.a.a.A.A.a.a.p.a.a.dQtQtQtQt",
    "QtQtQtQtQt.8.R.S.0.T.U.O.5.B.h.D.b.b.H.i.r.y.D.b.s.n.A.n.A.a.n.a.a.n.a.dQtQtQtQt",
    "QtQtQtQtQt.9.R#..S.0.T###a.B.5.h.a.f.b.c.C#b.f.X.D.D.K.n.a.a.a.a.n.n.a.1QtQtQtQt",
    "QtQtQtQtQt#c.Y.R.Y.U.S.I.I.5.r.r.h.D.f.o.v.M.i.D.K.K.a.A.a.a.X.n.X.X.a#dQtQtQtQt",
    "QtQtQtQtQt#e#f.8.Y.Z.T.T.U#g.5.B.h.h.h.a.s.K.D.K.D.A.E.a.a.X.n.c.i.c.aQtQtQtQtQt",
    "QtQtQtQtQt#h#c.9.R.S.T.0.S.U.B.V.r.r.r.h.h.K.K.K.K.D.a.n.H.i.b.b.f.b.AQtQtQtQtQt",
    "QtQtQtQtQtQt#i#c.R#..S.T.T.U.5.5.5.r.r.h.h.h.D.K.D.a.X.c.b.b.z.z.g.b.qQtQtQtQtQt",
    "QtQtQtQtQtQt#j#c.8.R#..S.S.S##.B.V.5.h.h.h.h.h.D.D.a.X.b.z.l.M#k.t.aQtQtQtQtQtQt",
    "QtQtQtQtQtQt#l#m#f.R.R.3.S.T.U.B.B.V.r.h.h.h.D.D.a.H.b.z.d#k.k.k.l.d#lQtQtQtQtQt",
    "QtQtQtQtQtQtQt#n#c.8.9#..S.S.U##.5.B.V.r.h.h.D.D.X.b.l.d.k.j#o.k.aQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt#m#c.R.Y.S.S.0.U.5.5.B.r.h.h.D.X.H.l#k.k#p.v.v.z.mQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt#o#i#q.R.R#..S.3.U#g.B.B.r.h.D.c.f.d.k#r.v.v.k.fQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt#h#e#q.R.R.Z.T.I##.5.r.r.D.s.b.l.##p.v.v#s.HQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQt#n#e.8.8.S.S.S###a.5.r.y#t.f.t.F.v.v#s.nQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQt#u#c.Y.R.S.3.O.U.B.B.y.L.z#k.j#d.Q.aQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQt#l#f.R.Y.S.U.O.5.r.y.H.l.#.j.#.LQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQt#p#e.R.R.S.U.I.B.y.f.d.k#k.tQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQt.8.R.R##.I.r.4.g#k.b.uQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#v.9.S.I.h.l.l.zQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#w#..y.H.vQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt.uQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt"
};

static const char* const a_3_data[] = {
    "40 40 89 2",
    "Qt c None",
    "#m c #200000",
    "#o c #310000",
    "#i c #4a0000",
    "#g c #4a0010",
    "#r c #521010",
    "#k c #523039",
    "#l c #620800",
    "#. c #621010",
    "#v c #622029",
    "#p c #6a0810",
    ".5 c #731000",
    ".0 c #731010",
    ".6 c #7b1010",
    "#b c #7b1029",
    "#u c #7b2829",
    "#s c #941000",
    ".V c #941010",
    ".W c #941029",
    ".7 c #941810",
    ".4 c #941829",
    "#j c #9c1029",
    ".8 c #9c1810",
    ".O c #9c2010",
    ".X c #a42029",
    "#c c #ac1810",
    ".P c #ac1829",
    ".Q c #ac2010",
    ".M c #ac2029",
    "#n c #ac9d94",
    "## c #b42839",
    "#d c #bd2010",
    ".D c #bd2029",
    "#e c #bd3039",
    ".N c #bd8d7b",
    ".# c #bdb6bd",
    "#t c #cd2010",
    ".w c #cd2029",
    ".E c #cd3029",
    ".1 c #cd3039",
    ".S c #cdaeac",
    ".I c #de2029",
    "#a c #de2839",
    ".k c #de3029",
    ".u c #de3039",
    ".9 c #de3839",
    ".F c #de3852",
    "#f c #de4839",
    ".p c #de5052",
    ".v c #de796a",
    ".q c #de95ac",
    ".j c #de9dac",
    ".n c #deaeac",
    ".K c #deb6bd",
    ".l c #debeac",
    ".e c #debebd",
    ".r c #decebd",
    ".m c #ded6d5",
    "#q c #dedeee",
    ".i c #e6596a",
    ".T c #e69594",
    ".B c #e69d94",
    ".R c #e6a5ac",
    ".G c #e6ded5",
    ".o c #e6e6ee",
    ".U c #ee2829",
    ".2 c #ee2839",
    ".C c #ee3029",
    ".t c #ee3039",
    ".Y c #ee3829",
    ".J c #ee3839",
    ".h c #ee3852",
    ".z c #ee4839",
    ".b c #ee5052",
    ".g c #ee6152",
    ".y c #ee616a",
    ".c c #ee717b",
    ".L c #ee797b",
    ".Z c #ee8d94",
    ".a c #ee957b",
    ".f c #ee9594",
    "#h c #eeaeac",
    ".3 c #eebebd",
    ".d c #eeded5",
    ".A c #eee6d5",
    ".x c #eeeeee",
    ".s c #f6716a",
    "#w c #f6cebd",
    ".H c #f6d6d5",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt.#QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt.a.b.b.c.dQt.eQtQtQtQtQtQtQtQtQtQtQt.f.g.h.i.dQtQtQtQtQtQtQtQt",
    "QtQtQtQtQt.#Qt.j.k.b.l.m.n.c.h.oQtQtQtQtQtQtQtQtQt.p.g.q.r.r.s.t.uQtQtQtQtQtQtQt",
    "QtQtQtQtQtQt.v.w.u.s.n.x.d.y.z.t.cQtQtQtQtQtQt.A.u.t.b.B.x.x.B.b.t.CQtQtQtQtQtQt",
    "QtQtQtQtQt.r.D.D.E.F.s.j.G.x.y.C.C.pQtQtQtQt.H.w.I.J.m.x.K.L.g.h.C.t.CQtQtQtQtQt",
    "QtQtQtQtQt.M.M.M.D.E.b.y.f.n.x.n.k.I.LQtQt.x.I.I.y.x.r.q.c.y.b.t.C.C.C.HQtQtQtQt",
    "QtQtQtQt.N.O.P.Q.M.E.F.s.c.q.n.e.r.C.I.RQt.w.w.f.l.S.j.T.s.b.h.J.C.U.C.UQtQtQtQt",
    "QtQtQtQt.V.W.X.O.M.Q.k.b.s.T.T.f.j.a.Y.I.I.k.c.f.f.q.f.Z.s.b.C.t.C.C.C.tQtQtQtQt",
    "QtQtQtQt.0.V.V.V.X.O.D.E.h.s.s.y.y.y.g.C.w.1.b.g.y.s.y.g.h.C.t.C.2.C.U.C.3QtQtQt",
    "QtQtQtQt.0.V.4.W.O.X.M.M.w.k.b.b.b.h.z.t.D.E.u.p.b.b.b.h.t.2.C.C.t.2.C.C.TQtQtQt",
    "QtQtQtQt.5.6.W.7.W.8.M.D.D.w.k.h.b.Y.J.t.M.E.k.9.z.9.C.I.C.t.U.U.C.t.C.t.cQtQtQt",
    "QtQtQtQt#..0.6.V.7.8.O.M.M.Q.w.u.b.h.Y.t.D##.E.9.k.k.k#a.C.U.C.t.U.C.C.t.cQtQtQt",
    "QtQtQtQt.5.0#b.6.4.8#c.M.Q#d.D.I.C.b.h.h#e.k.F#f.C.k.U.U.U.t.C.t.C.t.t.C.LQtQtQt",
    "QtQtQtQt#g.5.0.V.7.7.8.X.Q.M.D.w.I.z.s.R.x.L.b.w.I.k#a.U.C.U.t.t.Y.Y.h.t#hQtQtQt",
    "QtQtQtQt#i#..0.6.W.7#j#c.Q.M.D#d.w.w.C.b.b.h.I.I.k.I.I.U.2.t.C.J.z.z.C.t.dQtQtQt",
    "QtQtQtQt#k#i#l.6.7.W.8.8.M.M.Q.D.D.w.w.w.I.w.E.I.I.k.C.U.C.t.t.z.h.b.b.UQtQtQtQt",
    "QtQtQtQt.m#i#..0.5.7.4.O.O.Q.Q.M.D.w.Q.w.w.w.I.k.k.I.k.C.h.Y.b.b.g.s.b.bQtQtQtQt",
    "QtQtQtQt.##m#i.0.0.V.7.V#j.P.M.M#d.D.D.w.w.w.w.I.k.k.t.z.b.b.b.c.s.c.h.xQtQtQtQt",
    "QtQtQtQtQt#n#o#p.0.V.V.4.8.X#c#d.M.D#d.w.w.w.I.I.k.t.t.z.b.y.s.T.j.a.kQtQtQtQtQt",
    "QtQtQtQtQtQt#m#i#..6.V.W.O#j.Q.M.M.Q.D#d.w.w.k.I.k.J.b.b.y.a.f.n.l.b.dQtQtQtQtQt",
    "QtQtQtQtQtQt.##o#.#..V.W.7.8.M.Q.Q.D.Q.w.w.E.I.k.u.Y.b.s.f.j.r#q.n.CQtQtQtQtQtQt",
    "QtQtQtQtQtQtQt#o#g#..6.6.7.O#j.P.P.Q.Q.D.w.w.E.k.J.b.y.a.j.G.x.A.zQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt#o#i#..6.4.V.7.O#c.Q.M.D.D.E.u.F.h.g.L.j#q.x.x.f#hQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt.G#r#r#l#s.W.4#j.P.Q.M.Q#t.1.k.k.b.c.f.m.x.x.R.LQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQt.m#i#..0.0.V.V.O#c.M.D.D.E.k.b.g.a.l.x.x.j.pQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQt.G#r#p#.#s.V#j.8.Q.M.D.w.k.b.s.j.G#q.f.sQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQt#i.0.0.6.W.O.Q.D.M.E#f.g.T.n.r.y.nQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQt#r.0.0.V.4.8.M.M.1.h.s.q.j.b.oQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQt#n#..0.V.O.Q.M.E.g.a.s.bQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#u.6.V.8.D.b.s.h.HQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#v.8##.h.3QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt.A#wQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt"
};

static const char* const a_4_data[] = {
    "40 40 198 2",
    "Qt c None",
    "aN c #200000",
    "aJ c #290000",
    "aC c #310000",
    "aL c #311829",
    "aM c #4a0000",
    "aY c #4a0010",
    "ay c #4a0800",
    "aT c #520800",
    "a5 c #521000",
    "aK c #521010",
    "aP c #523039",
    "as c #5a0800",
    "aD c #5a0810",
    "aU c #5a1010",
    "aZ c #620800",
    "an c #620810",
    "az c #621010",
    "ac c #622029",
    "aG c #6a0800",
    "aQ c #6a0810",
    "ai c #6a1000",
    "at c #6a1010",
    "ad c #731000",
    "ao c #731010",
    "aF c #735952",
    "#X c #7b1010",
    "#K c #7b1029",
    "ae c #7b1810",
    "b. c #7b2829",
    "#7 c #8b1010",
    "aj c #8b1810",
    "ba c #8b3039",
    "ah c #8b5952",
    "am c #8b596a",
    "a1 c #8b616a",
    "#8 c #941000",
    "#L c #941010",
    "#Y c #941029",
    "#s c #941810",
    "#M c #941829",
    "a8 c #945952",
    "aO c #9c1029",
    "#A c #9c1810",
    "#0 c #9c1829",
    "#Z c #9c2010",
    "#W c #9c6152",
    ".8 c #a41810",
    "ak c #a41829",
    "#t c #a42010",
    "#9 c #a42029",
    "aA c #ac1810",
    "af c #ac1829",
    "#j c #ac2010",
    "a. c #ac2029",
    "bc c #ac5952",
    "aX c #ac9d94",
    "ag c #b42010",
    ".R c #b42029",
    "#k c #b42839",
    "au c #bd2010",
    ".S c #bd2029",
    "a2 c #bd2810",
    "#O c #bd2829",
    "#B c #bd3029",
    "a9 c #bd3039",
    "#i c #bd8d7b",
    "ar c #bdb6bd",
    "aH c #cd2010",
    ".F c #cd2029",
    "aE c #cd2810",
    "#1 c #cd2829",
    "#4 c #cd2839",
    ".9 c #cd3029",
    ".T c #cd3039",
    "#J c #cdaeac",
    "#d c #d52029",
    "aI c #d52810",
    "#w c #d52829",
    "#2 c #d52839",
    "aV c #d53029",
    "#F c #d53039",
    "#l c #d53829",
    "#. c #d53839",
    ".2 c #de2029",
    ".n c #de2829",
    "aq c #de2839",
    ".G c #de3029",
    "#C c #de3039",
    "a3 c #de3829",
    "#N c #de3839",
    "a6 c #de3852",
    "al c #de4839",
    ".D c #de4852",
    ".7 c #de5052",
    "aB c #de796a",
    "#c c #de95ac",
    ".s c #de9dac",
    ".E c #dea5ac",
    ".W c #deaeac",
    "#o c #deb6ac",
    ".z c #deb6bd",
    "aS c #debeac",
    ".M c #debebd",
    ".B c #decebd",
    ".r c #ded6d5",
    "aR c #dedeee",
    "#f c #dee6ee",
    "#v c #e62029",
    ".Z c #e62829",
    "#6 c #e62839",
    ".C c #e63029",
    "#5 c #e63039",
    "ax c #e63829",
    ".P c #e63839",
    "#P c #e64839",
    "#m c #e64852",
    "b# c #e65052",
    ".L c #e6596a",
    ".# c #e6696a",
    "#I c #e6716a",
    "#p c #e68d94",
    ".e c #e69594",
    ".v c #e69d94",
    ".1 c #e6a5ac",
    ".p c #e6ae94",
    ".m c #e6aeac",
    "ap c #e6bebd",
    ".3 c #e6cebd",
    ".I c #e6ded5",
    ".A c #e6e6ee",
    "#U c #ee2829",
    ".0 c #ee2839",
    "#T c #ee3029",
    ".O c #ee3039",
    "#S c #ee3829",
    ".k c #ee3839",
    ".x c #ee3852",
    "#3 c #ee4839",
    "#Q c #ee4852",
    "#D c #ee5052",
    "## c #ee5952",
    "aW c #ee6152",
    "#e c #ee616a",
    "av c #ee717b",
    ".b c #ee797b",
    ".N c #ee8d7b",
    ".h c #ee8d94",
    ".i c #ee957b",
    ".c c #ee9594",
    "a7 c #eeaeac",
    ".f c #eebebd",
    ".Q c #eeded5",
    "bd c #eee6d5",
    ".q c #eeeeee",
    "#r c #f62829",
    "aa c #f62839",
    "#h c #f63029",
    "#V c #f63039",
    "a# c #f63829",
    "#H c #f63839",
    ".a c #f63852",
    "#R c #f64839",
    "#u c #f64852",
    "#G c #f65052",
    ".g c #f6596a",
    ".Y c #f6696a",
    "#q c #f6716a",
    ".4 c #f6717b",
    "#n c #f6797b",
    ".V c #f68d7b",
    "bb c #f6cebd",
    ".l c #f6d6d5",
    ".w c #f6eeee",
    "#z c #f6f6ee",
    ".X c #f6f6ff",
    "a0 c #f6ffee",
    "ab c #ff2829",
    "#y c #ff2839",
    ".5 c #ff3029",
    ".6 c #ff3039",
    "aw c #ff3829",
    ".K c #ff3839",
    ".J c #ff4839",
    ".u c #ff4852",
    ".o c #ff5052",
    ".j c #ff5952",
    "#E c #ff596a",
    ".d c #ff6152",
    ".U c #ff616a",
    ".y c #ff6952",
    "#a c #ff696a",
    ".t c #ff697b",
    "#x c #ff716a",
    "#g c #ff717b",
    "#b c #ff796a",
    "a4 c #ff797b",
    ".H c #ffffff",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQt.#.a.b.c.d.eQtQtQtQtQtQtQtQtQtQtQtQt.f.g.h.i.j.k.lQtQtQtQtQtQtQt",
    "QtQtQtQtQtQt.m.n.o.p.q.r.s.t.u.vQtQtQtQtQtQtQtQt.w.x.y.e.z.A.B.y.C.DQtQtQtQtQtQt",
    "QtQtQtQtQt.E.F.G.d.s.q.H.I.y.J.K.LQtQtQtQtQtQt.f.G.k.o.E.H.H.M.N.u.O.PQtQtQtQtQt",
    "QtQtQtQt.Q.R.S.T.D.U.V.W.q.X.Y.Z.0.OQtQtQtQt.1.2.n.k.3.X.B.v.4.j.u.5.6.7QtQtQtQt",
    "QtQtQtQt.8.R.R.9#.###a#b#c.z.q.m.G.C.7QtQt.l#d.2#e#f.B.E.i#g.U.o.J.6.5#h.QQtQtQt",
    "QtQtQt#i.8#j.R#k#l#m#a#n.e.e.s.W#o.k.n.7.Q#d.n#p#o.E.s.e.h#q.U.u.K.6#h.5#rQtQtQt",
    "QtQtQt#s.8#t#t.R.T.D.d#a#n.N.b#b#n#b#u.n#v#w#a#a#x#n.N.N.4#a.o.J.5#h#h#y.5#zQtQt",
    "QtQtQt#s#A.8.8#t.R#B#C#D#E.U.d.j.o.o.o.6.F#F#m#D#G.j.d.U.j.u.K#H#h#r.5.6.6#IQtQt",
    "QtQt#J#K#L#s#M.8#j.R.F#C#N#u.u.u.J#H.K.K#O.T#N#C#P#Q#Q#R#S.O#T#U#V#U#r.5.5.OQtQt",
    "QtQt#W#X#L#Y#Z#0#j.R.S#1#2#N#H#H#3.k#V#h.R#B#4#C#N#C#5#5.O#6#U.0#T#V.5.5.5#VQtQt",
    "QtQt#W#X#7#8#s#t#9a..R.R#1#w.Pa##S.k#h.O.R#O#1#F#C.G.G#6.Z#T#U#U#Uaa#h.6.5abQtQt",
    "QtQtacadae#7#s#Aafa.ag.R.F#d.n.k.a.aa##h#O#O.9#.#N.G#C.C.Z#U#U#U#Uaa#h.6#y#hQtQt",
    "QtQtahai#Kaj#s#tak#j.R.R.S#1#w#C#H.u.J.K.9#1#Nal.G.G.n.n.Z.Z#T.0aa#h#h.6.6aaQtQt",
    "QtQtamanao#X#s#A.8aga..R.R.S#1#w.G#R.U.vap#a#m#Caq.n.n.Z.n.C.0#U#V#V.6.6.6#rQtQt",
    "QtQtarasat#X#s#0.8#t#j.R.Rau.S#1#1.G#Q.dav#Qaq#C.n.naq.C#v#5#T#V#Vaw.J.K.6axQtQt",
    "QtQtQtayazaoaj#s#AakaA#j.R.Rau.S.F#w#2.n.naq#w#w.n.n.n.Z.Z#U#T.O#H.J.J.u.KaBQtQt",
    "QtQtQtaCaDat#7#s#0#t.8af.Rag.Rau.S.FaE.F#d#d.2#w.n.n.n.Z#6#T.O#H.J.u.j.j#V.wQtQt",
    "QtQtQtaFayaG#X#s#s#Zakafa..R.R.Rau.FaH.FaI#d#d#w.n.n.n#6.O.O#H.J.o.d.t#E.ZQtQtQt",
    "QtQtQtQtaJaKao#7#s#0akafafag.Rau.Rau.S#1#w#d#d#d.n.n.n#5#T#H#u.o#a#n.e.j.NQtQtQt",
    "QtQtQtQtaLaMaiao#L#s#tak#j.R.Rag.S.S.FaH#w#w#d.n.n.n.G#T.k.J.o#a#p.s.e.OQtQtQtQt",
    "QtQtQtQt.waNasataj#saO.8afaA.R.Rag.S.F.FaE.F.2#w.n#C#5.x.J.o#a#p.W.z#q#IQtQtQtQt",
    "QtQtQtQtQtaPayaQ#Xaj#s#0.8aAa..Ragau.F.S.F#1#daq.n.G.k#R.o#a.i#oaRaS.kQtQtQtQtQt",
    "QtQtQtQtQtQtaJaTai#X#L#AakaA#j.R.Rau.S.S.F#w#w#2.G#5.k.o#a.V#o.X#z.Y.lQtQtQtQtQt",
    "QtQtQtQtQtQt.AaCaUai#7#L#Z.8a.#j.R.Rau.S.F#1#waV#C#3#u.U.e.M#z.H.1aWQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtaXaYaZao#7#s#A#t#ta..R.Rau.S#w#w.G#P.J.y.V#oa0.H.3#HQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQta1aYaGaoaj#Y#0.8a.agag.Ra2#1#wa3#Q.da4.W#z.Hap#VQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtaXa5aiaoaj#s#0#t#j.Rag.S#1.Ga6#G.4.saR.q.1.kQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQt.raUataoaj#L#0.8#j.R.S#1aV#m.d.i#o.Mav##QtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtaUataoaj#s#A.8.R.R.9#F#Q.t#c.v.oa7QtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQta8ad#X#X#s#0a..Ra9#NaW.4.d#TQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQt#zb.#X#7#M#9.RaVb#.j.0apQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt#zba#7#A.R#m.PbbQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtbc#FbdQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt"
};


static const char* const p_icon_data[] = {
    "16 16 111 2",
    "  	c None",
    ". 	c #DC0202",
    "+ 	c #FF0A0A",
    "@ 	c #FF1010",
    "# 	c #FF1A1A",
    "$ 	c #FE2424",
    "% 	c #FF0808",
    "& 	c #FE0303",
    "* 	c #F50101",
    "= 	c #DF0000",
    "- 	c #BE0000",
    "; 	c #FC0505",
    "> 	c #FF0909",
    ", 	c #FF1D1D",
    "' 	c #FF2D2D",
    ") 	c #FF3B3B",
    "! 	c #FF2B2B",
    "~ 	c #FF1818",
    "{ 	c #FE0A0A",
    "] 	c #EE0303",
    "^ 	c #D40101",
    "/ 	c #A40000",
    "( 	c #FF0303",
    "_ 	c #FF0707",
    ": 	c #FF0F0F",
    "< 	c #FF3333",
    "[ 	c #FF4A4A",
    "} 	c #FF4E4E",
    "| 	c #FF3939",
    "1 	c #F30B0B",
    "2 	c #D10303",
    "3 	c #B50101",
    "4 	c #9F0000",
    "5 	c #8B0000",
    "6 	c #FF0202",
    "7 	c #FF0505",
    "8 	c #FF0B0B",
    "9 	c #FF2F2F",
    "0 	c #FF4B4B",
    "a 	c #FF5555",
    "b 	c #FF3E3E",
    "c 	c #FC1B1B",
    "d 	c #D80606",
    "e 	c #AE0000",
    "f 	c #940000",
    "g 	c #800000",
    "h 	c #6C0000",
    "i 	c #630000",
    "j 	c #FF0101",
    "k 	c #FF2020",
    "l 	c #FF3636",
    "m 	c #FF4040",
    "n 	c #FE2E2E",
    "o 	c #EF3B3C",
    "p 	c #D56F73",
    "q 	c #C07278",
    "r 	c #994043",
    "s 	c #660909",
    "t 	c #500000",
    "u 	c #FD0303",
    "v 	c #FB0707",
    "w 	c #F90F0F",
    "x 	c #F91A1A",
    "y 	c #F51D1C",
    "z 	c #EA4043",
    "A 	c #E9C9D7",
    "B 	c #EAF1FF",
    "C 	c #E8EEFF",
    "D 	c #E8E5F3",
    "E 	c #AD919B",
    "F 	c #711316",
    "G 	c #EE0101",
    "H 	c #D00505",
    "I 	c #C70808",
    "J 	c #B70707",
    "K 	c #C67C86",
    "L 	c #E1E9FF",
    "M 	c #D8DAFE",
    "N 	c #CACAFE",
    "O 	c #D9DAFF",
    "P 	c #DBDFFE",
    "Q 	c #8C7797",
    "R 	c #BA0000",
    "S 	c #A90101",
    "T 	c #930101",
    "U 	c #B18590",
    "V 	c #DFEBFF",
    "W 	c #C4C6FD",
    "X 	c #9694CC",
    "Y 	c #A8A7E2",
    "Z 	c #C0CDFF",
    "` 	c #A9BEEA",
    " .	c #7D0000",
    "..	c #5A0000",
    "+.	c #7F5057",
    "@.	c #D7DFF9",
    "#.	c #CACFFE",
    "$.	c #A5A3DD",
    "%.	c #A8A8E6",
    "&.	c #BAC9FF",
    "*.	c #B7CCFA",
    "=.	c #510000",
    "-.	c #670C0E",
    ";.	c #B194AB",
    ">.	c #E8ECFF",
    ",.	c #D7DAFF",
    "'.	c #C1CCFF",
    ").	c #C3D5FF",
    "!.	c #8E9098",
    "~.	c #C5CAE9",
    "{.	c #C5D4FA",
    "                                ",
    "                                ",
    "    . . . .         . . . .     ",
    "  . + @ # $ .     . % & * = -   ",
    ". ; > @ , ' ) ) ! ~ { ] ^ - / / ",
    ". ( _ : , < [ } | , 1 2 3 4 5 5 ",
    ". 6 7 8 ~ 9 0 a b c d e f g h i ",
    ". j ( _ : k l m n o p q r s t h ",
    "  . j u v w x y z A B C D E F   ",
    "    . G . H I J K L M N O P Q   ",
    "      . R S T g U V W X Y Z `   ",
    "        . S  ...+.@.#.$.%.&.*.  ",
    "          . ..=.-.;.>.,.'.).`   ",
    "            . =..   !.~.{.`     ",
    "              .                 ",
    "                                "
};



static const char* const t_open_icon_data[] = {
    "20 17 245 2",
    "  	c None",
    ". 	c #969CD7",
    "+ 	c #9198D4",
    "@ 	c #A1A7DC",
    "# 	c #BABEE6",
    "$ 	c #DFE1F7",
    "% 	c #C3C7EB",
    "& 	c #979CD7",
    "* 	c #AEB4E2",
    "= 	c #D8DAF4",
    "- 	c #F1F1FE",
    "; 	c #EAEAFB",
    "> 	c #DBDCF5",
    ", 	c #A2A8DE",
    "' 	c #3A75DD",
    ") 	c #A1CEF7",
    "! 	c #9CCBF7",
    "~ 	c #94C7F6",
    "{ 	c #8EC3F6",
    "] 	c #6FA5ED",
    "^ 	c #678BDB",
    "/ 	c #A7AEDF",
    "( 	c #CED0ED",
    "_ 	c #ECEDFB",
    ": 	c #F2F2FF",
    "< 	c #EEEEFD",
    "[ 	c #E9E9FB",
    "} 	c #E7E7FA",
    "| 	c #C6CAED",
    "1 	c #7F8BD2",
    "2 	c #4981E0",
    "3 	c #CBF4FF",
    "4 	c #C4F0FF",
    "5 	c #BBEBFF",
    "6 	c #ACD6F7",
    "7 	c #9FB9E9",
    "8 	c #B8C4EB",
    "9 	c #E1E4F6",
    "0 	c #F9F9FE",
    "a 	c #F8F8FF",
    "b 	c #E5E5F9",
    "c 	c #E1E2F8",
    "d 	c #A6ADE1",
    "e 	c #477FDF",
    "f 	c #C1EAFD",
    "g 	c #AACAF0",
    "h 	c #B2C0E8",
    "i 	c #DADEF3",
    "j 	c #F4F5FC",
    "k 	c #FEFEFF",
    "l 	c #FDFDFF",
    "m 	c #E6E7FA",
    "n 	c #BCC8F1",
    "o 	c #89A3E6",
    "p 	c #7994DF",
    "q 	c #4263CB",
    "r 	c #447DDF",
    "s 	c #BCE3F9",
    "t 	c #AEC1EB",
    "u 	c #DBDFF3",
    "v 	c #FCFDFE",
    "w 	c #FFFFFF",
    "x 	c #ECECFD",
    "y 	c #B2C2F0",
    "z 	c #4E7BDD",
    "A 	c #719CEC",
    "B 	c #7FA8F1",
    "C 	c #759FEE",
    "D 	c #6A96EC",
    "E 	c #6490EC",
    "F 	c #0932C1",
    "G 	c #417ADF",
    "H 	c #BCE8FE",
    "I 	c #85B1EE",
    "J 	c #6C92E0",
    "K 	c #89ADEF",
    "L 	c #87AAED",
    "M 	c #84A8EB",
    "N 	c #82A4E9",
    "O 	c #7FA0E8",
    "P 	c #7B9CE6",
    "Q 	c #6C8FE2",
    "R 	c #4273DB",
    "S 	c #71A2F2",
    "T 	c #64A6FE",
    "U 	c #4C97FE",
    "V 	c #4F9BFE",
    "W 	c #317CF4",
    "X 	c #103FD6",
    "Y 	c #0129BF",
    "Z 	c #3E78DE",
    "` 	c #A4D5F8",
    " .	c #5991E8",
    "..	c #8EB0ED",
    "+.	c #B9CFF7",
    "@.	c #BBD7FB",
    "#.	c #B0D2FA",
    "$.	c #A5CBFA",
    "%.	c #98C1F9",
    "&.	c #8BB9F7",
    "*.	c #83B2F8",
    "=.	c #82B4FC",
    "-.	c #6AABFE",
    ";.	c #4693FE",
    ">.	c #428EFC",
    ",.	c #397EF5",
    "'.	c #1343D9",
    ").	c #0128C3",
    "!.	c #3B76DD",
    "~.	c #C9F2FF",
    "{.	c #8EC1F3",
    "].	c #6493E5",
    "^.	c #E0EFFD",
    "/.	c #D8E9FC",
    "(.	c #DBF6FE",
    "_.	c #CBF0FF",
    ":.	c #B1DFFF",
    "<.	c #92C6FE",
    "[.	c #76B1FE",
    "}.	c #609EFD",
    "|.	c #4B8DFB",
    "1.	c #387AF5",
    "2.	c #2864EB",
    "3.	c #1645DA",
    "4.	c #0526CC",
    "5.	c #0019C8",
    "6.	c #0022BE",
    "7.	c #3773DD",
    "8.	c #BCE8FC",
    "9.	c #6EA4EE",
    "0.	c #93B7EF",
    "a.	c #E1F9FF",
    "b.	c #BEE1F9",
    "c.	c #B3DBFB",
    "d.	c #A2D2FF",
    "e.	c #80B6FF",
    "f.	c #659EFB",
    "g.	c #4C87F7",
    "h.	c #3371F3",
    "i.	c #1E5BEC",
    "j.	c #1247E1",
    "k.	c #0735D5",
    "l.	c #0027CD",
    "m.	c #001BC9",
    "n.	c #0019C5",
    "o.	c #0022BC",
    "p.	c #3571DC",
    "q.	c #ACDAF9",
    "r.	c #598EE5",
    "s.	c #C3DFFA",
    "t.	c #C9F0FF",
    "u.	c #A7DCFF",
    "v.	c #8BC2FD",
    "w.	c #83B9FD",
    "x.	c #74ACFE",
    "y.	c #5E98FB",
    "z.	c #4380F6",
    "A.	c #2866F0",
    "B.	c #0F4DE7",
    "C.	c #043BDD",
    "D.	c #0135D8",
    "E.	c #002BD2",
    "F.	c #001BCB",
    "G.	c #001BC1",
    "H.	c #2F6BDA",
    "I.	c #8CBEF4",
    "J.	c #6594E5",
    "K.	c #D9F4FF",
    "L.	c #BCE7FF",
    "M.	c #A9DBFF",
    "N.	c #93C6FD",
    "O.	c #87B6FB",
    "P.	c #7EAEFB",
    "Q.	c #6A9CF8",
    "R.	c #4F83F1",
    "S.	c #3368EA",
    "T.	c #2157E4",
    "U.	c #2051E0",
    "V.	c #1E48D8",
    "W.	c #1A3ED0",
    "X.	c #122EC9",
    "Y.	c #0421BD",
    "Z.	c #2563D8",
    "`.	c #6096E9",
    " +	c #A3C0F2",
    ".+	c #EBF8FF",
    "++	c #E5F5FF",
    "@+	c #DDEFFF",
    "#+	c #D2E4FE",
    "$+	c #C7D8FA",
    "%+	c #BFD1F8",
    "&+	c #B4C5F4",
    "*+	c #A3B5EE",
    "=+	c #95A8E9",
    "-+	c #8DA0E6",
    ";+	c #8999E0",
    ">+	c #8390DA",
    ",+	c #7C89D5",
    "'+	c #576BCC",
    ")+	c #1633BC",
    "!+	c #1858D4",
    "~+	c #487CDD",
    "{+	c #EAF0FD",
    "]+	c #F7F7FF",
    "^+	c #F0F0FC",
    "/+	c #E9E9F8",
    "(+	c #E2E2F4",
    "_+	c #DADAF0",
    ":+	c #D3D3EC",
    "<+	c #CCCCE7",
    "[+	c #C5C5E4",
    "}+	c #BDBDDF",
    "|+	c #B7B7DC",
    "1+	c #667ACC",
    "2+	c #1B39BC",
    "3+	c #0C46CC",
    "4+	c #7399E6",
    "5+	c #FAFAFF",
    "6+	c #F2F2FC",
    "7+	c #E8E8F6",
    "8+	c #DFDFF1",
    "9+	c #D5D5EA",
    "0+	c #CBCCE5",
    "a+	c #C2C3DF",
    "b+	c #BABBD9",
    "c+	c #B2B3D4",
    "d+	c #ACADD1",
    "e+	c #435CC2",
    "f+	c #053FC9",
    "g+	c #3A6ED9",
    "h+	c #5B85DE",
    "i+	c #5A84DD",
    "j+	c #5982DD",
    "k+	c #5880DC",
    "l+	c #557DDA",
    "m+	c #5178D8",
    "n+	c #4B72D3",
    "o+	c #476DD0",
    "p+	c #4267CB",
    "q+	c #3C61C7",
    "r+	c #375BC4",
    "s+	c #3356C0",
    "t+	c #2F51BD",
    "u+	c #284BBB",
    "v+	c #0F2FB7",
    "                        . +             ",
    "                    @ # $ %             ",
    "                & * = - ; > ,           ",
    "' ) ! ~ { ] ^ / ( _ : < [ } | 1         ",
    "2 3 4 5 6 7 8 9 0 a : < [ b c d         ",
    "e 3 f g h i j k l a : < m n o p q       ",
    "r 3 s t u v w w l a : x y z A B C D E F ",
    "G 3 H I J K L M N O P Q R S T U V W X Y ",
    "Z 3 `  ...+.@.#.$.%.&.*.=.-.;.>.,.'.).  ",
    "!.~.{.].^./.(._.:.<.[.}.|.1.2.3.4.5.6.  ",
    "7.8.9.0.a.b.c.d.e.f.g.h.i.j.k.l.m.n.o.  ",
    "p.q.r.s.t.u.v.w.x.y.z.A.B.C.D.E.F.G.    ",
    "H.I.J.K.L.M.N.O.P.Q.R.S.T.U.V.W.X.Y.    ",
    "Z.`. +.+++@+#+$+%+&+*+=+-+;+>+,+'+)+    ",
    "!+~+{+w w l ]+^+/+(+_+:+<+[+}+|+1+2+    ",
    "3+4+w w w w 5+6+7+8+9+0+a+b+c+d+e+      ",
    "f+g+h+i+j+k+l+m+n+o+p+q+r+s+t+u+v+      "
};

static const char* const t_save_icon_data[] = {
    "18 17 257 2",
    "  	c None",
    ". 	c #BBBCCE",
    "+ 	c #B6B7CB",
    "@ 	c #BBBBCD",
    "# 	c #ADAEC3",
    "$ 	c #A5A7BE",
    "% 	c #A2A3BB",
    "& 	c #A7A7BE",
    "* 	c #A3A3BB",
    "= 	c #9D9DB6",
    "- 	c #9999B3",
    "; 	c #9494AF",
    "> 	c #8F8FAC",
    ", 	c #8A8AA8",
    "' 	c #8585A4",
    ") 	c #64688F",
    "! 	c #4F527F",
    "~ 	c #E1E5EC",
    "{ 	c #E1E5ED",
    "] 	c #FFFFFF",
    "^ 	c #7F90AF",
    "/ 	c #7082A6",
    "( 	c #7E8FAF",
    "_ 	c #B3BDD0",
    ": 	c #868EAC",
    "< 	c #37386C",
    "[ 	c #B4B5C9",
    "} 	c #D7DDE6",
    "| 	c #D8DDE7",
    "1 	c #465E8C",
    "2 	c #576C96",
    "3 	c #7486A8",
    "4 	c #A3AFC6",
    "5 	c #848EAC",
    "6 	c #353669",
    "7 	c #AEB0C5",
    "8 	c #D2D8E3",
    "9 	c #D3D9E4",
    "0 	c #586D97",
    "a 	c #7F8FAF",
    "b 	c #9AA7C0",
    "c 	c #9DA9C2",
    "d 	c #757FA0",
    "e 	c #2E2F63",
    "f 	c #A8AAC0",
    "g 	c #CCD3DF",
    "h 	c #CED4E1",
    "i 	c #9FACC4",
    "j 	c #B9C2D3",
    "k 	c #CBD2DF",
    "l 	c #FDFDFE",
    "m 	c #8E9BB5",
    "n 	c #666F92",
    "o 	c #2A2B60",
    "p 	c #A1A4BC",
    "q 	c #C6CEDC",
    "r 	c #C5CDDB",
    "s 	c #EEF1F5",
    "t 	c #EDF0F4",
    "u 	c #ECEFF3",
    "v 	c #EBEEF3",
    "w 	c #EAEDF2",
    "x 	c #E9ECF1",
    "y 	c #E8EBF0",
    "z 	c #E7EAF0",
    "A 	c #E6E9EF",
    "B 	c #E5E8EF",
    "C 	c #E1E4EB",
    "D 	c #D0D3DE",
    "E 	c #7884A3",
    "F 	c #5C668A",
    "G 	c #26275C",
    "H 	c #9B9EB8",
    "I 	c #C1C9D9",
    "J 	c #B7C1D2",
    "K 	c #BAC3D4",
    "L 	c #B5BFD1",
    "M 	c #B1BBCE",
    "N 	c #ADB8CC",
    "O 	c #A8B4C9",
    "P 	c #A4B0C7",
    "Q 	c #A0ACC4",
    "R 	c #9BA9C1",
    "S 	c #95A3BD",
    "T 	c #8D9AB6",
    "U 	c #7F8BA9",
    "V 	c #7480A0",
    "W 	c #626F93",
    "X 	c #545E84",
    "Y 	c #222359",
    "Z 	c #9498B3",
    "` 	c #BBC4D4",
    " .	c #AEB9CD",
    "..	c #AAB5CA",
    "+.	c #A5B1C7",
    "@.	c #9BA8C1",
    "#.	c #909EB9",
    "$.	c #8897B4",
    "%.	c #808FAE",
    "&.	c #7382A2",
    "*.	c #6A789A",
    "=.	c #637194",
    "-.	c #5C6A8F",
    ";.	c #566489",
    ">.	c #4C567C",
    ",.	c #1F2056",
    "'.	c #8F92AF",
    ").	c #ACB7CB",
    "!.	c #B4BED0",
    "~.	c #B0BBCE",
    "{.	c #A6B1C7",
    "].	c #9CA7BF",
    "^.	c #939FB9",
    "/.	c #8A96B1",
    "(.	c #838EAB",
    "_.	c #7B87A5",
    ":.	c #7581A0",
    "<.	c #6F7A9B",
    "[.	c #687496",
    "}.	c #526186",
    "|.	c #454F76",
    "1.	c #1C1D54",
    "2.	c #888CAA",
    "3.	c #AFBACD",
    "4.	c #FDFDFD",
    "5.	c #F4F4F7",
    "6.	c #EBEBF0",
    "7.	c #E4E4EB",
    "8.	c #DFDFE7",
    "9.	c #D9D9E3",
    "0.	c #D4D4DF",
    "a.	c #CECEDA",
    "b.	c #C8C8D6",
    "c.	c #C2C2D2",
    "d.	c #BCBCCE",
    "e.	c #5C678B",
    "f.	c #3D466F",
    "g.	c #181950",
    "h.	c #8286A6",
    "i.	c #FBFBFC",
    "j.	c #F2F2F5",
    "k.	c #E9E9EF",
    "l.	c #DDDDE6",
    "m.	c #D8D8E2",
    "n.	c #D2D2DE",
    "o.	c #CDCDDA",
    "p.	c #C7C7D6",
    "q.	c #BCBCCD",
    "r.	c #B6B6C9",
    "s.	c #556185",
    "t.	c #37406A",
    "u.	c #15164D",
    "v.	c #7B7FA1",
    "w.	c #A6B0C6",
    "x.	c #DCDCE9",
    "y.	c #C4C4DB",
    "z.	c #BFBFD7",
    "A.	c #BBBBD4",
    "B.	c #B6B6CF",
    "C.	c #B1B1CC",
    "D.	c #ACACC7",
    "E.	c #A8A8C4",
    "F.	c #A2A2C0",
    "G.	c #9E9EBC",
    "H.	c #9999B8",
    "I.	c #A1A1BC",
    "J.	c #4E597F",
    "K.	c #303A63",
    "L.	c #12134A",
    "M.	c #757A9D",
    "N.	c #99A4BD",
    "O.	c #DDDDE8",
    "P.	c #CFCFDF",
    "Q.	c #C9C9DB",
    "R.	c #C4C4D7",
    "S.	c #BFBFD3",
    "T.	c #BABACF",
    "U.	c #B4B4CB",
    "V.	c #B0B0C7",
    "W.	c #AAAAC3",
    "X.	c #A5A5BF",
    "Y.	c #9F9FBB",
    "Z.	c #A1A1BA",
    "`.	c #485379",
    " +	c #2A345E",
    ".+	c #101148",
    "++	c #6E7398",
    "@+	c #919EB8",
    "#+	c #8E9AB4",
    "$+	c #D1D1E0",
    "%+	c #BFBFD5",
    "&+	c #BBBBD1",
    "*+	c #B6B6CD",
    "=+	c #B0B0C9",
    "-+	c #ACACC6",
    ";+	c #A6A6C1",
    ">+	c #A2A2BE",
    ",+	c #9C9CB9",
    "'+	c #9797B6",
    ")+	c #9292B2",
    "!+	c #9797B3",
    "~+	c #414D73",
    "{+	c #242E58",
    "]+	c #0D0E45",
    "^+	c #666A91",
    "/+	c #8693AF",
    "(+	c #8591AD",
    "_+	c #CCCCDC",
    ":+	c #BDBDD2",
    "<+	c #B8B8CE",
    "[+	c #B3B3CA",
    "}+	c #ADADC6",
    "|+	c #A9A9C3",
    "1+	c #A3A3BE",
    "2+	c #9E9EBB",
    "3+	c #9999B6",
    "4+	c #9595B2",
    "5+	c #8F8FAF",
    "6+	c #9292AF",
    "7+	c #3B476E",
    "8+	c #1F2952",
    "9+	c #0B0C43",
    "0+	c #5D628A",
    "a+	c #7D8AA8",
    "b+	c #7D88A7",
    "c+	c #C8C8D8",
    "d+	c #BCBCD0",
    "e+	c #B6B6CC",
    "f+	c #B1B1C8",
    "g+	c #ACACC4",
    "h+	c #A7A7C0",
    "i+	c #9C9CB8",
    "j+	c #9797B4",
    "k+	c #9292B0",
    "l+	c #8D8DAC",
    "m+	c #8D8DAB",
    "n+	c #364168",
    "o+	c #1A234D",
    "p+	c #090A40",
    "q+	c #545782",
    "r+	c #727A9B",
    "s+	c #6F7797",
    "t+	c #BCBCC9",
    "u+	c #B6B6C5",
    "v+	c #B1B1C1",
    "w+	c #ACACBD",
    "x+	c #A6A6B9",
    "y+	c #A1A1B6",
    "z+	c #9C9CB1",
    "A+	c #9797AE",
    "B+	c #9292A9",
    "C+	c #8D8DA6",
    "D+	c #8787A2",
    "E+	c #82829E",
    "F+	c #2B345A",
    "G+	c #131942",
    "H+	c #07073E",
    "  . + @ # $ % & * = - ; > , ' ) !   ",
    ". ~ { ] ^ / ( ] ] ] ] ] ] ] ] _ : < ",
    "[ } | ] 1 2 3 ] ] ] ] ] ] ] ] 4 5 6 ",
    "7 8 9 ] 0 a b ] ] ] ] ] ] ] ] c d e ",
    "f g h ] i j k ] ] ] ] ] ] ] l m n o ",
    "p q r s t u v w x y z A B C D E F G ",
    "H I J K L M N O P Q R S T U V W X Y ",
    "Z `  ...+.Q @.S #.$.%.&.*.=.-.;.>.,.",
    "'.L ).!.~.).{.].^./.(._.:.<.[.}.|.1.",
    "2.3.!.] 4.5.6.7.8.9.0.a.b.c.d.e.f.g.",
    "h...3.i.j.k.7.l.m.n.o.p.c.q.r.s.t.u.",
    "v.P w.x.y.z.A.B.C.D.E.F.G.H.I.J.K.L.",
    "M.c N.O.P.Q.R.S.T.U.V.W.X.Y.Z.`. +.+",
    "++@+#+$+%+&+*+=+-+;+>+,+'+)+!+~+{+]+",
    "^+/+(+_+:+<+[+}+|+1+2+3+4+5+6+7+8+9+",
    "0+a+b+c+d+e+f+g+h+I.i+j+k+l+m+n+o+p+",
    "q+r+s+t+u+v+w+x+y+z+A+B+C+D+E+F+G+H+"
};

static const char* const t_saveas_icon_data[] = {
    "18 20 314 2",
    "  	c None",
    ". 	c #EDBB7B",
    "+ 	c #DE9B6E",
    "@ 	c #D17B47",
    "# 	c #F9D004",
    "$ 	c #FFE45F",
    "% 	c #FEE8C3",
    "& 	c #EFC7A8",
    "* 	c #D07843",
    "= 	c #BABCCD",
    "- 	c #B6B7CB",
    "; 	c #BBBBCD",
    "> 	c #ACADC2",
    ", 	c #A5A7BE",
    "' 	c #A2A3BB",
    ") 	c #A7A7BE",
    "! 	c #A3A3BA",
    "~ 	c #9D9DB6",
    "{ 	c #9999B3",
    "] 	c #C4A248",
    "^ 	c #FDDF01",
    "/ 	c #FFC704",
    "( 	c #FFBA1A",
    "_ 	c #F8A648",
    ": 	c #BD6D39",
    "< 	c #BDBFD0",
    "[ 	c #E1E4EC",
    "} 	c #E3E7EE",
    "| 	c #FFFFFF",
    "1 	c #7688AA",
    "2 	c #7082A6",
    "3 	c #8797B4",
    "4 	c #FEFAF3",
    "5 	c #F8CE1E",
    "6 	c #FFDD03",
    "7 	c #FFBF04",
    "8 	c #FFAB18",
    "9 	c #E68830",
    "0 	c #89727C",
    "a 	c #323268",
    "b 	c #B6B8CB",
    "c 	c #D7DCE6",
    "d 	c #DBE0E8",
    "e 	c #3B5585",
    "f 	c #586D97",
    "g 	c #7E8FAE",
    "h 	c #FFFFFE",
    "i 	c #FAD993",
    "j 	c #FEE205",
    "k 	c #FFCF0D",
    "l 	c #FFBA13",
    "m 	c #FD9E25",
    "n 	c #BD7E53",
    "o 	c #838BA8",
    "p 	c #303065",
    "q 	c #B0B3C7",
    "r 	c #D1D7E3",
    "s 	c #D6DCE6",
    "t 	c #4F6691",
    "u 	c #8090B0",
    "v 	c #A1AEC5",
    "w 	c #FDF4E4",
    "x 	c #FAD849",
    "y 	c #FFE115",
    "z 	c #FFC81C",
    "A 	c #FFB225",
    "B 	c #EE8A2B",
    "C 	c #97878A",
    "D 	c #747D9E",
    "E 	c #2A2B60",
    "F 	c #AAADC3",
    "G 	c #CBD2DF",
    "H 	c #D2D8E3",
    "I 	c #9BA8C1",
    "J 	c #B9C2D4",
    "K 	c #CFD5E1",
    "L 	c #FFFEFE",
    "M 	c #FBE3A4",
    "N 	c #FCDF28",
    "O 	c #FFD527",
    "P 	c #FFC62A",
    "Q 	c #FCA638",
    "R 	c #EAA569",
    "S 	c #8289A2",
    "T 	c #656E91",
    "U 	c #26275D",
    "V 	c #A4A7BE",
    "W 	c #C5CDDB",
    "X 	c #C8D0DD",
    "Y 	c #EEF1F5",
    "Z 	c #EDF0F4",
    "` 	c #ECEFF3",
    " .	c #EBEEF2",
    "..	c #EAEDF2",
    "+.	c #EAEAE9",
    "@.	c #F9D658",
    "#.	c #FFE734",
    "$.	c #FFCD37",
    "%.	c #FFBF3E",
    "&.	c #EF9B44",
    "*.	c #C7A696",
    "=.	c #727E9F",
    "-.	c #5C6489",
    ";.	c #22235A",
    ">.	c #9EA1BA",
    ",.	c #C0C8D8",
    "'.	c #B7C1D3",
    ").	c #B9C3D4",
    "!.	c #B5BFD1",
    "~.	c #B1BBCE",
    "{.	c #ACB7CC",
    "].	c #A8B4C9",
    "^.	c #BDB297",
    "/.	c #FBDE3D",
    "(.	c #FFDE45",
    "_.	c #FFCD44",
    ":.	c #FFBA4F",
    "<.	c #C4804A",
    "[.	c #767992",
    "}.	c #606E92",
    "|.	c #545D82",
    "1.	c #1F2057",
    "2.	c #989BB5",
    "3.	c #BAC3D4",
    "4.	c #AEB9CD",
    "5.	c #A9B5CA",
    "6.	c #A4B0C7",
    "7.	c #9FACC4",
    "8.	c #9AA8C1",
    "9.	c #99A3B6",
    "0.	c #D7BB5F",
    "a.	c #FFEC57",
    "b.	c #FFD852",
    "c.	c #FFCA56",
    "d.	c #F5A953",
    "e.	c #846968",
    "f.	c #5C6A8F",
    "g.	c #566489",
    "h.	c #4B547B",
    "i.	c #1C1D54",
    "j.	c #9296B2",
    "k.	c #B4BED1",
    "l.	c #B4BED0",
    "m.	c #B0BACE",
    "n.	c #ACB7CB",
    "o.	c #A5B0C7",
    "p.	c #B3A892",
    "q.	c #F2DA5E",
    "r.	c #FFE663",
    "s.	c #FFD85D",
    "t.	c #FFC669",
    "u.	c #D38C50",
    "v.	c #757790",
    "w.	c #687495",
    "x.	c #515F85",
    "y.	c #444D75",
    "z.	c #191A52",
    "A.	c #8B8FAD",
    "B.	c #AEB9CC",
    "C.	c #FCFCFD",
    "D.	c #F4F4F7",
    "E.	c #EBE8E8",
    "F.	c #F3D98C",
    "G.	c #FEEC73",
    "H.	c #FFDF6D",
    "I.	c #FFD76E",
    "J.	c #F4B56B",
    "K.	c #CEA994",
    "L.	c #C2C1D1",
    "M.	c #BCBCCE",
    "N.	c #535F84",
    "O.	c #3A436C",
    "P.	c #14154C",
    "Q.	c #868AA9",
    "R.	c #FBFBFC",
    "S.	c #F1F1F4",
    "T.	c #E9E9EF",
    "U.	c #E6E0D8",
    "V.	c #FADA6B",
    "W.	c #FFED83",
    "X.	c #FFDF77",
    "Y.	c #FFD580",
    "Z.	c #E1A46C",
    "`.	c #C1AEB1",
    " +	c #B9B9CB",
    ".+	c #AEAEC2",
    "++	c #48547A",
    "@+	c #313963",
    "#+	c #111249",
    "$+	c #7F83A4",
    "%+	c #A3AFC6",
    "&+	c #ABB5CA",
    "*+	c #DADAE8",
    "=+	c #C4C4DB",
    "-+	c #BFBFD7",
    ";+	c #C2BBC6",
    ">+	c #F9D583",
    ",+	c #FFE896",
    "'+	c #FFE089",
    ")+	c #FCD18B",
    "!+	c #B48777",
    "~+	c #9493B3",
    "{+	c #8B8BAB",
    "]+	c #9393AF",
    "^+	c #3F4B72",
    "/+	c #2B335D",
    "(+	c #0F0F47",
    "_+	c #797EA0",
    ":+	c #9BA8C0",
    "<+	c #9FA9C0",
    "[+	c #DCDCE7",
    "}+	c #CECEDF",
    "|+	c #C9C9DA",
    "1+	c #CDBDB8",
    "2+	c #F7D3A6",
    "3+	c #FFEECD",
    "4+	c #FCE3B4",
    "5+	c #DBA172",
    "6+	c #9C8E9F",
    "7+	c #9494B1",
    "8+	c #9090AD",
    "9+	c #9292AD",
    "0+	c #3A456D",
    "a+	c #262D58",
    "b+	c #0D0D45",
    "c+	c #72779B",
    "d+	c #8E9CB6",
    "e+	c #949FB8",
    "f+	c #D0D0DF",
    "g+	c #BFBFD5",
    "h+	c #BABAD1",
    "i+	c #C1AAA3",
    "j+	c #EBCFB3",
    "k+	c #F8E4CE",
    "l+	c #D5AB93",
    "m+	c #9E8892",
    "n+	c #8C8CAC",
    "o+	c #8888A8",
    "p+	c #8484A5",
    "q+	c #8989A7",
    "r+	c #343F68",
    "s+	c #202953",
    "t+	c #0B0B43",
    "u+	c #696E94",
    "v+	c #8491AD",
    "w+	c #8C96B1",
    "x+	c #CBCBDB",
    "y+	c #BCBCD2",
    "z+	c #B8B8CE",
    "A+	c #B59E9D",
    "B+	c #B29080",
    "C+	c #BC9383",
    "D+	c #988593",
    "E+	c #8F8FAD",
    "F+	c #8A8AA9",
    "G+	c #8686A5",
    "H+	c #8181A2",
    "I+	c #8585A4",
    "J+	c #313C64",
    "K+	c #1E264F",
    "L+	c #0A0B42",
    "M+	c #60658D",
    "N+	c #7B88A7",
    "O+	c #848EAB",
    "P+	c #C7C7D7",
    "Q+	c #BBBBD0",
    "R+	c #B6B6CC",
    "S+	c #B39899",
    "T+	c #AA7561",
    "U+	c #9D8D9B",
    "V+	c #9292AF",
    "W+	c #8E8EAC",
    "X+	c #8888A7",
    "Y+	c #8585A5",
    "Z+	c #8B8BA9",
    "`+	c #2E3A62",
    " @	c #19224C",
    ".@	c #080940",
    "+@	c #575B85",
    "@@	c #6F7899",
    "#@	c #757C9B",
    "$@	c #BCBCC9",
    "%@	c #B6B6C5",
    "&@	c #B1B1C1",
    "*@	c #ABA3AF",
    "=@	c #A399A6",
    "-@	c #9C9CB1",
    ";@	c #9797AD",
    ">@	c #9292A9",
    ",@	c #8C8CA4",
    "'@	c #8B8BA4",
    ")@	c #8787A1",
    "!@	c #82829E",
    "~@	c #252D54",
    "{@	c #121841",
    "]@	c #06063E",
    "^@	c #424373",
    "/@	c #37386A",
    "(@	c #323365",
    "_@	c #4A4A74",
    ":@	c #484873",
    "<@	c #464671",
    "[@	c #43436F",
    "}@	c #41416E",
    "|@	c #3F3F6C",
    "1@	c #3D3D6B",
    "2@	c #3B3B69",
    "3@	c #393967",
    "4@	c #373766",
    "5@	c #353564",
    "6@	c #333363",
    "7@	c #0D0E43",
    "8@	c #06073D",
    "9@	c #04043E",
    "                          . + @     ",
    "                        # $ % & *   ",
    "  = - ; > , ' ) ! ~ { ] ^ / ( _ :   ",
    "< [ } | 1 2 3 | | | 4 5 6 7 8 9 0 a ",
    "b c d | e f g | | h i j k l m n o p ",
    "q r s | t u v | | w x y z A B C D E ",
    "F G H | I J K | L M N O P Q R S T U ",
    "V W X Y Z `  ...+.@.#.$.%.&.*.=.-.;.",
    ">.,.'.).!.~.{.].^./.(._.:.<.[.}.|.1.",
    "2.3.4.5.6.7.8.9.0.a.b.c.d.e.f.g.h.i.",
    "j.k.{.l.m.n.o.p.q.r.s.t.u.v.w.x.y.z.",
    "A.B.).| C.D.E.F.G.H.I.J.K.L.M.N.O.P.",
    "Q.].!.R.S.T.U.V.W.X.Y.Z.`. +.+++@+#+",
    "$+%+&+*+=+-+;+>+,+'+)+!+~+{+]+^+/+(+",
    "_+:+<+[+}+|+1+2+3+4+5+6+7+8+9+0+a+b+",
    "c+d+e+f+g+h+i+j+k+l+m+n+o+p+q+r+s+t+",
    "u+v+w+x+y+z+A+B+C+D+E+F+G+H+I+J+K+L+",
    "M+N+O+P+Q+R+S+T+U+V+W+X+G+Y+Z+`+ @.@",
    "+@@@#@$@%@&@*@=@-@;@>@,@'@)@!@~@{@]@",
    "^@/@(@_@:@<@[@}@|@1@2@3@4@5@6@7@8@9@"
};


static const char* const t_close_icon_data[] = {
    "16 16 76 2",
    "Qt c None",
    ".# c #292c29",
    ".v c #313031",
    ".i c #313431",
    "#h c #393839",
    "#j c #181c18",
    "#g c #201c20",
    "#f c #202020",
    "#i c #202420",
    ".w c #292429",
    ".h c #292829",
    "#. c #292c29",
    ".a c #312c31",
    ".5 c #313031",
    ".2 c #313431",
    ".V c #393839",
    ".Q c #393c39",
    ".u c #414041",
    ".x c #4a484a",
    ".g c #4a4c4a",
    ".7 c #524c52",
    ".f c #525052",
    ".e c #525552",
    ".y c #5a555a",
    ".d c #5a595a",
    ".H c #5a5d5a",
    ".c c #625d62",
    ".z c #626162",
    ".b c #626562",
    ".A c #6a656a",
    ".I c #6a696a",
    ".W c #6a6d6a",
    ".B c #736d73",
    ".k c #737173",
    ".C c #737573",
    ".J c #7b797b",
    ".l c #7b7d7b",
    ".D c #837d83",
    ".E c #838183",
    ".n c #838583",
    ".m c #8b858b",
    ".F c #8b898b",
    ".o c #8b8d8b",
    ".6 c #948d94",
    ".p c #949194",
    ".q c #949594",
    ".O c #9c959c",
    ".r c #9c999c",
    "#c c #9c9d9c",
    ".s c #a4a1a4",
    ".0 c #a4a5a4",
    ".K c #acaaac",
    ".3 c #acaeac",
    ".P c #b4aeb4",
    ".1 c #b4b2b4",
    ".4 c #b4b6b4",
    ".G c #bdb6bd",
    ".9 c #bdbebd",
    ".t c #c5bec5",
    ".8 c #c5c2c5",
    "#d c #c5c6c5",
    "#b c #cdc6cd",
    "#a c #cdcacd",
    ".L c #cdcecd",
    "#e c #d5ced5",
    ".N c #d5d6d5",
    ".R c #ded6de",
    ".j c #dedade",
    ".M c #dedede",
    ".U c #e6dee6",
    ".X c #e6e2e6",
    ".T c #e6e6e6",
    ".Y c #eee6ee",
    ".Z c #eeeaee",
    "## c #eeeeee",
    ".S c #ffffff",
    "Qt.#.a.b.c.d.d.e.f.f.f.g.f.h.iQt",
    ".#.c.j.k.l.m.n.o.o.p.q.r.s.t.u.v",
    ".a.j.w.x.y.d.z.A.B.C.D.E.F.o.G.h",
    ".z.E.x.H.H.I.I.k.J.D.E.F.o.q.K.u",
    ".c.l.y.z.j.L.C.J.E.E.M.N.O.r.P.Q",
    ".d.m.e.I.R.S.M.D.E.T.S.U.O.s.K.V",
    ".d.n.z.W.C.M.S.X.Y.S.Z.r.s.0.1.2",
    ".e.o.b.k.J.D.U.S.S.Y.O.s.s.3.4.5",
    ".f.6.B.J.E.E.Y.S.S.Z.s.s.K.4.t.a",
    ".7.p.k.D.E.T.S.Y.Y.S.Z.K.1.8.9#.",
    ".f.q.D.E.M.S.Z.r.s##.S##.9#a#b.w",
    ".x.r.D.F.T.U.O.s.s.K.Z.Z.8.N#a.w",
    ".f#c.F.p.O.r.s.s.K.1.9#d#e.M#a#f",
    ".h.t.F.q.r.s.0.3.4.8#a.N.j.S.J.h",
    ".i.u.G.K.P.K.1.4.t.8#b#b.U.J#g#h",
    "Qt.v.w.u.V.V.2.5#.#.#i.w#j.h#hQt"
};

static const char* const t_add_icon_data[] = {
    "20 18 157 2",
    "  	c None",
    ". 	c #000000",
    "+ 	c #C5C5C5",
    "@ 	c #8A9691",
    "# 	c #548987",
    "$ 	c #91B098",
    "% 	c #B9B8B9",
    "& 	c #7E7D7F",
    "* 	c #7C7C7C",
    "= 	c #808D87",
    "- 	c #628685",
    "; 	c #C7D5AF",
    "> 	c #E3E5A7",
    ", 	c #D6D6AF",
    "' 	c #BAB8B4",
    ") 	c #C3C2C5",
    "! 	c #C8C6CA",
    "~ 	c #BFBFBF",
    "{ 	c #C9C3C5",
    "] 	c #C0CECC",
    "^ 	c #98D3CE",
    "/ 	c #69D1CD",
    "( 	c #58BCBB",
    "_ 	c #639993",
    ": 	c #698B7C",
    "< 	c #A6A570",
    "[ 	c #8D9571",
    "} 	c #9B9E96",
    "| 	c #B2B6AF",
    "1 	c #A3A39F",
    "2 	c #908C8B",
    "3 	c #848181",
    "4 	c #848486",
    "5 	c #A5A2A7",
    "6 	c #D2CECD",
    "7 	c #BFBEBC",
    "8 	c #ADACAC",
    "9 	c #A6A8A7",
    "0 	c #A3A9A8",
    "a 	c #A2ADAC",
    "b 	c #96A3A2",
    "c 	c #75807B",
    "d 	c #525A54",
    "e 	c #1F2019",
    "f 	c #141511",
    "g 	c #4F4D47",
    "h 	c #7E7670",
    "i 	c #B1ADAB",
    "j 	c #C9C6C6",
    "k 	c #CDCBCB",
    "l 	c #C5C4C4",
    "m 	c #AFAEAD",
    "n 	c #A8A8A8",
    "o 	c #AEAEAE",
    "p 	c #D1CDCE",
    "q 	c #D1CCCE",
    "r 	c #C5C2C3",
    "s 	c #BBB9B9",
    "t 	c #B2B7B2",
    "u 	c #A7B2AB",
    "v 	c #999E98",
    "w 	c #7D7C79",
    "x 	c #676460",
    "y 	c #5C6661",
    "z 	c #4D877E",
    "A 	c #6BABAA",
    "B 	c #A8CFD1",
    "C 	c #C8DFDD",
    "D 	c #D2DFDD",
    "E 	c #DCDDDF",
    "F 	c #D3D2D2",
    "G 	c #9C9D9C",
    "H 	c #AEAEAC",
    "I 	c #BFBFBC",
    "J 	c #C0BFBE",
    "K 	c #B3B2B3",
    "L 	c #A7A7A7",
    "M 	c #A7A9A7",
    "N 	c #B0B4B2",
    "O 	c #B7B8B7",
    "P 	c #B5B1A9",
    "Q 	c #C9BE92",
    "R 	c #BACBA5",
    "S 	c #6CC0BE",
    "T 	c #44B2B9",
    "U 	c #50C6C9",
    "V 	c #77DEDB",
    "W 	c #9FEBE4",
    "X 	c #C1EAE2",
    "Y 	c #CAD0CB",
    "Z 	c #919390",
    "` 	c #ADABAB",
    " .	c #A2A3A0",
    "..	c #A0A19F",
    "+.	c #A0A1A1",
    "@.	c #A1A1A1",
    "#.	c #A8A8A6",
    "$.	c #B8B6B3",
    "%.	c #CAC3BA",
    "&.	c #DBD1AC",
    "*.	c #EBE297",
    "=.	c #EAEFB0",
    "-.	c #96D6C8",
    ";.	c #41B7B2",
    ">.	c #1DB3B3",
    ",.	c #2BBCBD",
    "'.	c #51CCC9",
    ").	c #7AD0CA",
    "!.	c #94B5B0",
    "~.	c #808481",
    "{.	c #949394",
    "].	c #919191",
    "^.	c #9F9F9F",
    "/.	c #ACACAC",
    "(.	c #B4B5B4",
    "_.	c #BEBDBA",
    ":.	c #CFC7B8",
    "<.	c #E4D7A5",
    "[.	c #E8E58B",
    "}.	c #F1EE96",
    "|.	c #BBDEB0",
    "1.	c #64CCB6",
    "2.	c #2EBABA",
    "3.	c #30ACB6",
    "4.	c #3CA4AC",
    "5.	c #479FA2",
    "6.	c #689794",
    "7.	c #6F7673",
    "8.	c #929293",
    "9.	c #989998",
    "0.	c #B5B5B5",
    "a.	c #C1C1BF",
    "b.	c #CBC9C1",
    "c.	c #D7D2B5",
    "d.	c #E6E085",
    "e.	c #E5E87E",
    "f.	c #D1E6A7",
    "g.	c #92DBC5",
    "h.	c #5DD5D2",
    "i.	c #72BFC9",
    "j.	c #709AA5",
    "k.	c #4C7B7E",
    "l.	c #7C8180",
    "m.	c #919392",
    "n.	c #A8A7A7",
    "o.	c #B6B5B2",
    "p.	c #BEBFB4",
    "q.	c #C1C0AD",
    "r.	c #C6BA85",
    "s.	c #C9C5B2",
    "t.	c #C3D0C6",
    "u.	c #9ECFC4",
    "v.	c #90AFAA",
    "w.	c #888A89",
    "x.	c #ABAAA9",
    "y.	c #949494",
    "z.	c #B0AFAE",
    "                                        ",
    "                                        ",
    "  . . . . .     . . . .       . . . .   ",
    "  . .     .     . .     .     . .     . ",
    "  . .     .     . .     .     . .     . ",
    "  . . . . .     . .     .     . .     . ",
    "  . .     .     . .     .     . .     . ",
    "  . . + @ . # $ . . . . % &   . . . .   ",
    "      * = - # $ ; > , ' % ) !           ",
    "  ~ { ] ^ / ( _ : < [ } | 1 2 3 4 5     ",
    "6 7 8 9 0 a b c d e f g h i j k l m n   ",
    "o p q r s t u v w x y z A B C D E F G   ",
    "H I J K L M N O P Q R S T U V W X Y Z   ",
    "`  ...+.@.#.$.%.&.*.=.-.;.>.,.'.).!.~.  ",
    "5 {.].^./.(._.:.<.[.}.|.1.2.3.4.5.6.7.  ",
    "  5 8.9.L 0.a.b.c.d.e.f.g.h.i.j.k.7.    ",
    "    5 l.m.n.o.p.q.r.r.s.t.u.v.w.7.      ",
    "        7 x.y.y.y.y.y.y.y.z.7           "
};



static const char* const t_rescan_icon_data[] = {
    "20 20 108 2",
    "  	c None",
    ". 	c #00D820",
    "+ 	c #8AE395",
    "@ 	c #7EAA83",
    "# 	c #A0B7A6",
    "$ 	c #3C7B82",
    "% 	c #7AA08E",
    "& 	c #C4D4B0",
    "* 	c #E3E5A7",
    "= 	c #D3D3AC",
    "- 	c #1DD338",
    "; 	c #21D43C",
    "> 	c #898484",
    ", 	c #B7DBB9",
    "' 	c #BDD2CE",
    ") 	c #7BD5CE",
    "! 	c #53CCCC",
    "~ 	c #639E9A",
    "{ 	c #62887E",
    "] 	c #A3A26E",
    "^ 	c #8C9470",
    "/ 	c #B5C3B3",
    "( 	c #B0BDAE",
    "_ 	c #7D7D7E",
    ": 	c #9F9CA1",
    "< 	c #25D23E",
    "[ 	c #B8C6B9",
    "} 	c #A2A2A2",
    "| 	c #A8A4A4",
    "1 	c #AEADAD",
    "2 	c #828681",
    "3 	c #575C57",
    "4 	c #0C0D0D",
    "5 	c #030303",
    "6 	c #4A4641",
    "7 	c #837872",
    "8 	c #CCCBCB",
    "9 	c #D7D3D2",
    "0 	c #D9D9D9",
    "a 	c #AEADAC",
    "b 	c #D2DDD1",
    "c 	c #D9D2D5",
    "d 	c #CBC8C9",
    "e 	c #BEBCBA",
    "f 	c #AABCB3",
    "g 	c #ACB3AC",
    "h 	c #8E8B88",
    "i 	c #86817C",
    "j 	c #7A8781",
    "k 	c #4AA198",
    "l 	c #6FC9CD",
    "m 	c #C0E3E6",
    "n 	c #C9E6E0",
    "o 	c #DCDFE2",
    "p 	c #EFEFEF",
    "q 	c #BAC4B7",
    "r 	c #AEAFAE",
    "s 	c #9C9C9C",
    "t 	c #949494",
    "u 	c #AEABAB",
    "v 	c #C2C0C3",
    "w 	c #CAC7C2",
    "x 	c #F2E39F",
    "y 	c #E1F5BB",
    "z 	c #72D2DC",
    "A 	c #1CA0AA",
    "B 	c #1DC9C6",
    "C 	c #6BEBE6",
    "D 	c #9EF8E8",
    "E 	c #CCD7CE",
    "F 	c #8D8D8D",
    "G 	c #939393",
    "H 	c #A7A7A8",
    "I 	c #AEB0AD",
    "J 	c #B4B7B4",
    "K 	c #C4BAB1",
    "L 	c #EAD89D",
    "M 	c #E5E28F",
    "N 	c #F4EAA5",
    "O 	c #A6D4BB",
    "P 	c #3CBFAC",
    "Q 	c #0FA2A8",
    "R 	c #0D9CA0",
    "S 	c #3BB8BB",
    "T 	c #8E8F8F",
    "U 	c #A5A5A5",
    "V 	c #BAC0BB",
    "W 	c #BEC5BF",
    "X 	c #39D34F",
    "Y 	c #DAD3BC",
    "Z 	c #EBE783",
    "` 	c #EDF67D",
    " .	c #CEEEA8",
    "..	c #6AD8CB",
    "+.	c #39D4DD",
    "@.	c #92BBCE",
    "#.	c #438B87",
    "$.	c #A7A7A9",
    "%.	c #7C9F82",
    "&.	c #C5C5B4",
    "*.	c #CEC083",
    "=.	c #D2CFBE",
    "-.	c #C9DDD2",
    ";.	c #89D1C3",
    ">.	c #949594",
    ",.	c #909090",
    "'.	c #9CA19C",
    ").	c #949995",
    "                      .                 ",
    "                      . . .             ",
    "                      . . . . .         ",
    "          . . . . . . . . . . . . .     ",
    "        . . . . . . . . . . . . .       ",
    "      . . . +         . . . . .         ",
    "    . . @ # $ % & * = . - ; >           ",
    "  . . , ' ) ! ~ { ] ^ . / ( > _ :       ",
    "  . < [ } | 1 2 3 4 5 6 7 8 9 0 a .     ",
    "  . b c d e f g h i j k l m n o p .     ",
    "  . q r s t u v w x y z A B C D E .     ",
    "  . F G H I J K L M N O P Q R S . .     ",
    "    : T U V W X Y Z `  ...+.@.#.. .     ",
    "      $.%.. . . &.*.*.=.-.;.>.. .       ",
    "      . . . . . ,.G ,.'.).. . .         ",
    "    . . . . . . . . . . . . .           ",
    "  . . . . . . . . . . . . .             ",
    "      . . . . .                         ",
    "          . . .                         ",
    "              .                         "
};

static const char* const t_delete_icon_data[] = {
    "18 18 194 2",
    "  	c None",
    ". 	c #DCDCDC",
    "+ 	c #DDDDDD",
    "@ 	c #DADADA",
    "# 	c #D3D3D3",
    "$ 	c #C4C4C4",
    "% 	c #C2C2C2",
    "& 	c #F50000",
    "* 	c #EB0000",
    "= 	c #F60D09",
    "- 	c #D9615D",
    "; 	c #D4C5C5",
    "> 	c #E8E8E8",
    ", 	c #E0E0E0",
    "' 	c #CCCCCC",
    ") 	c #C8C8C8",
    "! 	c #C6C6C6",
    "~ 	c #C5C5C5",
    "{ 	c #A89191",
    "] 	c #E71E1E",
    "^ 	c #FF0000",
    "/ 	c #E30000",
    "( 	c #FE0000",
    "_ 	c #FF0200",
    ": 	c #EC0300",
    "< 	c #C44949",
    "[ 	c #D4C2C2",
    "} 	c #D6D6D6",
    "| 	c #CBCBCB",
    "1 	c #BFBFBF",
    "2 	c #BDBDBD",
    "3 	c #B6B6B6",
    "4 	c #B79898",
    "5 	c #D43A3A",
    "6 	c #F60000",
    "7 	c #F20000",
    "8 	c #CE0000",
    "9 	c #F43939",
    "0 	c #FF0808",
    "a 	c #EC0303",
    "b 	c #CD8C8C",
    "c 	c #BABABA",
    "d 	c #B39797",
    "e 	c #D83B3B",
    "f 	c #F00000",
    "g 	c #C01717",
    "h 	c #D0B9B9",
    "i 	c #D13939",
    "j 	c #FF0404",
    "k 	c #E45040",
    "l 	c #DCC7C2",
    "m 	c #EFEFEF",
    "n 	c #BEBEBE",
    "o 	c #B9A2A2",
    "p 	c #D33939",
    "q 	c #F80000",
    "r 	c #F13F3F",
    "s 	c #D16565",
    "t 	c #877373",
    "u 	c #8D8D8D",
    "v 	c #9E8282",
    "w 	c #D03E33",
    "x 	c #FF0E00",
    "y 	c #FF0D00",
    "z 	c #C34642",
    "A 	c #EAE0E0",
    "B 	c #C0B2B2",
    "C 	c #D83939",
    "D 	c #C29999",
    "E 	c #ACACAC",
    "F 	c #929292",
    "G 	c #D7D7D7",
    "H 	c #B19892",
    "I 	c #BE322B",
    "J 	c #E90000",
    "K 	c #9C4141",
    "L 	c #D93030",
    "M 	c #F90000",
    "N 	c #FD0000",
    "O 	c #ED3A3A",
    "P 	c #BB9595",
    "Q 	c #9C9C9C",
    "R 	c #8B8B8B",
    "S 	c #CECECE",
    "T 	c #E2E2E2",
    "U 	c #D1D1D1",
    "V 	c #C0ADAD",
    "W 	c #C53636",
    "X 	c #E70D0D",
    "Y 	c #FA0000",
    "Z 	c #FC0505",
    "` 	c #D42D2D",
    " .	c #B29595",
    "..	c #969696",
    "+.	c #818181",
    "@.	c #787878",
    "#.	c #6C6C6C",
    "$.	c #B2B2B2",
    "%.	c #C7C7C7",
    "&.	c #D9D9D9",
    "*.	c #D4D4D4",
    "=.	c #E1D5D5",
    "-.	c #B92323",
    ";.	c #FF0101",
    ">.	c #FD0303",
    ",.	c #E51D1D",
    "'.	c #635454",
    ").	c #8E8E8E",
    "!.	c #A3A3A3",
    "~.	c #8F8F8F",
    "{.	c #A6A6A6",
    "].	c #979797",
    "^.	c #919191",
    "/.	c #999999",
    "(.	c #D01E1E",
    "_.	c #FD0404",
    ":.	c #E81E1E",
    "<.	c #615454",
    "[.	c #727272",
    "}.	c #C0C0C0",
    "|.	c #D5D5D5",
    "1.	c #DFDFDF",
    "2.	c #CDCDCD",
    "3.	c #C5A6A6",
    "4.	c #E01E1E",
    "5.	c #E51E1E",
    "6.	c #ED0E0E",
    "7.	c #F90606",
    "8.	c #BF2626",
    "9.	c #A68E8E",
    "0.	c #E4E4E4",
    "a.	c #CAA4A4",
    "b.	c #EB3333",
    "c.	c #EE2525",
    "d.	c #B62222",
    "e.	c #6C5E5E",
    "f.	c #963030",
    "g.	c #F20605",
    "h.	c #F40400",
    "i.	c #D85656",
    "j.	c #E6E6E6",
    "k.	c #C39A9A",
    "l.	c #E83535",
    "m.	c #EA2828",
    "n.	c #BD8F8F",
    "o.	c #B8B8B8",
    "p.	c #959595",
    "q.	c #807474",
    "r.	c #BC2F2D",
    "s.	c #F00800",
    "t.	c #EC2F2F",
    "u.	c #DFB1B1",
    "v.	c #BCBCBC",
    "w.	c #BE9898",
    "x.	c #E23131",
    "y.	c #E42828",
    "z.	c #B68383",
    "A.	c #B4B4B4",
    "B.	c #C3C3C3",
    "C.	c #EEEEEE",
    "D.	c #C9B6B6",
    "E.	c #B24646",
    "F.	c #D82B2B",
    "G.	c #E83636",
    "H.	c #D0A3A3",
    "I.	c #BFA0A0",
    "J.	c #E32B2B",
    "K.	c #E42424",
    "L.	c #AC7B7B",
    "M.	c #AEAEAE",
    "N.	c #F0F0F0",
    "O.	c #9F9F9F",
    "P.	c #8E6565",
    "Q.	c #D62929",
    "R.	c #C12A2A",
    "S.	c #C9AEAE",
    "T.	c #D31C1C",
    "U.	c #A87A7A",
    "V.	c #B1B1B1",
    "W.	c #E3E3E3",
    "X.	c #A1A1A1",
    "Y.	c #7E7E7E",
    "Z.	c #815B5B",
    "`.	c #C92929",
    " +	c #B02A2A",
    ".+	c #7A5353",
    "++	c #9A5F5F",
    "@+	c #B7B7B7",
    "#+	c #C9C9C9",
    "$+	c #A5A5A5",
    "%+	c #725454",
    "&+	c #DD2A2A",
    "*+	c #E1E1E1",
    "=+	c #9E9E9E",
    "          . + + @ @ @ # $ %   & *   ",
    "      = - ; > , ' ) ! ~ % { ] ^ ^ / ",
    "    ( _ : < [ } | 1 2 3 4 5 6 ^ 7 8 ",
    "  9 0 ^ ^ a b + @ 1 c d e ^ ^ f g   ",
    "  h i 0 ^ j k l m n o p q ^ r s t u ",
    "@ ~ v w x ^ y z A B C q ^ r D E F u ",
    ". G 1 H I f ^ J K L M N O P Q F u R ",
    "S T @ U V W 7 ^ X Y Z `  ...+.@.#.$.",
    "%.&.U *.' =.-.& ;.>.,.'.).!.~.F {.1 ",
    "].~.^./...1 (.^ >._.:.<.[.%.}.|.% 1.",
    "| 2.) }.% 3.4.6 5.6.7.8.9.} T T 1.0.",
    "|.m &.2.a.b.c.d.e.f.g.h.i.1.+ . j.0.",
    "1 . }.k.l.m.n.o.p.q.r.s.t.u.&.+ j.' ",
    "&.v.w.x.y.z.A.B.%.C.D.E.F.G.H.0.'   ",
    "  I.J.K.L.M.c ) ~ N.. O.P.Q.R.S.%   ",
    "  J.T.U.M.V.o.' %.W.@ X.Y.Z.`. +.+  ",
    "  T.++p.$.@+c ! S #+, $+$+).%+&+    ",
    "        @+}.}.&.U | 0.*+=+@+        "
};

static const char* const t_config_icon_data[] = {
    "20 16 21 1",
    " 	c None",
    ".	c #949494",
    "+	c #ABABAB",
    "@	c #9C9C9C",
    "#	c #797979",
    "$	c #5C5C5C",
    "%	c #525252",
    "&	c #9D9D9D",
    "*	c #9E9E9E",
    "=	c #AAAAAA",
    "-	c #A9A9A9",
    ";	c #B26D00",
    ">	c #753B03",
    ",	c #844B00",
    "'	c #7D4509",
    ")	c #7F4609",
    "!	c #743B04",
    "~	c #743C0A",
    "{	c #6D3707",
    "]	c #693307",
    "^	c #643006",
    "                    ",
    "      .+@#$  ###    ",
    "     @+@#%  ##&&    ",
    "    .+*#%   ##&     ",
    "   .=&#%    ##   &# ",
    "  .-&#%;>   ##  &&# ",
    " .-&#% ,;> .-&##### ",
    ".-&#%   >;.-&#####  ",
    " &#%     .-&#%      ",
    "  %     .-&#%       ",
    "       .-&#%;'      ",
    "      .-&#% >;)     ",
    "     .-&#%   !;~    ",
    "    .-&#%     {;]   ",
    "     &#%       ^;^  ",
    "      %         ^^  "
};

/*
static const char* const t_help_icon_data[] = {
"20 20 135 2",
"  	c None",
". 	c #000000",
"+ 	c #193D3C",
"@ 	c #163B3A",
"# 	c #213C3B",
"$ 	c #336261",
"% 	c #3B8281",
"& 	c #307C7B",
"* 	c #163F3E",
"= 	c #111A1A",
"- 	c #568886",
"; 	c #56908E",
"> 	c #539695",
", 	c #478F8D",
"' 	c #398786",
") 	c #2C7A78",
"! 	c #123837",
"~ 	c #101817",
"{ 	c #141D1C",
"] 	c #679594",
"^ 	c #77ACAB",
"/ 	c #71A8A7",
"( 	c #66A2A1",
"_ 	c #5A9B99",
": 	c #4D9392",
"< 	c #3E8A88",
"[ 	c #328381",
"} 	c #267C7A",
"| 	c #0E3534",
"1 	c #121616",
"2 	c #557776",
"3 	c #78ADAC",
"4 	c #7EB0AF",
"5 	c #82B3B2",
"6 	c #6CA6A4",
"7 	c #5E9D9C",
"8 	c #509594",
"9 	c #418C8A",
"0 	c #348583",
"a 	c #287D7B",
"b 	c #1C7674",
"c 	c #083231",
"d 	c #030A0A",
"e 	c #284A4A",
"f 	c #6F8F8E",
"g 	c #7FB1B0",
"h 	c #83B3B2",
"i 	c #5E9E9C",
"j 	c #287E7C",
"k 	c #116F6D",
"l 	c #022E2D",
"m 	c #041313",
"n 	c #186866",
"o 	c #759594",
"p 	c #81B2B1",
"q 	c #78ACAC",
"r 	c #66A3A1",
"s 	c #5A9B9A",
"t 	c #4D9391",
"u 	c #3E8B89",
"v 	c #338482",
"w 	c #277C7A",
"x 	c #1B7573",
"y 	c #0F6E6C",
"z 	c #03615F",
"A 	c #013231",
"B 	c #011111",
"C 	c #096A68",
"D 	c #196B6A",
"E 	c #1D3838",
"F 	c #85A7A6",
"G 	c #539796",
"H 	c #47908E",
"I 	c #3A8886",
"J 	c #2F817F",
"K 	c #247B79",
"L 	c #197472",
"M 	c #0D6D6B",
"N 	c #025856",
"O 	c #000B0B",
"P 	c #011F1F",
"Q 	c #07605F",
"R 	c #0B2525",
"S 	c #76ACAB",
"T 	c #559896",
"U 	c #48908E",
"V 	c #3E8A89",
"W 	c #348482",
"X 	c #297E7C",
"Y 	c #1D6E6C",
"Z 	c #115E5D",
"` 	c #263333",
" .	c #414141",
"..	c #989898",
"+.	c #012727",
"@.	c #075E5C",
"#.	c #1B7574",
"$.	c #70A8A7",
"%.	c #458E8D",
"&.	c #368684",
"*.	c #2B7977",
"=.	c #195251",
"-.	c #355252",
";.	c #555555",
">.	c #E3E3E3",
",.	c #F2F2F2",
"'.	c #8C8C8C",
").	c #6BA5A4",
"!.	c #23504F",
"~.	c #264A49",
"{.	c #7F7F7F",
"].	c #BFBFBF",
"^.	c #FFFFFF",
"/.	c #E9E9E9",
"(.	c #B4B4B4",
"_.	c #3C3C3C",
":.	c #2E3D3D",
"<.	c #959797",
"[.	c #B8B8B8",
"}.	c #EEEEEE",
"|.	c #C1C1C1",
"1.	c #565656",
"2.	c #2A2A2A",
"3.	c #093837",
"4.	c #8E8E8E",
"5.	c #F8F8F8",
"6.	c #C3C3C3",
"7.	c #616161",
"8.	c #1B1B1B",
"9.	c #070707",
"0.	c #012B2A",
"a.	c #9D9D9D",
"b.	c #646464",
"c.	c #3F3F3F",
"d.	c #131313",
"                                        ",
"                                        ",
"                . + @                   ",
"            . # $ % & *                 ",
"        . = - ; > , ' ) !               ",
"    ~ { ] ^ / ( _ : < [ } |             ",
"  1 2 3 4 5 3 6 7 8 9 0 a b c           ",
"d e f 5 g h 3 6 i 8 9 0 j b k l         ",
"m n e o p q / r s t u v w x y z A       ",
"B C D E F 3 ( 7 G H I J K L M N O .     ",
"  P Q x R F S T U V W X Y Z `  ...      ",
"    +.@.#.E o $.%.&.*.=.-.;.>.,.'.      ",
"      +.Q D e f ).!.~.{.].^./.(._..     ",
"        P C n e :.<.[.^.}.|.1.2..       ",
"          P C 3.4.5.5.6.7.8.9.          ",
"            P 0.a.(.b.. .               ",
"              . c.d..                   ",
"                .                       ",
"                                        ",
"                                        "};
*/

static const char* const t_new_icon_data[] = {
    "16 20 109 2",
    "  	c None",
    ". 	c #CED1DD",
    "+ 	c #D2D6E0",
    "@ 	c #D3D7E1",
    "# 	c #CED2DD",
    "$ 	c #C8CCD8",
    "% 	c #C1C5D3",
    "& 	c #B5BACA",
    "* 	c #B6BBCB",
    "= 	c #D3D6E1",
    "- 	c #FFFFFF",
    "; 	c #FDFEFE",
    "> 	c #F6F8FB",
    ", 	c #F1F3F8",
    "' 	c #E7E9F1",
    ") 	c #D5D8E3",
    "! 	c #D0D4DF",
    "~ 	c #BABECE",
    "{ 	c #FEFEFE",
    "] 	c #F9FAFC",
    "^ 	c #F4F5F9",
    "/ 	c #E9EBF2",
    "( 	c #D8DBE5",
    "_ 	c #EAECF3",
    ": 	c #D2D6E1",
    "< 	c #B9BECE",
    "[ 	c #FBFCFD",
    "} 	c #F7F8FB",
    "| 	c #EBEDF3",
    "1 	c #DBDEE7",
    "2 	c #F0F1F6",
    "3 	c #D0D3DF",
    "4 	c #BABFCF",
    "5 	c #F8F9FB",
    "6 	c #E5E7ED",
    "7 	c #B8BCC8",
    "8 	c #C9CBD4",
    "9 	c #C8CBD4",
    "0 	c #C0C4CF",
    "a 	c #A7ACBC",
    "b 	c #9095A7",
    "c 	c #FCFDFE",
    "d 	c #EAECF1",
    "e 	c #C1C5D1",
    "f 	c #C1C4D0",
    "g 	c #B7BBC9",
    "h 	c #AEB3C2",
    "i 	c #ADB1C1",
    "j 	c #848898",
    "k 	c #FAFBFC",
    "l 	c #F5F7FA",
    "m 	c #EFF1F7",
    "n 	c #E8EAF2",
    "o 	c #D7DBE5",
    "p 	c #CCD0DC",
    "q 	c #CACEDA",
    "r 	c #7D828E",
    "s 	c #FDFDFD",
    "t 	c #F3F5F9",
    "u 	c #E0E2EB",
    "v 	c #D7DAE5",
    "w 	c #D1D4E0",
    "x 	c #7E828F",
    "y 	c #E4E6EF",
    "z 	c #DEE1EA",
    "A 	c #D9DDE7",
    "B 	c #D6DAE4",
    "C 	c #7F8390",
    "D 	c #F3F4F9",
    "E 	c #ECEEF4",
    "F 	c #E6E8F0",
    "G 	c #E1E4EC",
    "H 	c #DFE2EB",
    "I 	c #DCDFE9",
    "J 	c #EDEFF6",
    "K 	c #EAECF4",
    "L 	c #E3E6EF",
    "M 	c #E1E4ED",
    "N 	c #E0E3EC",
    "O 	c #808490",
    "P 	c #FCFDFD",
    "Q 	c #F7F9FB",
    "R 	c #F4F6F9",
    "S 	c #F0F2F8",
    "T 	c #E6E8F1",
    "U 	c #E4E7EF",
    "V 	c #E2E5EE",
    "W 	c #808491",
    "X 	c #F6F8FA",
    "Y 	c #F5F6FA",
    "Z 	c #F0F1F8",
    "` 	c #ECEEF5",
    " .	c #EBEDF4",
    "..	c #E8EBF2",
    "+.	c #E5E7F0",
    "@.	c #E9EBF3",
    "#.	c #818491",
    "$.	c #FCFCFD",
    "%.	c #F6F7FA",
    "&.	c #F2F4F8",
    "*.	c #F4F6FA",
    "=.	c #F2F4F9",
    "-.	c #EEF0F7",
    ";.	c #C5C8D4",
    ">.	c #818490",
    ",.	c #808390",
    "'.	c #7F838F",
    ").	c #7F828F",
    "!.	c #757986",
    ". + @ @ @ @ @ # $ % & *         ",
    "= - - - - - ; > , ' ) ! ~       ",
    "= - - - - - { ] ^ / ( _ : <     ",
    "= - - - - - - [ } | 1 } 2 3 4   ",
    "= - - - - - - [ 5 6 7 8 9 0 a b ",
    "= - - - - - - c ] d e f g h i j ",
    "= - - - - - - ; k l m n o p q r ",
    "= - - - - - - s k t ' u v : w x ",
    "= - - - - - { ] l m y z A ( B C ",
    "= - - - - - ; D E _ F y G H I C ",
    "= - - - - ; > m J K ' ' L M N O ",
    "= - { P P Q R , S J K n T U V W ",
    "= - P X X l Y t , Z `  ...F +.W ",
    "= - c } } X l t , S J  ./ ' U W ",
    "= - c } } X Y t , S J ` @.' +.#.",
    "= - c } } X Y t , S J ` @.' +.#.",
    "= - c } } X l t , S J ` @.' +.#.",
    "= - $.%.%.l R &., S J  ...' U W ",
    "= - ; k k ] 5 l *.=.S -.` _ ' #.",
    ";.>.>.O O O O O O ,.'.'.'.'.).!."
};



static const char* const t_find_icon_data[] = {
    "18 18 146 2",
    "  	c None",
    ". 	c #9CA9C3",
    "+ 	c #94ADCF",
    "@ 	c #8BB0D9",
    "# 	c #8FB2D8",
    "$ 	c #9CB2CD",
    "% 	c #9FAEC6",
    "& 	c #85A4CD",
    "* 	c #81B6E5",
    "= 	c #87CAF6",
    "- 	c #92D5F8",
    "; 	c #93D5F8",
    "> 	c #91D5F8",
    ", 	c #94CAEB",
    "' 	c #99B4CD",
    ") 	c #7E9DC7",
    "! 	c #80BCEC",
    "~ 	c #94D7F8",
    "{ 	c #A4E5FA",
    "] 	c #A8E8FB",
    "^ 	c #A0E2FA",
    "/ 	c #9ADCF8",
    "( 	c #A1E2FB",
    "_ 	c #A1DBF1",
    ": 	c #97B3CA",
    "< 	c #7D8BAC",
    "[ 	c #7BAFDF",
    "} 	c #AFEEFC",
    "| 	c #C4FAFD",
    "1 	c #C1F6FD",
    "2 	c #B0EEFC",
    "3 	c #AAE9FB",
    "4 	c #A7E9FB",
    "5 	c #B0EFFB",
    "6 	c #A6D4E3",
    "7 	c #8EA0B7",
    "8 	c #7289AF",
    "9 	c #84C8F5",
    "0 	c #A1E2FA",
    "a 	c #C2F9FD",
    "b 	c #DFFEFE",
    "c 	c #D4F8FD",
    "d 	c #B7F0FC",
    "e 	c #BEF7FD",
    "f 	c #BBF9FE",
    "g 	c #BFFBFE",
    "h 	c #BBF1F5",
    "i 	c #97B4C4",
    "j 	c #698AB9",
    "k 	c #8ED1F8",
    "l 	c #A2E4FA",
    "m 	c #BBF3FC",
    "n 	c #C5F5FD",
    "o 	c #C7F9FE",
    "p 	c #CDFEFE",
    "q 	c #CBFEFE",
    "r 	c #CEFEFE",
    "s 	c #CCFEFE",
    "t 	c #9CB7C5",
    "u 	c #658ABA",
    "v 	c #91D4F8",
    "w 	c #9DDFF9",
    "x 	c #A5E5FA",
    "y 	c #B0ECFB",
    "z 	c #C2F8FD",
    "A 	c #D4FDFE",
    "B 	c #D5FEFE",
    "C 	c #D7FEFE",
    "D 	c #DBFEFE",
    "E 	c #D8FEFE",
    "F 	c #9DB2C0",
    "G 	c #5F79A3",
    "H 	c #90D4F8",
    "I 	c #99DBF9",
    "J 	c #A5E5FB",
    "K 	c #B6F1FD",
    "L 	c #C8FDFE",
    "M 	c #D2FEFE",
    "N 	c #DAFEFE",
    "O 	c #E3FEFE",
    "P 	c #E8FEFE",
    "Q 	c #DCF6F8",
    "R 	c #98A8B8",
    "S 	c #546489",
    "T 	c #83BDE3",
    "U 	c #9EE0FA",
    "V 	c #B3F2FD",
    "W 	c #C6FDFE",
    "X 	c #D3FEFE",
    "Y 	c #E0FEFE",
    "Z 	c #ECFEFE",
    "` 	c #F4FEFE",
    " .	c #CCD8DF",
    "..	c #7C839A",
    "+.	c #6382A7",
    "@.	c #98D6F1",
    "#.	c #A6E8FB",
    "$.	c #C5FDFE",
    "%.	c #E5EDF1",
    "&.	c #969BAF",
    "*.	c #5D5D7B",
    "=.	c #7094B6",
    "-.	c #99D0E5",
    ";.	c #B3F1FC",
    ">.	c #C2FDFE",
    ",.	c #CEE4E9",
    "'.	c #979EB1",
    ").	c #777891",
    "!.	c #8E8B9B",
    "~.	c #657E9E",
    "{.	c #7FA1B7",
    "].	c #8AACBD",
    "^.	c #90AEBD",
    "/.	c #91A8B8",
    "(.	c #808DA2",
    "_.	c #656582",
    ":.	c #8A8798",
    "<.	c #9D8372",
    "[.	c #C58737",
    "}.	c #C67816",
    "|.	c #BB8E4F",
    "1.	c #EAC16F",
    "2.	c #E29C32",
    "3.	c #BB6E13",
    "4.	c #AC742D",
    "5.	c #E4C478",
    "6.	c #F5D07E",
    "7.	c #E09A31",
    "8.	c #B26915",
    "9.	c #9D6828",
    "0.	c #E0C27A",
    "a.	c #F5D07C",
    "b.	c #DE9A35",
    "c.	c #A96316",
    "d.	c #986831",
    "e.	c #DBBB76",
    "f.	c #F5CE78",
    "g.	c #D89634",
    "h.	c #7F491C",
    "i.	c #986C39",
    "j.	c #D5B571",
    "k.	c #C09554",
    "l.	c #795D51",
    "m.	c #784C27",
    "n.	c #765D53",
    "o.	c #5B566D",
    "      . + @ # $ %                   ",
    "    & * = - ; > , '                 ",
    "  ) ! ~ { ] ^ / ( _ :               ",
    "< [ - } | 1 2 3 4 5 6 7             ",
    "8 9 0 a b c d e f g h i             ",
    "j k l m c n o p q r s t             ",
    "u v w x y z A B C D E F             ",
    "G H I J K L M N O P Q R             ",
    "S T I U V W X Y Z `  ...            ",
    "  +.@.#.V $.X Y Z %.&.*.            ",
    "    =.-.;.>.r N ,.'.).!.            ",
    "      ~.{.].^./.(._.:.<.[.}.        ",
    "                      |.1.2.3.      ",
    "                      4.5.6.7.8.    ",
    "                        9.0.a.b.c.  ",
    "                          d.e.f.g.h.",
    "                            i.j.k.l.",
    "                              m.n.o."
};


static const char* const t_about_icon_data[] = {
    "16 16 68 2",
    "Qt c None",
    "#a c #002441",
    ".8 c #00244a",
    ".r c #00284a",
    ".1 c #002852",
    ".W c #002c52",
    ".4 c #003052",
    ".Q c #00305a",
    ".V c #00345a",
    ".y c #003462",
    "## c #003862",
    "#b c #00386a",
    ".# c #003c6a",
    ".I c #003c73",
    ".b c #00406a",
    ".a c #00446a",
    ".s c #004473",
    ".P c #004873",
    ".z c #082c4a",
    ".R c #082c52",
    ".j c #083052",
    ".k c #08406a",
    ".d c #084073",
    ".c c #084473",
    ".U c #08487b",
    ".x c #084c7b",
    ".9 c #084c83",
    ".J c #102c4a",
    ".7 c #103c62",
    ".i c #10446a",
    ".H c #104c7b",
    "#. c #10507b",
    ".q c #105583",
    ".p c #184862",
    ".h c #185583",
    ".O c #185983",
    ".0 c #185d8b",
    ".S c #185d94",
    ".w c #18618b",
    ".K c #186594",
    ".Y c #204c62",
    ".v c #20506a",
    ".e c #205d7b",
    ".A c #20618b",
    ".Z c #20658b",
    ".5 c #206594",
    ".g c #20698b",
    ".2 c #20699c",
    ".M c #295573",
    ".l c #296183",
    ".N c #296d94",
    ".G c #296d9c",
    ".f c #297194",
    ".F c #317194",
    ".X c #3179a4",
    ".E c #39799c",
    ".m c #397da4",
    ".3 c #3981a4",
    ".n c #3981ac",
    ".T c #4185ac",
    ".B c #4189b4",
    ".t c #418db4",
    ".u c #4a91b4",
    ".C c #5299bd",
    ".D c #5299c5",
    ".L c #529dc5",
    ".6 c #fffaff",
    ".o c #ffffff",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt",
    "QtQtQtQtQtQt.#.#.#.a.bQtQtQtQtQt",
    "QtQtQtQt.c.d.e.f.g.h.i.j.cQtQtQt",
    "QtQtQt.k.l.m.n.o.o.p.q.d.r.cQtQt",
    "QtQtQt.s.m.t.u.o.o.v.w.x.y.zQtQt",
    "QtQt.#.A.B.C.D.u.E.F.G.H.I.y.JQt",
    "QtQt.#.K.t.L.o.o.o.M.N.O.P.Q.RQt",
    "QtQt.#.S.T.C.D.o.o.v.N.O.U.V.WQt",
    "QtQt.V.q.X.t.u.o.o.Y.Z.0.P.Q.1Qt",
    "QtQt.y.k.2.m.3.o.o.p.O.q.#.4.rQt",
    "QtQtQt.1.U.5.6.o.o.o.7.#.W.8QtQt",
    "QtQtQt.c.r.#.x.9#.#.##.4#a.cQtQt",
    "QtQtQtQt.c.r###b.V.W.r#a.cQtQtQt",
    "QtQtQtQtQtQt.R.R.W.8.rQtQtQtQtQt",
    "QtQtQtQtQtQtQtQtQtQtQtQtQtQtQtQt"
};



static const char* const t_help_icon_data[] = {
    "20 20 215 2",
    "  	c None",
    ". 	c #4381FF",
    "+ 	c #4884FE",
    "@ 	c #4D87FE",
    "# 	c #4C86FE",
    "$ 	c #4481FE",
    "% 	c #3D7DFF",
    "& 	c #4583FF",
    "* 	c #5B91FE",
    "= 	c #7AA5FE",
    "- 	c #98B9FC",
    "; 	c #AAC3F9",
    "> 	c #AAC4F9",
    ", 	c #95B7FB",
    "' 	c #7DA7FD",
    ") 	c #5D91FC",
    "! 	c #3F7DFD",
    "~ 	c #4F89FE",
    "{ 	c #76A3FD",
    "] 	c #AAC6FD",
    "^ 	c #CCDCFB",
    "/ 	c #DEE8FB",
    "( 	c #E2EAF9",
    "_ 	c #E0E9FA",
    ": 	c #DFE8FA",
    "< 	c #D3E0FB",
    "[ 	c #B5CCFB",
    "} 	c #7CA6FB",
    "| 	c #4B85FA",
    "1 	c #4C87FE",
    "2 	c #81AAFD",
    "3 	c #BCD2FD",
    "4 	c #CEDDFB",
    "5 	c #DBE5FA",
    "6 	c #EAEFFB",
    "7 	c #F9FBFE",
    "8 	c #FAFCFE",
    "9 	c #F6F9FD",
    "0 	c #E3EBFB",
    "a 	c #D9E5FB",
    "b 	c #C3D6FB",
    "c 	c #8AAFF9",
    "d 	c #4682FB",
    "e 	c #6C9CFE",
    "f 	c #AFCAFE",
    "g 	c #C4D7FC",
    "h 	c #D6E2FA",
    "i 	c #F4F7FD",
    "j 	c #F9FBFF",
    "k 	c #E9F0FF",
    "l 	c #E5EEFF",
    "m 	c #F1F6FF",
    "n 	c #FCFDFF",
    "o 	c #ECF2FD",
    "p 	c #D2E1FD",
    "q 	c #B7CEFC",
    "r 	c #709EFB",
    "s 	c #3576FE",
    "t 	c #528BFE",
    "u 	c #8EB3FE",
    "v 	c #B4CDFF",
    "w 	c #BCD1FB",
    "x 	c #E6EDFC",
    "y 	c #FDFEFF",
    "z 	c #E3EDFF",
    "A 	c #C1D5FF",
    "B 	c #BCD1FF",
    "C 	c #C8D9FC",
    "D 	c #F4F7FC",
    "E 	c #D7E4FF",
    "F 	c #B4CDFE",
    "G 	c #8CB1FC",
    "H 	c #4480FC",
    "I 	c #407FFF",
    "J 	c #6296FE",
    "K 	c #9ABBFE",
    "L 	c #A6C3FF",
    "M 	c #B3CBFD",
    "N 	c #E6EDFE",
    "O 	c #F7F9FF",
    "P 	c #C0D4FF",
    "Q 	c #A7C4FF",
    "R 	c #A6C2FE",
    "S 	c #AFC7F5",
    "T 	c #F8FAFF",
    "U 	c #C1D4FE",
    "V 	c #82AAFD",
    "W 	c #6D9DFD",
    "X 	c #4682FE",
    "Y 	c #266DFF",
    "Z 	c #417FFF",
    "` 	c #6B9BFE",
    " .	c #92B6FF",
    "..	c #94B7FF",
    "+.	c #98BAFF",
    "@.	c #AEC9FF",
    "#.	c #B0CAFF",
    "$.	c #9ABBFF",
    "%.	c #A3BEF6",
    "&.	c #D6E1F6",
    "*.	c #FCFDFE",
    "=.	c #E3ECFE",
    "-.	c #8CB1FD",
    ";.	c #5990FF",
    ">.	c #568EFF",
    ",.	c #4986FF",
    "'.	c #266EFF",
    ").	c #417FFE",
    "!.	c #83ADFF",
    "~.	c #85AEFF",
    "{.	c #84ADFF",
    "].	c #86AAF3",
    "^.	c #DDE6F8",
    "/.	c #FBFCFE",
    "(.	c #F0F5FE",
    "_.	c #A9C5FD",
    ":.	c #6095FE",
    "<.	c #548DFF",
    "[.	c #508AFF",
    "}.	c #3E7DFE",
    "|.	c #6296FD",
    "1.	c #6E9EFF",
    "2.	c #6E9DFF",
    "3.	c #76A1F8",
    "4.	c #CEDBF6",
    "5.	c #FEFEFF",
    "6.	c #EAF0FE",
    "7.	c #9CBCFC",
    "8.	c #5C92FE",
    "9.	c #558DFF",
    "0.	c #256DFE",
    "a.	c #3A7BFF",
    "b.	c #6195FF",
    "c.	c #6095FF",
    "d.	c #5F94FE",
    "e.	c #7FA8F9",
    "f.	c #E5EBF9",
    "g.	c #ACC7FD",
    "h.	c #4B87FF",
    "i.	c #246BFD",
    "j.	c #3779FF",
    "k.	c #4B86FF",
    "l.	c #5D92FF",
    "m.	c #5E93FF",
    "n.	c #5D93FF",
    "o.	c #7CA5FC",
    "p.	c #DAE5FB",
    "q.	c #D8E5FF",
    "r.	c #7CA7FF",
    "s.	c #5C92FF",
    "t.	c #5B91FF",
    "u.	c #3F7EFF",
    "v.	c #2269FC",
    "w.	c #427FFE",
    "x.	c #5C91FE",
    "y.	c #6497FF",
    "z.	c #6F9DFC",
    "A.	c #B0C9FA",
    "B.	c #ACC6FC",
    "C.	c #6F9EFF",
    "D.	c #5A90FE",
    "E.	c #3375FD",
    "F.	c #3879FD",
    "G.	c #548BFC",
    "H.	c #6D9CFD",
    "I.	c #709FFF",
    "J.	c #709EFF",
    "K.	c #82A8F7",
    "L.	c #C9D9F8",
    "M.	c #CDDCFA",
    "N.	c #8DB3FF",
    "O.	c #70A0FF",
    "P.	c #6C9CFD",
    "Q.	c #4C85FB",
    "R.	c #286DFB",
    "S.	c #407EFB",
    "T.	c #6194FA",
    "U.	c #77A4FC",
    "V.	c #7BA7FF",
    "W.	c #7BA6FE",
    "X.	c #9CBAF8",
    "Y.	c #F0F4FD",
    "Z.	c #F5F8FF",
    "`.	c #ABC7FF",
    " +	c #7CA8FF",
    ".+	c #77A3FC",
    "++	c #5D90F8",
    "@+	c #3475F8",
    "#+	c #417EFA",
    "$+	c #5F92F9",
    "%+	c #86AEFE",
    "&+	c #95B7FD",
    "*+	c #C4D7FF",
    "=+	c #C9DBFF",
    "-+	c #9CBCFF",
    ";+	c #87AEFE",
    ">+	c #7BA5FA",
    ",+	c #5B8FF7",
    "'+	c #3877F6",
    ")+	c #3778FB",
    "!+	c #4C85F9",
    "~+	c #6B99F5",
    "{+	c #83A8F6",
    "]+	c #94B6FC",
    "^+	c #95B7FC",
    "/+	c #83A8F5",
    "(+	c #6996F3",
    "_+	c #4882F6",
    ":+	c #2F72F8",
    "<+	c #3073FB",
    "[+	c #3375FC",
    "}+	c #3376FE",
    "|+	c #3275FD",
    "1+	c #3173FA",
    "2+	c #2C6FF8",
    "                                        ",
    "              . + @ # $ %               ",
    "          & * = - ; > , ' ) !           ",
    "        ~ { ] ^ / ( _ : < [ } |         ",
    "      1 2 3 4 5 6 7 8 9 0 a b c d       ",
    "    . e f g h i j k l m n o p q r s     ",
    "    t u v w x y z A B C D j E F G H     ",
    "  I J K L M N O P Q R S D T U V W X Y   ",
    "  Z `  ...+.@.#.$...%.&.*.=.-.;.>.,.'.  ",
    "  ).W !.!.!.~.~.{.].^./.(._.:.<.<.[.'.  ",
    "  }.|.1.2.2.2.2.3.4.5.6.7.8.9.9.9.[.0.  ",
    "  a.9.b.b.b.c.d.e.f.O g.J ;.;.;.;.h.i.  ",
    "  j.k.l.m.n.l.l.o.p.q.r.l.s.s.s.t.u.v.  ",
    "    w.x.y.y.y.y.z.A.B.C.y.y.y.y.D.E.    ",
    "    F.G.H.I.I.J.K.L.M.N.O.I.I.P.Q.R.    ",
    "      S.T.U.V.W.X.Y.Z.`. +V..+++@+      ",
    "        #+$+} %+&+*+=+-+;+>+,+'+        ",
    "          )+!+~+{+]+^+/+(+_+:+          ",
    "              <+[+}+|+1+2+              ",
    "                                        "
};


static const char* const t_comment_icon_data[] = {
    "14 16 102 2",
    "  	c None",
    ". 	c #A4B6E6",
    "+ 	c #9CB2DE",
    "@ 	c #5A75AC",
    "# 	c #B4896A",
    "$ 	c #E6D2B4",
    "% 	c #FFFFFF",
    "& 	c #F6F6FF",
    "* 	c #E6EAF6",
    "= 	c #DEE6F6",
    "- 	c #C5D2EE",
    "; 	c #CDDAF6",
    "> 	c #5275AC",
    ", 	c #AC856A",
    "' 	c #E6CEB4",
    ") 	c #C58D52",
    "! 	c #FFFAFF",
    "~ 	c #EEEEFF",
    "{ 	c #E6E6F6",
    "] 	c #D5DEF6",
    "^ 	c #CDD6EE",
    "/ 	c #ACBEE6",
    "( 	c #AC8573",
    "_ 	c #E6CEAC",
    ": 	c #BD8541",
    "< 	c #E6EAFF",
    "[ 	c #CDDAEE",
    "} 	c #6A85B4",
    "| 	c #A48573",
    "1 	c #BD814A",
    "2 	c #FFF6FF",
    "3 	c #EEF2FF",
    "4 	c #DEE2F6",
    "5 	c #D5DAEE",
    "6 	c #B4917B",
    "7 	c #C58952",
    "8 	c #C58541",
    "9 	c #BD7D4A",
    "0 	c #835531",
    "a 	c #EEEEF6",
    "b 	c #C5A59C",
    "c 	c #C5854A",
    "d 	c #BD7D41",
    "e 	c #835029",
    "f 	c #180C08",
    "g 	c #F6F2FF",
    "h 	c #EEE6E6",
    "i 	c #F6EEE6",
    "j 	c #DEAA94",
    "k 	c #835031",
    "l 	c #201010",
    "m 	c #E6E2EE",
    "n 	c #EECABD",
    "o 	c #BD856A",
    "p 	c #B4795A",
    "q 	c #201410",
    "r 	c #EEDEDE",
    "s 	c #F6DED5",
    "t 	c #B49183",
    "u 	c #7B5D4A",
    "v 	c #5A4C41",
    "w 	c #292020",
    "x 	c #4A658B",
    "y 	c #EEEAFF",
    "z 	c #D5D2D5",
    "A 	c #F6DACD",
    "B 	c #B48D7B",
    "C 	c #7B5D52",
    "D 	c #312C29",
    "E 	c #5A555A",
    "F 	c #BDC6DE",
    "G 	c #948583",
    "H 	c #9C7D73",
    "I 	c #4A4039",
    "J 	c #6A696A",
    "K 	c #C5C6DE",
    "L 	c #C5CEE6",
    "M 	c #BDCEEE",
    "N 	c #526DA4",
    "O 	c #BDBABD",
    "P 	c #181410",
    "Q 	c #626162",
    "R 	c #BDBECD",
    "S 	c #A4B2C5",
    "T 	c #9CA1BD",
    "U 	c #8391A4",
    "V 	c #39507B",
    "W 	c #8B8583",
    "X 	c #ACAEB4",
    "Y 	c #9CA5B4",
    "Z 	c #9499AC",
    "` 	c #8B95A4",
    " .	c #8B95AC",
    "..	c #8B91A4",
    "+.	c #D5DEEE",
    "@.	c #CDD6F6",
    "#.	c #C5CEEE",
    "$.	c #BDCAE6",
    "%.	c #DEDEF6",
    "&.	c #D5DAF6",
    "*.	c #CDD2EE",
    "=.	c #5271AC",
    ". + . + . + . + @       # $ ",
    "+ % & & * = - ; + >   , ' ) ",
    ". % ! & ~ { ] ^ . / ( _ ) : ",
    "+ % & & * < [ ; } | ' ) : 1 ",
    ". % 2 & 3 * 4 5 6 $ 7 8 9 0 ",
    "+ ! & & a ~ 4 b ' c : d e f ",
    ". ! & & g ~ h i j c d k l   ",
    "+ ! 3 ~ * m i n o p 0 q     ",
    ". g g ~ < r s t u v w x     ",
    "+ & 3 y z A B C D E F @     ",
    ". & 3 4 G H I J K L M N     ",
    "+ & * O P Q R F S T U V     ",
    ". 3 { W X Y Z `  .U ..V     ",
    "+ 3 4 4 +.] ^ @.- #.$.@     ",
    ". ~ = 4 %.&.&.^ *.#.M >     ",
    "=.@ =.> =.@ =.> =.@ =.>     "
};



static const char* const t_remove_export_icon_data[] = {
    "18 16 7 1",
    " 	c None",
    ".	c #68568B",
    "+	c #6C5891",
    "@	c #6F5A95",
    "#	c #5F5377",
    "$	c #63567C",
    "%	c #665880",
    "                  ",
    "                  ",
    "                  ",
    "    .+@@      .+@@",
    "   .+@@      .+@@ ",
    "  .+@@      .+@@  ",
    " .+@@      .+@@   ",
    ".+@@      .+@@    ",
    "#$%%      #$%%    ",
    " #$%%      #$%%   ",
    "  #$%%      #$%%  ",
    "   #$%%      #$%% ",
    "    #$%%      #$%%",
    "                  ",
    "                  ",
    "                  "
};

static const char* const t_add_export_icon_data[] = {
    "18 16 7 1",
    " 	c None",
    ".	c #6F5A95",
    "+	c #6C5891",
    "@	c #68568B",
    "#	c #665880",
    "$	c #63567C",
    "%	c #5F5377",
    "                  ",
    "                  ",
    "                  ",
    "..+@      ..+@    ",
    " ..+@      ..+@   ",
    "  ..+@      ..+@  ",
    "   ..+@      ..+@ ",
    "    ..+@      ..+@",
    "    ##$%      ##$%",
    "   ##$%      ##$% ",
    "  ##$%      ##$%  ",
    " ##$%      ##$%   ",
    "##$%      ##$%    ",
    "                  ",
    "                  ",
    "                  "
};



static const char* const t_showc_icon_data[] = {
    "16 16 120 2",
    "Qt c None",
    "#K c None",
    "#R c #4a4483",
    ".e c #7371ac",
    ".V c #525094",
    ".n c #9c95c5",
    "#J c #41448b",
    "#C c #7b75ac",
    "#S c #6a65a4",
    ".3 c #9495c5",
    ".f c #8b89bd",
    "#Q c #4a488b",
    ".m c #736dac",
    ".o c #9c95c5",
    ".# c #8b85bd",
    "#v c #7375ac",
    ".I c #a4a1cd",
    "#X c #52508b",
    ".2 c #524c94",
    ".y c #9c99c5",
    "#B c #414083",
    ".P c #5a5d9c",
    ".x c #6a69a4",
    ".W c #9c9dcd",
    "#1 c #5a5594",
    "#L c #6a69a4",
    "#b c #8b8dbd",
    ".a c #8381b4",
    ".p c #9495c5",
    ".Q c #9c9dc5",
    "#I c #4a488b",
    "#a c #4a488b",
    ".g c #8b89bd",
    "#Y c #5a5d9c",
    "#P c #52508b",
    "#n c #8b85bd",
    ".d c #7b79b4",
    ".z c #9c99c5",
    "#W c #525594",
    ".J c #a49dcd",
    "#u c #4a488b",
    ".U c #525594",
    "#D c #7b79ac",
    "#A c #4a488b",
    ".H c #6a6da4",
    "#0 c #62619c",
    ".4 c #9c95c5",
    "#T c #6a69a4",
    ".b c #8b89bd",
    "#H c #5a5994",
    ".h c #8b8dbd",
    ".l c #8381b4",
    "#h c #525594",
    "#i c #8b8dbd",
    ".q c #9c99c5",
    ".1 c #6a65a4",
    ".A c #a4a5cd",
    ".K c #acaad5",
    "#O c #62619c",
    "#w c #9c95c5",
    ".X c #b4b2d5",
    ".O c #7371ac",
    "#M c #8b8dbd",
    "#V c #7b75ac",
    "#t c #625d9c",
    "#z c #6a6da4",
    ".i c #b4b2d5",
    "#c c #b4b2d5",
    "#Z c #6a6da4",
    ".c c #8381b4",
    ".r c #acaed5",
    "## c #8381b4",
    ".L c #cdcae6",
    ".R c #b4b2d5",
    "#N c #8b8dbd",
    "#m c #5a5594",
    ".w c #9c95c5",
    ".T c #a49dcd",
    "#G c #8385b4",
    "#s c #9491bd",
    ".B c #c5c2de",
    "#E c #cdc6e6",
    ".5 c #dedeee",
    "#U c #aca5cd",
    "#o c #bdbad5",
    "#y c #a4a1cd",
    ".j c #cdcae6",
    ".S c #e6e6f6",
    "#f c #9491bd",
    "#e c #9c95c5",
    "#l c #9c9dc5",
    "#g c #a4a1cd",
    ".k c #b4b6d5",
    ".G c #bdbad5",
    ".Y c #bdbade",
    ".0 c #bdbede",
    ".8 c #c5c2de",
    "#r c #cdc6e6",
    "#. c #cdcae6",
    "#d c #cdcee6",
    ".v c #d5cee6",
    "#j c #d5d2e6",
    "#p c #d5d2ee",
    ".F c #d5d6ee",
    "#k c #ded6ee",
    ".7 c #dedaee",
    "#q c #dedaf6",
    "#x c #dedeee",
    ".M c #e6deee",
    ".N c #e6def6",
    ".9 c #e6e2ee",
    ".Z c #e6e2f6",
    ".C c #e6e6f6",
    ".s c #eee6f6",
    ".E c #eeeaf6",
    ".6 c #eeeef6",
    "#F c #f6f2ff",
    ".t c #fff6ff",
    ".u c #fffaff",
    ".D c #ffffff",
    "QtQtQtQtQtQtQtQt.#.a.b.c.d.eQtQt",
    "QtQtQtQtQtQt.f.g.h.i.j.k.l.mQtQt",
    "QtQtQt.n.o.p.q.r.s.t.u.v.w.xQtQt",
    "QtQt.y.z.A.B.C.D.D.D.E.F.G.HQtQt",
    ".I.J.K.L.s.D.D.D.D.u.F.M.N.O.PQt",
    ".Q.R.S.t.D.D.D.D.u.t.k.s.M.T.U.V",
    ".W.X.s.C.N.C.u.D.u.E.Y.Z.N.0.1.2",
    ".3.4.5.D.6.7.G.8.9.7.k.N.F#.###a",
    "Qt#b#c.D.D.D.u#d#e#f.0#..v#.#g#h",
    "Qt.f#i.C.D.D.u.Z#..8#j#k#..8#l#m",
    "QtQt#n#o.D.D.t#p.s.9#q#j#r#s#t#u",
    "QtQt#v#w.9.u.s.7.9#x#d#y#z#A#BQt",
    "QtQt#C#D#E#F.Z#x.N.0#G#H#I#JQtQt",
    "QtQt#K#L#M.C.7.v#N#O#P#Q#RQtQtQt",
    "QtQtQt#S#T#o#U#V#W#XQtQtQtQtQtQt",
    "QtQtQtQt#Y#Z#0#1QtQtQtQtQtQtQtQt"
};

static const char* const t_deletec_icon_data[] = {
    "16 16 45 1",
    "e c None",
    "t c None",
    "m c None",
    "f c None",
    "d c None",
    ". c None",
    "w c #73819c",
    "B c #7b819c",
    "c c #7b859c",
    "F c #acb6cd",
    "A c #b4b6cd",
    "a c #b4becd",
    "b c #b4bed5",
    "# c #bdbed5",
    "s c #bdc2d5",
    "z c #bdc6d5",
    "E c #c5cad5",
    "H c #c5cade",
    "J c #c5cede",
    "K c #cdcede",
    "O c #cdd2de",
    "I c #cdd2e6",
    "N c #cdd6de",
    "Q c #cdd6e6",
    "k c #d5d6e6",
    "M c #d5dae6",
    "r c #d5dee6",
    "v c #dedee6",
    "l c #dedeee",
    "q c #dee2ee",
    "D c #e6e2ee",
    "P c #e6e6ee",
    "y c #e6e6f6",
    "j c #e6eaf6",
    "i c #eeeaee",
    "p c #eeeaf6",
    "u c #eeeef6",
    "C c #eef2f6",
    "o c #f6eeff",
    "L c #f6f2f6",
    "x c #f6f2ff",
    "h c #f6f6ff",
    "G c #ff0000",
    "n c #fffaff",
    "g c #ffffff",
    "..#a#b#a#bcdef..",
    "..aghhijklac.m..",
    "..#gnhopqr#sct..",
    "..aghhuuvlwcwc..",
    "..#gnhxuyvkzAB..",
    "..aghhCCDqEaFc..",
    "..#GGhhuyGG##w..",
    "..anGGppGGHHzc..",
    "..#hxGGGGkIJKB..",
    "..ahLuGGMMNIKc..",
    "..#hxGGGlMkIIw..",
    "..ahpGGGGvkkOc..",
    "..#LGGPqGGMQkB..",
    "..aGGyqqvGGkOc..",
    "..#uyPPqlMMkIw..",
    "..wcwcwcwcwcwc.."
};


static const char* const m_import_icon_data[] = {
    "16 18 24 1",
    " 	c None",
    ".	c #C12896",
    "+	c #002E55",
    "@	c #003565",
    "#	c #003C74",
    "$	c #004284",
    "%	c #004993",
    "&	c #004FA2",
    "*	c #0056B1",
    "=	c #005EC1",
    "-	c #0063D1",
    ";	c #006BE0",
    ">	c #0071EF",
    ",	c #0078FF",
    "'	c #003B74",
    ")	c #0050A3",
    "!	c #0056B2",
    "~	c #005DC1",
    "{	c #0064D1",
    "]	c #0071F0",
    "^	c #003564",
    "/	c #004283",
    "(	c #0050A2",
    "_	c #0057B2",
    "      ....      ",
    "                ",
    "      ....      ",
    "      ....      ",
    "                ",
    "      ....      ",
    "      ....      ",
    "   .  ....  .   ",
    "   .. .... ..   ",
    "   ..........   ",
    "    ........    ",
    "     ......     ",
    "      ....      ",
    "       ..       ",
    "  +@#$%&*=-;>,  ",
    "                ",
    "  +@'$%)!~{;],  ",
    "  +^'/%(_~{;>,  "
};

static const char* const m_export_icon_data[] = {
    "16 18 24 1",
    " 	c None",
    ".	c #00CC0A",
    "+	c #002E55",
    "@	c #003565",
    "#	c #003C74",
    "$	c #004284",
    "%	c #004993",
    "&	c #004FA2",
    "*	c #0056B1",
    "=	c #005EC1",
    "-	c #0063D1",
    ";	c #006BE0",
    ">	c #0071EF",
    ",	c #0078FF",
    "'	c #003B74",
    ")	c #0050A3",
    "!	c #0056B2",
    "~	c #005DC1",
    "{	c #0064D1",
    "]	c #0071F0",
    "^	c #003564",
    "/	c #004283",
    "(	c #0050A2",
    "_	c #0057B2",
    "       ..       ",
    "      ....      ",
    "     ......     ",
    "    ........    ",
    "   ..........   ",
    "   .. .... ..   ",
    "   .  ....  .   ",
    "      ....      ",
    "      ....      ",
    "                ",
    "      ....      ",
    "      ....      ",
    "                ",
    "      ....      ",
    "  +@#$%&*=-;>,  ",
    "                ",
    "  +@'$%)!~{;],  ",
    "  +^'/%(_~{;>,  "
};

static const char* const m_borrow_icon_data[] = {
    "32 32 53 1",
    ". c None",
    "# c None",
    "a c #000000",
    "V c #838583",
    "U c #8b898b",
    "S c #8b8d8b",
    "T c #948d94",
    "R c #949194",
    "L c #949594",
    "Q c #9c959c",
    "H c #9c999c",
    "Y c #9c9d9c",
    "X c #a49da4",
    "C c #a4a1a4",
    "A c #a4a5a4",
    "G c #aca5a4",
    "q c #aca5ac",
    "f c #acaaac",
    "l c #acaeac",
    "E c #b4aeac",
    "p c #b4aeb4",
    "c c #b4b2b4",
    "B c #b4b6b4",
    "t c #bdb6bd",
    "o c #bdbabd",
    "k c #bdbebd",
    "n c #c5bec5",
    "j c #c5c2c5",
    "i c #c5c6c5",
    "s c #cdc6cd",
    "M c #cdcacd",
    "O c #cdcecd",
    "N c #d5ced5",
    "P c #d5d2d5",
    "J c #d5d6d5",
    "I c #ded6de",
    "F c #dedade",
    "K c #dedede",
    "W c #e6dee6",
    "D c #e6e2e6",
    "u c #e6e6e6",
    "x c #eee6e6",
    "y c #eee6ee",
    "g c #eeeaee",
    "v c #eeeeee",
    "e c #f6eeee",
    "d c #f6eef6",
    "r c #f6f2ee",
    "w c #f6f2f6",
    "b c #f6f6f6",
    "z c #fff6ff",
    "m c #fffaff",
    "h c #ffffff",
    "................................",
    "................................",
    "#a############################a#",
    "aba##########################aca",
    "ada##########################aca",
    "adea##aaa##############aaa##acfa",
    "agda#ahhha############aijka#alfa",
    "#agdaabhhha##########ajjkkaalfa#",
    "#aggaammhha##########ajknoaapqa#",
    "##aggarmmhha########asknttapqa##",
    "##augaabmmha###aa###ajjooaafqa##",
    "###augvwbmma##aaja##ajooccfqa###",
    "###axygvwza#aahasiaa#aocclfAa###",
    "####ayuggwbahhhajsjjaoBclfCa####",
    "####aDyyevzzhhhasijnkBtEfACa####",
    "#####aDxuvvwbmhaajkkBBpfACa#####",
    "#####aDDyyddzaa##aaotcpfqCa#####",
    "####aFFDDyxva######acpAGCCHa####",
    "##aaIJFKDuga########affCCHHLaa##",
    "#aMNOPJFKDDa########aqCCHHQRSSa#",
    "aiMMNPaaaKDa########aACaaaQSTUUa",
    "asiMMa###aKDa######aACa###aSUUVa",
    "#aasa####aWWa######aCXa####aUaa#",
    "###a######aWKa####aqCa######a###",
    "##########aFWa####aCXa##########",
    "###########aFWa##aCCa###########",
    "############aFWaaACa############",
    "############aFFaCCYa############",
    "#############aFaCXa#############",
    "##############aaYa##############",
    "###############aa###############",
    "................................"
};

static const char* const t_sborrow_icon_data[] = {
    "32 32 56 1",
    ". c None",
    "# c None",
    "a c #000000",
    "X c #8b898b",
    "U c #8b8d8b",
    "T c #949194",
    "L c #949594",
    "H c #9c999c",
    "1 c #9c9d9c",
    "0 c #a49da4",
    "C c #a4a1a4",
    "A c #a4a5a4",
    "G c #aca5a4",
    "q c #aca5ac",
    "f c #acaaac",
    "l c #acaeac",
    "E c #b4aeac",
    "p c #b4aeb4",
    "c c #b4b2b4",
    "B c #b4b6b4",
    "t c #bdb6bd",
    "o c #bdbabd",
    "k c #bdbebd",
    "n c #c5bec5",
    "j c #c5c2c5",
    "i c #c5c6c5",
    "s c #cdc6cd",
    "M c #cdcacd",
    "O c #cdcecd",
    "N c #d5ced5",
    "P c #d5d2d5",
    "J c #d5d6d5",
    "I c #ded6de",
    "F c #dedade",
    "K c #dedede",
    "Z c #e6dee6",
    "D c #e6e2e6",
    "u c #e6e6e6",
    "R c #ee4841",
    "x c #eee6e6",
    "y c #eee6ee",
    "g c #eeeaee",
    "v c #eeeeee",
    "S c #f64841",
    "Q c #f6484a",
    "Y c #f6ea41",
    "W c #f6ee41",
    "e c #f6eeee",
    "d c #f6eef6",
    "r c #f6f2ee",
    "w c #f6f2f6",
    "b c #f6f6f6",
    "V c #ffee4a",
    "z c #fff6ff",
    "m c #fffaff",
    "h c #ffffff",
    "................................",
    "................................",
    "#a############################a#",
    "aba##########################aca",
    "ada##########################aca",
    "adea##aaa##############aaa##acfa",
    "agda#ahhha############aijka#alfa",
    "#agdaabhhha##########ajjkkaalfa#",
    "#aggaammhha##########ajknoaapqa#",
    "##aggarmmhha########asknttapqa##",
    "##augaabmmha###aa###ajjooaafqa##",
    "###augvwbmma##aaja##ajooccfqa###",
    "###axygvwza#aahasiaa#aocclfAa###",
    "####ayuggwbahhhajsjjaoBclfCa####",
    "####aDyyevzzhhhasijnkBtEfACa####",
    "#####aDxuvvwbmhaajkkBBpfACa#####",
    "#####aDDyyddzaa##aaotcpfqCa#####",
    "####aFFDDyxva######acpAGCCHa####",
    "##aaIJFKDuga########affCCHHLaa##",
    "#aMNOPJFKDDa########aqCQRQSTUUa#",
    "aiMMNPaaaKDa########aACSVWQSQXXa",
    "asiMMa###aKDa##QSQRQSQRQYVYVSQRa",
    "#aasa####aZZa##QVWVWVWVWVWVWVWQQ",
    "###a######aZKa#QYVYVYVYVYVYVYVYQ",
    "##########aFZa#SVWVWVWVWVWVWVWVS",
    "###########aFZaQYVYVYVYVYVYVYVYQ",
    "############aFZQVWVWVWVWVWVWVWQQ",
    "############aFFQRQSQRQSQYVYVRQS#",
    "#############aFaC0a####SVWQSQ###",
    "##############aa1a#####QSQR#####",
    "###############aa###############",
    "................................"
};

static const char* const t_cborrow_icon_data[] = {
    "32 32 50 1",
    ". c None",
    "# c None",
    "a c #000000",
    "U c #838583",
    "T c #8b898b",
    "R c #8b8d8b",
    "S c #948d94",
    "C c #a4a1a4",
    "A c #a4a5a4",
    "J c #aca5a4",
    "q c #aca5ac",
    "f c #acaaac",
    "l c #acaeac",
    "E c #b4aeac",
    "p c #b4aeb4",
    "c c #b4b2b4",
    "B c #b4b6b4",
    "t c #bdb6bd",
    "o c #bdbabd",
    "k c #bdbebd",
    "n c #c5bec5",
    "j c #c5c2c5",
    "i c #c5c6c5",
    "s c #cdc6cd",
    "N c #cdcacd",
    "P c #cdcecd",
    "O c #d5ced5",
    "Q c #d5d2d5",
    "L c #d5d6d5",
    "K c #ded6de",
    "I c #dedade",
    "M c #dedede",
    "V c #e6dee6",
    "D c #e6e2e6",
    "u c #e6e6e6",
    "G c #ee4841",
    "x c #eee6e6",
    "y c #eee6ee",
    "g c #eeeaee",
    "v c #eeeeee",
    "H c #f64841",
    "F c #f6484a",
    "e c #f6eeee",
    "d c #f6eef6",
    "r c #f6f2ee",
    "w c #f6f2f6",
    "b c #f6f6f6",
    "z c #fff6ff",
    "m c #fffaff",
    "h c #ffffff",
    "................................",
    "................................",
    "#a############################a#",
    "aba##########################aca",
    "ada##########################aca",
    "adea##aaa##############aaa##acfa",
    "agda#ahhha############aijka#alfa",
    "#agdaabhhha##########ajjkkaalfa#",
    "#aggaammhha##########ajknoaapqa#",
    "##aggarmmhha########asknttapqa##",
    "##augaabmmha###aa###ajjooaafqa##",
    "###augvwbmma##aaja##ajooccfqa###",
    "###axygvwza#aahasiaa#aocclfAa###",
    "####ayuggwbahhhajsjjaoBclfCa####",
    "####aDyyevzzhhhasijnkBtEfACa####",
    "#####aDxuvvwbmhFGFkkBBpfACa#GFH#",
    "#####aDDyyddzaaHFFFotcpfqCaHFFF#",
    "####aIIDDyxva##FHFGFcpAJCCGFHFG#",
    "##aaKLIMDuga####FHFFFffCCHFFFH##",
    "#aNOPQLIMDDa#####FHFGFCCGFHFGRa#",
    "aiNNOQaaaMDa######FHFFFHFFFHSTTa",
    "asiNNa###aMDa######FHFGFHFGRTTUa",
    "#aasa####aVVa######aFHFFFH#aTaa#",
    "###a######aVMa####aqGFHFGF##a###",
    "##########aIVa####aHFFFHFFF#####",
    "###########aIVa##aGFHFGFHFGF####",
    "############aIVaaHFFFH##FHFFF###",
    "############aIIaGFHFG####FHFGF##",
    "#############aIHFFFH######FHFFF#",
    "##############aFHFG########FHFG#",
    "###############FFH##########FHF#",
    "................................"
};


static const char* const t_colorconfig_icon_data[] = {
    "16 16 9 1",
    ". c None",
    "# c #000000",
    "a c #0000ff",
    "d c #00c200",
    "c c #00c2c5",
    "b c #c500c5",
    "f c #ff0000",
    "e c #ffaa5a",
    "g c #ffdeac",
    "......###.......",
    "......#aa##.....",
    "......###aa###..",
    "....###bb##aaa##",
    "....#cc##bb##aa#",
    "....###cc###bba#",
    "..###dd##ccc##a#",
    "..#ee##dd##cc#a#",
    "..###ee###ddc#a#",
    "..#ff##eee#dc#a#",
    ".##ffff##e#dc###",
    "#gg###ff#e#dc#..",
    "#gggg#ff#e#dc#..",
    "#ggg#fff#e###...",
    "#gg#.##f#e#.....",
    ".##....###......"
};


QPixmap * m_catalog_icon=NULL;
QPixmap * m_unknown_icon=NULL;
QPixmap * m_cd_icon=NULL;
QPixmap * m_dvd_icon=NULL;
QPixmap * m_hdd_icon=NULL;
QPixmap * m_net_icon=NULL;
QPixmap * m_flash_icon=NULL;
QPixmap * m_other_icon=NULL;
QPixmap * m_floppy_icon=NULL;
QPixmap * v_folderlocked_icon=NULL;
QPixmap * v_folderclosed_icon=NULL;
QPixmap * v_folderopen_icon=NULL;
QPixmap * v_file_icon=NULL;
QPixmap * v_back_icon=NULL;
QPixmap * a_0=NULL;
QPixmap * a_1=NULL;
QPixmap * a_2=NULL;
QPixmap * a_3=NULL;
QPixmap * a_4=NULL;
QPixmap * p_icon=NULL;
QPixmap * t_open_icon=NULL;
QPixmap * t_save_icon=NULL;
QPixmap * t_saveas_icon=NULL;
QPixmap * t_close_icon=NULL;
QPixmap * t_add_icon=NULL;
QPixmap * t_rescan_icon=NULL;
QPixmap * t_delete_icon=NULL;
QPixmap * t_config_icon=NULL;
QPixmap * t_help_icon=NULL;
QPixmap * t_new_icon=NULL;
QPixmap * t_find_icon=NULL;
QPixmap * t_about_icon=NULL;
QPixmap * t_add_export_icon=NULL;
QPixmap * t_remove_export_icon=NULL;
QPixmap * t_comment_icon=NULL;
QPixmap * t_deletec_icon=NULL;
QPixmap * t_showc_icon=NULL;
QPixmap * m_export_icon=NULL;
QPixmap * m_import_icon=NULL;
QPixmap * m_borrow_icon=NULL;
QPixmap * t_sborrow_icon=NULL;
QPixmap * t_cborrow_icon=NULL;
QPixmap * t_colorconfig_icon=NULL;

void init_icon_base ( void ) {

    if ( m_catalog_icon==NULL )
        m_catalog_icon=new QPixmap ( ( const char ** ) m_catalog_icon_data );

    if ( m_unknown_icon==NULL )
        m_unknown_icon=new QPixmap ( ( const char ** ) m_unknown_icon_data );

    if ( m_cd_icon==NULL )
        m_cd_icon=new QPixmap ( ( const char ** ) m_cd_icon_data );

    if ( m_dvd_icon==NULL )
        m_dvd_icon=new QPixmap ( ( const char ** ) m_dvd_icon_data );

    if ( m_hdd_icon==NULL )
        m_hdd_icon=new QPixmap ( ( const char ** ) m_hdd_icon_data );

    if ( m_net_icon==NULL )
        m_net_icon=new QPixmap ( ( const char ** ) m_net_icon_data );

    if ( m_flash_icon==NULL )
        m_flash_icon=new QPixmap ( ( const char ** ) m_flash_icon_data );

    if ( m_other_icon==NULL )
        m_other_icon=new QPixmap ( ( const char ** ) m_other_icon_data );

    if ( m_floppy_icon==NULL )
        m_floppy_icon=new QPixmap ( ( const char ** ) m_floppy_icon_data );

    if ( v_folderlocked_icon==NULL )
        v_folderlocked_icon=new QPixmap ( ( const char ** ) v_folderlocked_icon_data );

    if ( v_folderclosed_icon==NULL )
        v_folderclosed_icon=new QPixmap ( ( const char ** ) v_folderclosed_icon_data );

    if ( v_folderopen_icon==NULL )
        v_folderopen_icon=new QPixmap ( ( const char ** ) v_folderopen_icon_data );

    if ( v_file_icon==NULL )
        v_file_icon=new QPixmap ( ( const char ** ) v_file_icon_data );

    if ( v_back_icon==NULL )
        v_back_icon=new QPixmap ( ( const char ** ) v_back_icon_data );

    if ( a_0==NULL )
        a_0=new QPixmap ( ( const char ** ) a_0_data );

    if ( a_1==NULL )
        a_1=new QPixmap ( ( const char ** ) a_1_data );

    if ( a_2==NULL )
        a_2=new QPixmap ( ( const char ** ) a_2_data );

    if ( a_3==NULL )
        a_3=new QPixmap ( ( const char ** ) a_3_data );

    if ( a_4==NULL )
        a_4=new QPixmap ( ( const char ** ) a_4_data );

    if ( p_icon==NULL )
        p_icon=new QPixmap ( ( const char ** ) p_icon_data );

    if ( t_open_icon==NULL )
        t_open_icon=new QPixmap ( ( const char ** ) t_open_icon_data );

    if ( t_save_icon==NULL )
        t_save_icon=new QPixmap ( ( const char ** ) t_save_icon_data );

    if ( t_saveas_icon==NULL )
        t_saveas_icon=new QPixmap ( ( const char ** ) t_saveas_icon_data );

    if ( t_close_icon==NULL )
        t_close_icon=new QPixmap ( ( const char ** ) t_close_icon_data );

    if ( t_add_icon==NULL )
        t_add_icon=new QPixmap ( ( const char ** ) t_add_icon_data );

    if ( t_rescan_icon==NULL )
        t_rescan_icon=new QPixmap ( ( const char ** ) t_rescan_icon_data );

    if ( t_delete_icon==NULL )
        t_delete_icon=new QPixmap ( ( const char ** ) t_delete_icon_data );

    if ( t_config_icon==NULL )
        t_config_icon=new QPixmap ( ( const char ** ) t_config_icon_data );

    if ( t_help_icon==NULL )
        t_help_icon=new QPixmap ( ( const char ** ) t_help_icon_data );

    if ( t_new_icon==NULL )
        t_new_icon=new QPixmap ( ( const char ** ) t_new_icon_data );

    if ( t_find_icon==NULL )
        t_find_icon=new QPixmap ( ( const char ** ) t_find_icon_data );

    if ( t_about_icon==NULL )
        t_about_icon=new QPixmap ( ( const char ** ) t_about_icon_data );

    if ( t_comment_icon==NULL )
        t_comment_icon=new QPixmap ( ( const char ** ) t_comment_icon_data );

    if ( t_add_export_icon==NULL )
        t_add_export_icon=new QPixmap ( ( const char ** ) t_add_export_icon_data );

    if ( t_remove_export_icon==NULL )
        t_remove_export_icon=new QPixmap ( ( const char ** ) t_remove_export_icon_data );

    if ( t_showc_icon==NULL )
        t_showc_icon=new QPixmap ( ( const char ** ) t_showc_icon_data );

    if ( t_deletec_icon==NULL )
        t_deletec_icon=new QPixmap ( ( const char ** ) t_deletec_icon_data );

    if ( m_export_icon==NULL )
        m_export_icon=new QPixmap ( ( const char ** ) m_export_icon_data );

    if ( m_import_icon==NULL )
        m_import_icon=new QPixmap ( ( const char ** ) m_import_icon_data );

    if ( m_borrow_icon==NULL )
        m_borrow_icon=new QPixmap ( ( const char ** ) m_borrow_icon_data );

    if ( t_sborrow_icon==NULL )
        t_sborrow_icon=new QPixmap ( ( const char ** ) t_sborrow_icon_data );

    if ( t_cborrow_icon==NULL )
        t_cborrow_icon=new QPixmap ( ( const char ** ) t_cborrow_icon_data );

    if ( t_colorconfig_icon==NULL )
        t_colorconfig_icon=new QPixmap ( ( const char ** ) t_colorconfig_icon_data );


}

QPixmap * get_m_catalog_icon ( void ) {
    return m_catalog_icon;
}

QPixmap * get_m_unknown_icon ( void ) {
    return m_unknown_icon;
}

QPixmap * get_m_cd_icon ( void ) {
    return m_cd_icon;
}

QPixmap * get_m_dvd_icon ( void ) {
    return m_dvd_icon;
}

QPixmap * get_m_hdd_icon ( void ) {
    return m_hdd_icon;
}

QPixmap * get_m_net_icon ( void ) {
    return m_net_icon;
}

QPixmap * get_m_flash_icon ( void ) {
    return m_flash_icon;
}

QPixmap * get_m_other_icon ( void ) {
    return m_other_icon;
}

QPixmap * get_m_floppy_icon ( void ) {
    return m_floppy_icon;
}

QPixmap * get_v_folderlocked_icon ( void ) {
    return v_folderlocked_icon;
}

QPixmap * get_v_folderclosed_icon ( void ) {
    return v_folderclosed_icon;
}

QPixmap * get_v_folderopen_icon ( void ) {
    return v_folderopen_icon;
}

QPixmap * get_v_file_icon ( void ) {
    return v_file_icon;
}

QPixmap * get_v_back_icon ( void ) {
    return v_back_icon;
}

QPixmap * get_p_icon ( void ) {
    return p_icon;
}

QPixmap * get_t_open_icon ( void ) {
    return t_open_icon;
}

QPixmap * get_t_save_icon ( void ) {
    return t_save_icon;
}

QPixmap * get_t_saveas_icon ( void ) {
    return t_saveas_icon;
}

QPixmap * get_t_close_icon ( void ) {
    return t_close_icon;
}

QPixmap * get_t_add_icon ( void ) {
    return t_add_icon;
}

QPixmap * get_t_rescan_icon ( void ) {
    return t_rescan_icon;
}

QPixmap * get_t_delete_icon ( void ) {
    return t_delete_icon;
}

QPixmap * get_t_config_icon ( void ) {
    return t_config_icon;
}

QPixmap * get_t_help_icon ( void ) {
    return t_help_icon;
}

QPixmap * get_t_new_icon ( void ) {
    return t_new_icon;
}

QPixmap * get_t_find_icon ( void ) {
    return t_find_icon;
}

QPixmap * get_t_about_icon ( void ) {
    return t_about_icon;
}

QPixmap * get_t_comment_icon ( void ) {
    return t_comment_icon;
}

QPixmap * get_t_add_export_icon ( void ) {
    return t_add_export_icon;
}

QPixmap * get_t_remove_export_icon ( void ) {
    return t_remove_export_icon;
}

QPixmap * get_t_showc_icon ( void ) {
    return t_showc_icon;
}

QPixmap * get_t_deletec_icon ( void ) {
    return t_deletec_icon;
}

QPixmap * get_m_export_icon ( void ) {
    return m_export_icon;
}

QPixmap * get_m_import_icon ( void ) {
    return m_import_icon;
}

QPixmap * get_m_borrow_icon ( void ) {
    return m_borrow_icon;
}

QPixmap * get_t_sborrow_icon ( void ) {
    return t_sborrow_icon;
}

QPixmap * get_t_cborrow_icon ( void ) {
    return t_cborrow_icon;
}

QPixmap * get_t_colorconfig_icon ( void ) {
    return t_colorconfig_icon;
}


QPixmap * get_anim ( int i ) {
    switch ( i ) {
    case 0: return a_0;
    case 1: return a_1;
    case 2: return a_2;
    case 3: return a_3;
    case 4: return a_4;
    default: return a_1;
    }
}

QList<QPixmap> get_anim_list() {
	QList<QPixmap> animlist;
	for (int i=0;i<5;i++)
		animlist.append(*get_anim(i));
	return animlist;
}






